/*
 *
 * dbridge_db.c
 * Description:
 * DOCSIS bridge database implementation
 *
 * Copyright (C) 2008 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#define _DBRIDGE_DB_C_

/*! \file dbridge_db.c
    \brief the docsis bridge database
*/

/**************************************************************************/
/*      INCLUDES:                                                         */
/**************************************************************************/
#include "dbridge_db.h"
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include "dbridge_igmp.h"
#include "dbridge_netdev.h"
#include "dbridge_hal_filter.h"
#include "dbridge_mdf_db.h"
#include "dbridge_mcast.h"
#include <linux/ti_hil.h>

/* #UnihanBU6#, Joe Liao, 2008/07/17, Implement "Max CPE Definition", vsif 69, cableVoiceGatewayEmtaIncludedInCmMaxCpe. */
#include "autoconf.h" 

/**************************************************************************/
/*      EXTERNS Declaration:                                              */
/**************************************************************************/
 
/**************************************************************************/
/*      DEFINES:                                                          */
/**************************************************************************/

/**************************************************************************/
/*      LOCAL DECLARATIONS:                                               */
/**************************************************************************/

#define MIN_NUM_CPE 1 

 
/**************************************************************************/
/*      LOCAL VARIABLES:                                                  */
/**************************************************************************/

/*  Structure which keeps track of all the information for the DOCSIS Bridge. */
static DbridgeDataBase_t dbridge_db;
static DbridgeCounters_t  dbridge_counters; 
static unsigned  int devices_table_insert;
static unsigned  int alt_table_insert;
static unsigned  int data_base_init = 0;
static unsigned  int max_cpe = MIN_NUM_CPE;
static unsigned  int lbridge_interface_mask;

/* #UnihanBU6#, Joe Liao, 2008/07/17, Implement "Max CPE Definition", vsif 69, cableVoiceGatewayEmtaIncludedInCmMaxCpe. */
/* #UnihanBU6#, shadow, 2009/04/10, change the default value of  cableVoiceGatewayEmtaIncludedInCmMaxCpe to false, set DEFAULT_MAX_CPE_DEF to 1 */
#if (CONFIG_FOR_NETGEAR)/* #UnihanBU6#, Joe Liao, 2008/07/17 */
#define DEFAULT_MAX_CPE_DEF 1 
static unsigned char max_cpe_def = DEFAULT_MAX_CPE_DEF;
/* #UnihanBU6#,Alisa_Gao, 20090422, TT980:Client can access the CM_IP via MIB browser when SNMPCPEAccessControl(TLV55) is disable.*/
#define DEFAULT_CPE_ACCESS 0 /*default value=>disable*/
static unsigned int cpe_access = DEFAULT_CPE_ACCESS; 
/* #UnihanBU6#,Alisa_Gao, 20090422, Add End*/
#endif  /*end of (CONFIG_FOR_NETGEAR) */  
/* end of #UnihanBU6#, Joe Liao, 2008/07/17, Implement "Max CPE Definition", vsif 69, cableVoiceGatewayEmtaIncludedInCmMaxCpe. */


/**************************************************************************/
/*      INTERFACE FUNCTIONS Implementation:                               */
/**************************************************************************/

/**************************************************************************/
/*      LOCAL FUNCTIONS:                                                  */
/**************************************************************************/


/**************************************************************************/
/*! \fn int DbridgeDb_Init(void)                                     
 **************************************************************************
 *  \brief DOCSIS bridge data base initialization.
 *  \param[in] no input.
 *  \param[out] no output.
 *  \return OK or error status.
 **************************************************************************/
int DbridgeDb_Init(void)
{
    struct net_device *eth_dev =dev_get_by_name (ETH_DEV_NAME);
    struct net_device *usb_dev =dev_get_by_name (USB_DEV_NAME);
	memset ((void *)&dbridge_db, 0, sizeof(DbridgeDataBase_t));
    memset ((void *)&dbridge_counters, 0, sizeof(DbridgeCounters_t));

	devices_table_insert=0;
    alt_table_insert=0;
	data_base_init=1;

	/* This is patch for filtering need to be change when using usb0 */
	if(eth_dev)
		lbridge_interface_mask = ((unsigned long) 1 << ((eth_dev->ifindex)-1));
	if(usb_dev)
		lbridge_interface_mask |= ((unsigned long) 1 << ((usb_dev->ifindex)-1));
	return 0; /* OK*/
}

/**************************************************************************/
/*! \fn int DbridgeDb_ReInit(void)                                     
 **************************************************************************
 *  \brief DOCSIS bridge data base re-initialization.
 *  \param[in] no input.
 *  \param[out] no output.
 *  \return OK or error status.
 **************************************************************************/
int DbridgeDb_ReInit(void)
{
    unsigned int index;
    unsigned int cpe_removed=0;
    unsigned int cpe_type;
	/* remove all cpe from alt table */
	for (index=0;index<alt_table_insert;index++)
	{
     cpe_type = dbridge_db.altTable[index].macLearningFrom;
	 if ((cpe_type ==CPE_STATIC)||(cpe_type ==CPE_DYNAMIC))
	 {
		  HAL_MacDsUnicastDataFilterDelete(dbridge_db.altTable[index].macAddress);
		  memset ((void *)&dbridge_db.altTable[index], 0, sizeof(DbridgeAltEntry_t));
          cpe_removed++;
	 }
	}
     alt_table_insert -=cpe_removed;
    dbridge_counters.StaticCpe = 0;
	dbridge_counters.DynamicCpe = 0;
	max_cpe = MIN_NUM_CPE;

/* #UnihanBU6#, Joe Liao, 2008/07/17, Implement "Max CPE Definition", vsif 69, cableVoiceGatewayEmtaIncludedInCmMaxCpe. */
#if (CONFIG_FOR_NETGEAR)/* #UnihanBU6#, Joe Liao, 2008/07/17 */
      max_cpe_def = DEFAULT_MAX_CPE_DEF;
      cpe_access = DEFAULT_CPE_ACCESS;
#endif  /*end of (CONFIG_FOR_NETGEAR) */  
/* end of #UnihanBU6#, Joe Liao, 2008/07/17, Implement "Max CPE Definition", vsif 69, cableVoiceGatewayEmtaIncludedInCmMaxCpe. */

    /* remove all HW filters */

    /* remove all multicast filters */
    DbridgeDb_DelAllMcastMac();

	return 0; /* OK*/
}

/**************************************************************************/
/*! \fn DDbridgeAltEntry_t* DbridgeDb_AltGetMemberByMac(unsigned char* mac_address)
 **************************************************************************
 *  \brief The function retreives the dbridge ALT table member for the  
 *  \brief corresponding mac address.
 *  \param[in] cpe mac address.
 *  \return Success-pointer to the Dbridge ALT table, NULL-If no entry is found.
 **************************************************************************/
DbridgeAltEntry_t* DbridgeDb_AltGetMemberByMac(unsigned char* mac_address)
{
	int index;
	for (index=0;index<alt_table_insert;index++)
	{
	   if (memcmp ((void *)mac_address, (void *)&dbridge_db.altTable[index].macAddress[0], MAC_ADDRESS_LEN) == 0)
		{
			return &dbridge_db.altTable[index]; 
		}
	}
	return NULL;
}

/**************************************************************************/
/*! \fn DDbridgeAltEntry_t*  DbridgeDb_AltGetMemberByIndex(int index);
 **************************************************************************
 *  \brief The function retreives the dbridge ALT table member for the  
 *  \brief corresponding mac address.
 *  \param[in] ALt table index.
 *  \return Success-pointer to the Dbridge ALT table, NULL-If no entry is found.
 **************************************************************************/
DbridgeAltEntry_t*  DbridgeDb_AltGetMemberByIndex(int index)
{
	if(index >= 0 && index < alt_table_insert)
	{
		return &dbridge_db.altTable[index];
	}
	else
	{
		return NULL;
	}
}

/**************************************************************************/
/*! \fn int DbridgeDb_AltGetInfoByIndex(int index, unsigned char* mac_address, Maclearning_e *mac_learn_from, DbridgeDevice_t *dbridge_Devices)
 **************************************************************************
 *  \brief Get information from ALT table.
 *  \param[in] ALT table index.
 *  \param[out] mac address ,
 *  \param[out] mac address learn from - NOT_CPE, CPE_STATIC or CPE_DYNAMIC,
 *  \param[out] pointer to DbridgeDevice.
 *  \return 0 or error status.
 **************************************************************************/
int DbridgeDb_AltGetInfoByIndex(int index, unsigned char* mac_address, Maclearning_e *mac_learn_from, DbridgeDevice_t *dbridge_Devices)
{
    if(index >= 0 || index <alt_table_insert)
	{
      	*mac_learn_from=dbridge_db.altTable[index].macLearningFrom;
		dbridge_Devices=dbridge_db.altTable[index].dbridgeDevice;
		memcpy ((void *)mac_address,(void *)&dbridge_db.altTable[alt_table_insert].macAddress[0], MAC_ADDRESS_LEN);
		return 0;
	}
	else
	{
		*mac_learn_from=NOT_LEARN;
		dbridge_Devices=NULL;
		return -1;

	}
}
/**************************************************************************/
/*! \fn int DbridgeDb_AltAddMac(unsigned char* mac_address, Maclearning_e mac_learn_from,struct net_device *ptr_netdev, struct net_device*  sourceInterface)
 **************************************************************************
 *  \brief this function Add MAC address to ALT table or update device information for
 *  \brif existing cpe MAC that add from config file .
 *  \param[in] cpe mac address.
 *  \param[in] mac address from - NOT_CPE, CPE_STATIC or CPE_DYNAMIC,
 *  \param[in] pointer to docsis device table.
 *  \param[in] pointer to first interface where the packet received from.
 *  \param[out] no output.
 *  \return OK or error status.
 **************************************************************************/
int DbridgeDb_AltAddMac(unsigned char* mac_address, Maclearning_e mac_learn_from, 
						DbridgeDevice_t *dbridge_Devices,struct net_device*  sourceInterface)
{   
	DbridgeAltEntry_t* ptr_dbridge_alt; 

			
	ptr_dbridge_alt = DbridgeDb_AltGetMemberByMac(mac_address);
	if (ptr_dbridge_alt!=NULL)
	{
		/* MAC address exists update the corresponding device */
		if (mac_learn_from == CPE_DYNAMIC && ptr_dbridge_alt->dbridgeDevice==NULL)
		{
			ptr_dbridge_alt->dbridgeDevice= dbridge_Devices;
			ptr_dbridge_alt->sourceInterface = sourceInterface;
			printk(KERN_INFO "Update source from %.2x-%.2x-%.2x-%.2x-%.2x-%.2x %s\n",
				   (unsigned char)mac_address[0],
				   (unsigned char)mac_address[1],
				   (unsigned char)mac_address[2],
				   (unsigned char)mac_address[3],
				   (unsigned char)mac_address[4],
				   (unsigned char)mac_address[5],
    		       sourceInterface->name);
		}
		return 0; 
	}


	if (alt_table_insert < DBR_MAX_ALT_SIZE )
	{
		if (mac_learn_from==CPE_DYNAMIC || mac_learn_from==CPE_STATIC)
		{
			/* check if we have place for one more cpe */

/* #UnihanBU6#, Joe Liao, 2008/07/17, Implement "Max CPE Definition", vsif 69, cableVoiceGatewayEmtaIncludedInCmMaxCpe. */
#if (CONFIG_FOR_NETGEAR)/* #UnihanBU6#, Joe Liao, 2008/07/17 */			   
/* if mta as a cpe (MaxCpeDef = 0) : current cpe = static + dynamic  */
/* if mta not as cpe    (MaxCpeDef = 1) : current cpe = static + dynamic -1*/
                     int cpe_num = dbridge_counters.StaticCpe + dbridge_counters.DynamicCpe - max_cpe_def;
                     if (cpe_num < 0) cpe_num = 0; /*max_cpe is unsigned type*/
			if(cpe_num < max_cpe)		
#else/*TI original*/
			if ((dbridge_counters.StaticCpe + dbridge_counters.DynamicCpe)<max_cpe)
#endif  /*end of (CONFIG_FOR_NETGEAR) */  
/* end of #UnihanBU6#, Joe Liao, 2008/07/17, Implement "Max CPE Definition", vsif 69, cableVoiceGatewayEmtaIncludedInCmMaxCpe. */
			{
				if (mac_learn_from==CPE_DYNAMIC)
					dbridge_counters.DynamicCpe++;
				else
					dbridge_counters.StaticCpe++;
			} 
                        else
			{
				/* no place for more CPEs */
				return -1;
			}
			/* keep this information for IGMP */
            dbridge_db.altTable[alt_table_insert].cpe_number = dbridge_counters.DynamicCpe + dbridge_counters.StaticCpe;
		}
		memcpy ((void *)&dbridge_db.altTable[alt_table_insert].macAddress[0], (void *)mac_address, MAC_ADDRESS_LEN);
		/* add ptr to docsis device */
		dbridge_db.altTable[alt_table_insert].dbridgeDevice = dbridge_Devices;
		dbridge_db.altTable[alt_table_insert].sourceInterface = sourceInterface;
		dbridge_db.altTable[alt_table_insert].macLearningFrom=mac_learn_from;
	        
		/* open HW filter for unicast MAC address */   
		if(HAL_MacDsUnicastDataFilterAdd(mac_address)==0)
			printk(KERN_INFO "Add source filter for %.2x-%.2x-%.2x-%.2x-%.2x-%.2x %s\n",
				   (unsigned char)mac_address[0],
				   (unsigned char)mac_address[1],
				   (unsigned char)mac_address[2],
				   (unsigned char)mac_address[3],
				   (unsigned char)mac_address[4],
				   (unsigned char)mac_address[5],
			   sourceInterface->name);

		printk(KERN_INFO "Add index %d \n",alt_table_insert);
		alt_table_insert++;
		return 0;
	} 
        else
	{
		/* the table is full */
		return -1;
	}
}

#if CONFIG_FOR_NETGEAR /* #UnihanBU6#, tara, 2008/10/24 */
int DbridgeDb_AltAddIPaddr(unsigned char* mac_address, unsigned char* ip_address)
{
	DbridgeAltEntry_t* ptr_dbridge_alt; 

	ptr_dbridge_alt = DbridgeDb_AltGetMemberByMac(mac_address);
	if (ptr_dbridge_alt!=NULL)
	{
        if( memcmp(ptr_dbridge_alt->ipAddress, ip_address, IP_ADDRESS_LEN) )
        {
            printk(KERN_INFO "update ip address: %d.%d.%d.%d\n",ip_address[0],ip_address[1],ip_address[2],ip_address[3]);
            memcpy (ptr_dbridge_alt->ipAddress, ip_address, IP_ADDRESS_LEN);
        }
		/* MAC address exists update the corresponding device */
		return 0; 
	}
	return -1;
}
#endif /* CONFIG_FOR_NETGEAR */
/**************************************************************************/
/*! \fn int DbridgeDb_AltAddMacByDevName(unsigned char* mac_address, unsigned char* name)
 **************************************************************************
 *  \brief this function Add MAC address to ALT table for cm interface (wan0 or lan0)
 *  \param[in] mac address.
 *  \param[in] device name.
 *  \param[out] no output.
 *  \return OK or error status.
 **************************************************************************/
 int DbridgeDb_AltAddMacByDevName(unsigned char* mac_address, unsigned char* name)
 {
	 int index;
	 DbridgeDevice_t*   ptr_dbridge_dev;
	 int name_len = strlen(name);


	 for (index=0 ; index<devices_table_insert; index++)
     {

		 ptr_dbridge_dev = &dbridge_db.DbridgeDevices[index];
		 if((strlen(ptr_dbridge_dev->ptr_interface->name)== name_len) &&
			(memcmp ((void *)name, (void *)&ptr_dbridge_dev->ptr_interface->name, name_len)==0))
		 {
			 /* find match between inpute name and dbridge device table */
			 /* add the the mac information to ALT table */ 
			 if(DbridgeDb_AltAddMac(mac_address,DBRIDGE_INIT, ptr_dbridge_dev,NULL)==0) 
			    return 0;
			 else 
			    return -1;
		 }
	 }
	 return -1;
 }

/***************************************************************************/
/*! \fn int DbridgeDb_AltGetNumMember(void)
 **************************************************************************
 *  \param[in] None.
 *  \param[out] None.
 *  \return Number of member in ALT table.
 **************************************************************************/
int DbridgeDb_AltGetNumMember(void)
{
	return alt_table_insert;
}

/***************************************************************************/
/*! \fn int DbridgeDB_SetMaxCpe(unsigned  int mac_cpe_config)                                     
 **************************************************************************
 *  \brief set maximum  supported CPEs.
 *  \param[in] Max allow CPEs .
 *  \param[out] no output.
 *  \return OK or error status.
 **************************************************************************/
int DbridgeDB_SetMaxCpe(unsigned  int mac_cpe)
{
	max_cpe = mac_cpe;
	return 0;	/* OK*/
}

/***************************************************************************/
/*! \fn int DbridgeDB_GetMaxCpe()                                     
 **************************************************************************
 *  \brief Get maximum  supported CPEs.
 *  \param[in] None .
 *  \param[out] None.
 *  \return Max CPEs.
 **************************************************************************/
int DbridgeDB_GetMaxCpe(void)
{
	return max_cpe;	
}


/* #UnihanBU6#, Joe Liao, 2008/07/17, Implement "Max CPE Definition", vsif 69, cableVoiceGatewayEmtaIncludedInCmMaxCpe. */
#if (CONFIG_FOR_NETGEAR)/* #UnihanBU6#, Joe Liao, 2008/07/17 */
/***************************************************************************/
/*! \fn int DbridgeDB_SetMaxCpeDef(uconst unsigned  char value)                                     
 **************************************************************************
 *  \brief Set Max CPE Definition : Determines whether the MTA is counted as a CPE device according to eDOCSIS spec
            value --> 0: MTA as a CPE ,1: MTA not as a CPE
 *  \param[in]  Max CPE Definition 
 *  \param[out] no output.
 *  \return OK or error status.
 **************************************************************************/
int DbridgeDB_SetMaxCpeDef(const unsigned  char value)
{
	max_cpe_def= value;
	return 0;	/* OK*/
}

/***************************************************************************/
/*! \fn int DbridgeDB_GetMaxCpeDef()                                     
 **************************************************************************
 *  \brief Get Max CPE Definition : Determines whether the MTA is counted as a CPE device according to eDOCSIS spec
            value --> 0: MTA as a CPE ,1: MTA not as a CPE
 *  \param[in] None .
 *  \param[out] None.
 * return Max CPE Definition .
 **************************************************************************/
unsigned char  DbridgeDB_GetMaxCpeDef(void)
{
	return max_cpe_def;	
}

/* #UnihanBU6#,Alisa_Gao, 20090422, TT980:Client can access the CM_IP via MIB browser when SNMPCPEAccessControl(TLV55) is disable.*/
int DbridgeDB_SetCpeAccess(const unsigned int value)
{
	cpe_access = value;
	return 0;	/* OK*/
}
unsigned int  DbridgeDB_GetCpeAccess(void)
{
    return cpe_access;	
}
/* #UnihanBU6#,Alisa_Gao, 20090422, Add,End*/
#endif  /*end of (CONFIG_FOR_NETGEAR) */  
/* end of #UnihanBU6#, Joe Liao, 2008/07/17, Implement "Max CPE Definition", vsif 69, cableVoiceGatewayEmtaIncludedInCmMaxCpe. */


/***************************************************************************/
/*! \fn int DbridgeDb_DevAdd(struct net_device* ptr_net_dev,int (*push)(struct sk_buff* skb, 
			unsigned int fc_flags))
 ***************************************************************************
 *  \brief Add member to device table.
 *  \param[in] pointer to net device.
 *  \param[in] The PUSH Function to send the packet to Interface.
 *  \param[in] docsis interface type  - CPE eSAFE CABLE WAN IP or LAN IP.
 *  \return 0=OK or error status.
 **************************************************************************/
int DbridgeDb_DevAdd(struct net_device* ptr_net_dev,int (*push)(struct sk_buff* skb, 
														unsigned int fc_flags), int net_dev_type)
{
 if (devices_table_insert < DBR_MAX_INTERFACES)
 {
	 dbridge_db.DbridgeDevices[devices_table_insert].push = push;
	 dbridge_db.DbridgeDevices[devices_table_insert].ptr_interface = ptr_net_dev;
     dbridge_db.DbridgeDevices[devices_table_insert].net_dev_type = net_dev_type; 
	 devices_table_insert++;
	 return 0;
 }
 else
 {   
	 printk(KERN_WARNING "Device table is full\n");
	 
	 return -1;
 }

}

/*************************************************************************************/
/*! \fn DbridgeDevice_t* DbridgeDb_DevGetByInterface (struct net_device* ptr_interface)
 *************************************************************************************
 *  \brief The function retreives the dbridge device table member for the corresponding 
 *  \brief network interface.
 *  \param[in] pointer to net net device.
 *  \return Success-pointer to the Dbridge device table, NULL-If no entry is found.
 *************************************************************************************/
DbridgeDevice_t* DbridgeDb_DevGetByInterface (struct net_device* ptr_interface)
{
    int index;

    /* Search the DOCSIS Network device table. */
    for (index = 0; index <devices_table_insert; index++)
    {
        /* Compare the network interface for a match. */
        if (ptr_interface == dbridge_db.DbridgeDevices[index].ptr_interface)
            return &dbridge_db.DbridgeDevices[index];
    }

    /* No match found. */
    return NULL;
}

/*************************************************************************************/
/*! \fn DbridgeDevice_t* DbridgeDb_DevGetByIndex (struct net_device* ptr_interface)
 *************************************************************************************
 *  \brief The function retreives the dbridge device table member for the corresponding 
 *  \brief index.
 *  \param[in] index.
 *  \return Success-pointer to the Dbridge device table, NULL-If no entry is found.
 *************************************************************************************/
DbridgeDevice_t* DbridgeDb_DevGetByIndex (int index)
{
	if(index >= 0 && index < devices_table_insert)
	{
		return &dbridge_db.DbridgeDevices[index];
	}
	else
	{
		return NULL;
	}
}

/*************************************************************************************/
/*! \fn DDbridgeDevice_t* DbridgeDb_DevGetByType(int net_dev_type)
 *************************************************************************************
 *  \brief The function retreives the dbridge device table member for the corresponding 
 *  \brief type.
 *  \param[in] net dev type - CMCI, ESAFE CABLE WAN_IP or LAN_IP.
 *  \return Success-pointer to the Dbridge device table, NULL-If no entry is found.
 *************************************************************************************/
DbridgeDevice_t* DbridgeDb_DevGetByType(int net_dev_type)
{
    int index;

    /* Search the DOCSIS Network device table. */
    for (index = 0; index <devices_table_insert; index++)
    {
        /* Compare the network interface for a match. */
        if (net_dev_type == dbridge_db.DbridgeDevices[index].net_dev_type)
            return &dbridge_db.DbridgeDevices[index];
    }
    /* No match found. */
    return NULL;

}

/*************************************************************************************/
/*! \fn int DbridgeDb_GetNumOfDevice (void)
 *************************************************************************************
 *  \brief The function retreives the number of device connect to docsis bridge
 *  \param[in] none.
 *  \return number of device connect to docsis bridge.
 *
 *************************************************************************************/
int DbridgeDb_GetNumOfDevice (void)
{
	return devices_table_insert;
}

/*************************************************************************************/
/*! \fn void DbridgeDb_SetMode (int mode)
 *************************************************************************************
 *  \brief set docsis bridge mode
 *  \param[in] docsis bridge mode - off, registered pre_registration or naco_off 
 *  \return none.
 *************************************************************************************/
void DbridgeDb_SetMode(int mode)
{
    unsigned int mdfMode;

	if((dbridge_db.mode==DBR_REGISTERED_MODE)&&((mode == DBR_PRE_REGISTRATION_MODE) || (mode == DBR_NACO_OFF_MODE)))
	{
		DbridgeDb_ReInit();
        DbridgeMdfDb_ReInit();
	}

    else if (mode == DBR_PRE_REGISTRATION_MODE)
    {
        DbridgeMdfDb_ReInit();
    }

	/* Set IGMP mode */ 
        /* need to move the ti_register_selective_packet_handler to deferent place */
	if (mode == DBR_REGISTERED_MODE)
	{
		extern int ti_register_selective_packet_handler (char *br_name, int (*packet_handler)(struct sk_buff *skb), int priority);
		ti_register_selective_packet_handler (LBRIGE_CONNECTION_DEV_NAME, ti_selective_forward, 1);

        /* if MDF is disabled - start the igmp module */
        DbridgeMdfDb_GetMdfMode(&mdfMode);
        if (mdfMode == DBR_MDF_DISABLE_MODE)
        {
            ti_igmp_start();
        }

		if (DBR_REGISTERED_MODE != dbridge_db.mode)
		{
			struct net_device * dev = dev_get_by_name (CABLE_DEV_NAME);
			ti_hil_pp_event (TI_BRIDGE_PORT_FORWARD, (void *)dev);
		}
	}
	else
    {
		ti_igmp_stop();
		if (DBR_REGISTERED_MODE == dbridge_db.mode)
		{
			struct net_device * dev = dev_get_by_name (CABLE_DEV_NAME);
			ti_hil_pp_event (TI_BRIDGE_PORT_DISABLED, (void *)dev);
		}
    }

	dbridge_db.mode=mode;
	
}

/* #UnihanBU6#,Jamie,2008/07/29, export it for button module called */
EXPORT_SYMBOL(DbridgeDb_SetMode);

/*************************************************************************************/
/*! \fn iint DbridgeDb_GetMode (void)
 *************************************************************************************
 *  \brief get docsis bridge mode
 *  \param[in] none
 *  \return docsis bridge mode - off, registered pre_registration or naco_off.
 *************************************************************************************/
int DbridgeDb_GetMode(void)
{
    return dbridge_db.mode;
}
/* #UnihanBU6#,Jamie,2008/07/29, export it for button module called */
EXPORT_SYMBOL(DbridgeDb_GetMode);

/*************************************************************************************/
/*! \fn iDbridgeCounter_t* Dbridge_GetCounters(void)
 *************************************************************************************
 *  \brief get docsis bridge counters
 *  \param[in] none
 *  \return pointer to the Dbridge counter structure.
 *************************************************************************************/
DbridgeCounters_t* Dbridge_GetCounters(void)
{
	return &dbridge_counters;
}

/***************************************************************************/
/*! \fn int DbridgeDB_GetlbridgeInterfaceMask(void)                                     
 **************************************************************************
 *  \brief Get lbridge Interface Mask.
 *  \param[in] None .
 *  \param[out] None.
 *  \return lbridge Interface Masks.
 **************************************************************************/
int DbridgeDB_GetlbridgeInterfaceMask(void)
{
	return lbridge_interface_mask;	
}
