/*
 *
 * hal_phy_basic_regs.h
 * Description:
 * defines the MAC interrupt registers and their content
 *
 * Copyright (C) 2008 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#ifndef _HAL_BASIC_PHY_REGS_H_
#define _HAL_BASIC_PHY_REGS_H_

/**********************************/               
/*     PHY Interrupt Registers    */               
/**********************************/

#define PHY_RX1_RX2_INTERRUPT_ENABLE                0x0C2
#define PHY_RX1_RX2_INTERRUPT_STATUS                0x0C3 /* IMPORTANT - not used in the application */
/* Bit Mapping */
#define PHY_RX2_QAM_LOST                            13
#define PHY_RX2_QAM_LOCKED                          12
#define PHY_RX2_MPEG_LOCKED                         11
#define PHY_RX2_MPEG_LOST                           10
#define PHY_RX2_FEC_LOCKED                          9
#define PHY_RX2_FEC_LOST                            8
#define PHY_RX2_FEC_PMONITOR                        7
#define PHY_RX1_QAM_LOST                            6
#define PHY_RX1_QAM_LOCKED                          5
#define PHY_RX1_MPEG_LOCKED                         4
#define PHY_RX1_MPEG_LOST                           3
#define PHY_RX1_FEC_LOCKED                          2
#define PHY_RX1_FEC_LOST                            1
#define PHY_RX1_FEC_PMONITOR                        0


#define PHY_RX3_RX4_INTERRUPT_ENABLE                0x0C4
#define PHY_RX3_RX4_INTERRUPT_STATUS                0x0C5 /* IMPORTANT - not used in the application */
/* Bit Mapping */
#define PHY_RX4_QAM_LOST                            13
#define PHY_RX4_QAM_LOCKED                          12
#define PHY_RX4_MPEG_LOCKED                         11
#define PHY_RX4_MPEG_LOST                           10
#define PHY_RX4_FEC_LOCKED                          9
#define PHY_RX4_FEC_LOST                            8
#define PHY_RX4_FEC_PMONITOR                        7
#define PHY_RX3_QAM_LOST                            6
#define PHY_RX3_QAM_LOCKED                          5
#define PHY_RX3_MPEG_LOCKED                         4
#define PHY_RX3_MPEG_LOST                           3
#define PHY_RX3_FEC_LOCKED                          2
#define PHY_RX3_FEC_LOST                            1
#define PHY_RX3_FEC_PMONITOR                        0


/**********************************/               
/*      QAM register mapping      */               
/**********************************/
#define PHY_QAM_MODE_0_REGISTER 0x01 /*(R/W, 0x0001)*/
/* Bit     Name                    Default Description  */
/* 2:0     QAM TYPE    			   001     000 - 16QAM                                                          */
/*                                         001 - 64QAM                                                          */
/*                                         010 - 256QAM                                                         */
/*                                         011 - 1024QAM                                                        */
/*                                         100 - 32QAM                                                          */
/*                                         101 - 128QAM                                                         */
/*                                         110 - QPSK                                                           */
/*                                         Others - Reserved                                                    */
/* 3       COMMON RESTART MODE     0       0 - Common_restart is a pulse derived from the Common_ready posedge  */
/*                                         1 - Common_restart is a level inversion of Common_ready              */
/* 5:4     Reserved                0                                                                            */
/* 6       FE INVERSE SPECTRUM     0       0 - Disable                                                          */
/*                                         1 - Enable                                                           */
/* 9:7     Reserved                0                                                                            */
/* 10      QAM ANNEX TYPE          0       0 - B,1 - A                                                          */
/* 11      OC DIFF MODE            0       0 - Normal                                                           */
/*                                         1 - Inverted                                                         */
#define PHY_QAM_TYPE                0

#define PHY_QAM_TYPE_16QAM          0
#define PHY_QAM_TYPE_64QAM          1
#define PHY_QAM_TYPE_256QAM         2
#define PHY_QAM_TYPE_1024QAM        3
#define PHY_QAM_TYPE_32QAM          4
#define PHY_QAM_TYPE_128QAM         5
#define PHY_QAM_TYPE_QPSK           6

#define PHY_QAM_TYPE_MASK           0x07

#define PHY_COMMON_RESTART MODE     3  
#define PHY_FE_INVERSE_SPECTRUM		6 
#define PHY_FE_INVERSE_SPECTRUM_BIT (1<<PHY_FE_INVERSE_SPECTRUM)
#define PHY_QAM_ANNEX_TYPE          10 
#define PHY_QAM_ANNEX_ANA           (1<<PHY_QAM_ANNEX_TYPE) 
#define PHY_OC_DIFF_MODE            11 

/*******************************************************/               
/* FEC Status registers (duplicated for each receiver) */
/*******************************************************/               
#define PHY_RX_FEC_STATUS(ch)     0xA3 + (0x100*((ch)-1))

/* Bit Mapping */
#define PHY_RX_FEC_STATUS_INT_PREMASK               2 
#define PHY_RX_FEC_STATUS_TR_SYNC                   1 /* This is the bit that indicates FEC lock.  */
#define PHY_RX_FEC_STATUS_MPEG_SYNC                 0


#define PHY_RX_CHANNEL_GRUN			0x100
#define PHY_RX_CH_REG_OFFSET(chnl,reg_num) (((chnl)*PHY_RX_CHANNEL_GRUN)+(reg_num))

#endif /* _HAL_BASIC_PHY_REGS_H_ */

