/*
 * CGI helper functions
 *
 * Copyright 2003, Broadcom Corporation
 * All Rights Reserved.                
 *                                     
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Broadcom Corporation;   
 * the contents of this file may not be disclosed to third parties, copied
 * or duplicated in any form, in whole or in part, without the prior      
 * written permission of Broadcom Corporation.                            
 *
 * $Id: RgSecurity.c,v 1.1.1.1 2010/06/30 06:49:33 eating Exp $
 */
 
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <syslog.h>
#include <arpa/inet.h>

#include "httpd.h"

#include "sys_types.h"
#include "nvram_db.h"
#include "production_db.h"
#include "docsis_nvram_mngr.h"
/* #UnihanBU6#, Jimmy 2008/06/12, add for vendor nvram */
#include "system_shmdb_util.h"
#include "vendornvm_common.h"

//Allen 20080527
int security_set(int eid, webs_t wp, char *passwd);
int security_get(int eid, webs_t wp, int argc, char_t **argv);


#define MAX_REP_STR         128
#define STR_WIDTH           40
#define NGTR_PASS           "NetgearPassword"
#define NGTR_PASS_REENTER   "NetgearPasswordReEnter"
#define RESET_YES           "RestoreFactoryYes"
#define RESET_NO            "RestoreFactoryNo"
#define YES_STR             "0x01"

#define WEBCM_BUFFER_SIZE	(4*1024)

static char *errormessage[256];

int b64_decode_table[256] = {
    -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,  /* 00-0F */
    -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,  /* 10-1F */
    -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,62,-1,-1,-1,63,  /* 20-2F */
    52,53,54,55,56,57,58,59,60,61,-1,-1,-1,-1,-1,-1,  /* 30-3F */
    -1, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,  /* 40-4F */
    15,16,17,18,19,20,21,22,23,24,25,-1,-1,-1,-1,-1,  /* 50-5F */
    -1,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,  /* 60-6F */
    41,42,43,44,45,46,47,48,49,50,51,-1,-1,-1,-1,-1,  /* 70-7F */
    -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,  /* 80-8F */
    -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,  /* 90-9F */
    -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,  /* A0-AF */
    -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,  /* B0-BF */
    -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,  /* C0-CF */
    -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,  /* D0-DF */
    -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,  /* E0-EF */
    -1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1   /* F0-FF */
};

unsigned short B64Decode(char* authorization, char *user, char *password, int length)
{   
    char space[length+1];
    int size = sizeof(space);
    
    const char* cp;
    int space_idx, phase;
    int d, prev_d=0;
    unsigned char c;

    space_idx = 0;
    phase = 0;
    for ( cp = authorization; *cp != '\0'; ++cp )
	{
        d = b64_decode_table[(int)*cp];
        if ( d != -1 )
        {
            switch ( phase )
            {
            case 0:
                ++phase;
                break;
            case 1:
                c = ( ( prev_d << 2 ) | ( ( d & 0x30 ) >> 4 ) );
                if ( space_idx < size )
                    space[space_idx++] = c;
                ++phase;
                break;
            case 2:
                c = ( ( ( prev_d & 0xf ) << 4 ) | ( ( d & 0x3c ) >> 2 ) );
                if ( space_idx < size )
                    space[space_idx++] = c;
                ++phase;
                break;
            case 3:
                c = ( ( ( prev_d & 0x03 ) << 6 ) | d );
                if ( space_idx < size )
                    space[space_idx++] = c;
                phase = 0;
                break;
            }
            prev_d = d;
        }
    }
    
    // space_idx is where the auth string should be terminated
    space[space_idx] = '\0';
    // separate the index into username and password fill it the returned buffers
    char * delimiter = strstr((char*)space,":");
    if(delimiter)
    {
       memcpy(user,space,delimiter-(char*)space);
       memcpy(password,delimiter+1,strlen(space)-strlen(user)-1);
       return 1;
    }
    return 0;
}

int
security_set(int eid, webs_t wp, char *passwd)
{
    Uint8 AccessLevel=0;

    /* Initial all DB connections*/
    ProductionDb_RetrieveAccess();
    NvramDb_RetrieveAccess();
/* #UnihanBU6#, Jimmy 2008/06/12, add for vendor nvram */
/* #UnihanBU6#, Eating Chen, 2009/12/10, System_ShmDb_RetrieveAccess() function 
 * shall be called once in each process/task 
 */
#if 0
    System_ShmDb_RetrieveAccess();
#endif
    VendorNvm_Common_RetrieveAccess();

    char pass_nv_ori[32];
    if( AccessLevel )
    {
        System_ShmDb_GetHttpMsoPassword(pass_nv_ori);
    }   
    else
    {
        System_ShmDb_GetHttpAdminPassword(pass_nv_ori);
    }
    
    if( AccessLevel )
    {
        char new_password[SYS_DFLT_HTTP_MSO_PASS_SIZE];    
        /*#UnihanBU6# Water Huang, 2009.2.10
         * Modify for Comcast security requirement.
         * EMTA-17: Password Encryption
         */      
        #ifdef CONFIG_FOR_NETGEAR_CVD31XT
        MD5((const unsigned char *)passwd,strlen(passwd),(unsigned char *)new_password);
        #else
        strncpy(new_password,passwd,SYS_DFLT_HTTP_MSO_PASS_SIZE);
        #endif        
        System_ShmDb_SetHttpMsoPassword(new_password, strlen(new_password), 1);
    }
    else
    {
         char new_password[SYS_DFLT_HTTP_ADMIN_PASS_SIZE];    
        /*#UnihanBU6# Water Huang, 2009.2.10
         * Modify for Comcast security requirement.
         * EMTA-17: Password Encryption
         */      
       #ifdef CONFIG_FOR_NETGEAR_CVD31XT
        MD5((const unsigned char *)passwd,strlen(passwd),(unsigned char *)new_password);
        #else
        strncpy(new_password,passwd,SYS_DFLT_HTTP_ADMIN_PASS_SIZE);
        #endif      
        System_ShmDb_SetHttpAdminPassword(new_password, strlen(new_password), 1);
    }
    VendorNvm_Common_SaveToNvram();                  
    sprintf(errormessage, "New Admin Password Applied");
  
	return 0;
}

int
security_get(int eid, webs_t wp, int argc, char_t **argv)
{
    char *name;
	
	if (ejArgs(argc, argv, "%s", &name) < 1) 
	{
		websError(wp, 400, "Insufficient args\n");
		return -1;
	}

    if(strcmp(name, "errormessage") == 0) /* list all wireless mac filte */
    {	
        if (strlen(errormessage) != 0 )
        {
            websWrite(wp, "%s", errormessage);
	        sprintf(errormessage, "");
        }
	    else
	    {
            websWrite(wp, "");
	    }
    }
	return 0;
}

int 
auth_get(char *userid, char *passwd, char *realm)
{
    strncpy (realm, "NETGEAR", AUTH_MAX);
/* #UnihanBU6#, Jimmy 2008/06/12, add for vendor nvram */
    System_ShmDb_GetHttpAdminUserName(userid);
    System_ShmDb_GetHttpAdminPassword(passwd);
    return 0;
}

/* #UnihanBU6#,Rainy, 20080618, implement to get MSO user name and password.  */
int 
auth_get_admin(char *userid, char *passwd, char *realm)
{ 
	strncpy (realm, "NETGEAR", AUTH_MAX);
	System_ShmDb_GetHttpMsoUserName(userid);
	System_ShmDb_GetHttpMsoPassword(passwd);
	return 0;
}

