/*
 * CGI helper functions
 *
 * Copyright 2003, Broadcom Corporation
 * All Rights Reserved.                
 *                                     
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Broadcom Corporation;   
 * the contents of this file may not be disclosed to third parties, copied
 * or duplicated in any form, in whole or in part, without the prior      
 * written permission of Broadcom Corporation.
 *
 * $Id: battery_info.c,v 1.1.1.1 2010/06/30 06:49:33 eating Exp $
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "httpd.h"
#include "bbu_common_utils.h"
#include "sys_dflt.h"
#include "autoconf.h"

/* #UnihanBU6#, jwlin, 2009/07/16 add checking condition when querying battery status */
#include "bbu_mib_if.h"
#include "system_shmdb_util.h"

#define BATTERY_STATUS_SELECTED  "Selected"
#define BATTERY_STATUS_UNSELECTED "Not Selected"

int BATTERYINFO_Handle(int eid, webs_t wp, int argc, char_t **argv);
int BATTERYINFO_get_powersource(webs_t wp,char_t *name);
int BATTERYINFO_get_batterynum(webs_t wp,char_t *name);
int BATTERYINFO_get_inputvoltage(webs_t wp,char_t *name);
int BATTERYINFO_get_temperature(webs_t wp,char_t *name);
int BATTERYINFO_get_remaintime(webs_t wp,char_t *name);
int BATTERYINFO_get_upsstatus(webs_t wp,char_t *name);
int BATTERYINFO_get_upscapacity(webs_t wp,char_t *name);
int BATTERYINFO_get_upsvoltage(webs_t wp,char_t *name);
int BATTERYINFO_get_upsremaintime(webs_t wp,char_t *name);
int BATTERYINFO_get_upsselected(webs_t wp,char_t *name);
int BATTERYINFO_get_ifdelaycm(webs_t wp,char_t *name);
int BATTERYINFO_get_ifdelayeth(webs_t wp,char_t *name);
int BATTERYINFO_get_ifdelayusb(webs_t wp,char_t *name);
extern  int get_code_index(char *name, char *prex);

struct cgi_handler BATTERYINFO_handlers[]=
{
    {"power_source",BATTERYINFO_get_powersource},
    {"battery_num",BATTERYINFO_get_batterynum},
    {"voltage_in", BATTERYINFO_get_inputvoltage},
    {"temperature",BATTERYINFO_get_temperature},
    {"remaining_time",BATTERYINFO_get_remaintime},
    {"ups_status0",BATTERYINFO_get_upsstatus},
    {"ups_status1",BATTERYINFO_get_upsstatus},
    {"ups_capacity0",BATTERYINFO_get_upscapacity},
    {"ups_capacity1",BATTERYINFO_get_upscapacity},
    {"ups_voltage0", BATTERYINFO_get_upsvoltage},
    {"ups_voltage1",BATTERYINFO_get_upsvoltage},
    {"ups_remaintime0",BATTERYINFO_get_upsremaintime},
    {"ups_remaintime1",BATTERYINFO_get_upsremaintime},
    {"ups_selected0",BATTERYINFO_get_upsselected},
    {"ups_selected1", BATTERYINFO_get_upsselected},
    
    {"if_delay_cm",BATTERYINFO_get_ifdelaycm},
    {"if_delay_eth",BATTERYINFO_get_ifdelayeth},
    {"if_delay_usb", BATTERYINFO_get_ifdelayusb},
    {NULL,NULL}
};

char battery_status[][16]=
{
    "Not present",
    "unknow",
    "Normal",
    "Low",
    "Depleted"
};

int BATTERY_Handle(int eid, webs_t wp, int argc, char_t **argv)
{
    char_t*name;
    struct cgi_handler *handler;
    
    if(BBU_Common_Utils_Init() == NOK)
    {
        websWrite(wp,"%s","unknow");
        return -1;
    }
    
    if(ejArgs(argc,argv,"%s",&name)<1)
    {
        websError(wp,400,"Insufficient args\n");
        return -1;
    }
    
    for(handler=&BATTERYINFO_handlers[0];handler->pattern;handler++)
    {
    	if(strncmp(handler->pattern,name,strlen(handler->pattern))==0)
    		handler->output(wp,name);
    }
    
    return 0;
}


int BATTERYINFO_get_powersource(webs_t wp,char_t *name)
{
    Int32 source = 0;
    if(Bbu_GetUpsOutputSource(&source) == NOK)
    {
        websWrite(wp,"%s","unknow");
    }
    else
    {
        websWrite(wp,"%s",(source == BBU_MIB_OUTPUT_SOURCE_NONE? "No Power":
                                     (source == BBU_MIB_OUTPUT_SOURCE_NORMAL?" AC Power":
                                     (source == BBU_MIB_OUTPUT_SOURCE_BATTERY? "Battery":"N/A"))));
    }
    
    return 0;
}

int BATTERYINFO_get_batterynum(webs_t wp,char_t *name)
{
    Uint32 num_of_bat = 0;
    if(BBU_Common_Utils_GetNumOfBatties(&num_of_bat) == NOK)
    {
        websWrite(wp,"%s","unknow");
    }
    else 
    {
        websWrite(wp,"%d",num_of_bat);
    }
    
    return 0;
}

int BATTERYINFO_get_inputvoltage(webs_t wp,char_t *name)
{
	return 0;
}

int BATTERYINFO_get_temperature(webs_t wp,char_t *name)
{
    Int32 temper = 0;
    
    if( BBU_Common_Utils_GetBatteryContainerTemperature(&temper) == NOK)
    {
        websWrite(wp,"%s","unknow");
    }
    else
    {
        websWrite(wp,"%d deg. C",temper);
    }
    return 0;
}
int BATTERYINFO_get_remaintime(webs_t wp,char_t *name)
{
	return 0;
}


int BATTERYINFO_get_upsstatus(webs_t wp,char_t *name)
{
    int index=get_code_index(name,"ups_status");
    Int32 bat_status = BBU_MIB_BATTERY_STATUS_UNKNOWN;
    /* #UnihanBU6#, jwlin, 2009/07/16 add checking condition when querying battery status */
    Uint32 percentRemaining;
    unsigned int percentThreshold;
    
    if(BBU_Common_Utils_GetBatteryStatus(index,&bat_status) == OK)
    {	
        /* #UnihanBU6#, jwlin, 2009/07/16 add checking condition when querying battery status */
        if((bat_status==BBU_MIB_BATTERY_STATUS_BATTERY_NORMAL)||(bat_status==BBU_MIB_BATTERY_STATUS_BATTERY_LOW))
        {
            /* #UnihanBU6#, jwlin, 2009/07/17 fix incorrect logical checking */
            if((BBU_Common_Utils_GetBatteryPercentRemaining(index, &percentRemaining)==OK)&&(System_ShmDb_GetChargerLowChargeThreshold(&percentThreshold)==OK))
            {
                if(percentRemaining > percentThreshold){
                    /* battery status is actually normal */
                    bat_status = BBU_MIB_BATTERY_STATUS_BATTERY_NORMAL;
                }
                else{
                    /* battery status is actually low */
                    bat_status = BBU_MIB_BATTERY_STATUS_BATTERY_LOW;
                }
            }
        }
        websWrite(wp,"%s",battery_status[bat_status]);
        return 0;
    }
    websWrite(wp,"unknow");	
    return 0;
}

int BATTERYINFO_get_upscapacity(webs_t wp,char_t *name)
{	
    int index=get_code_index(name,"ups_capacity");
    long bat_capacity = 0;
    /* #UnihanBU6#, Alisa_Gao, 2009/04/08 , TT1362:Battery display incorrectly,Start*/
    #if (CONFIG_FOR_NETGEAR_BATTERY_2S2P)
    Int32 bat_status = BBU_MIB_BATTERY_STATUS_UNKNOWN;
    if(BBU_Common_Utils_GetBatteryStatus(index,&bat_status) == OK)
    {
        if(bat_status != 0)/*0 is unknow*/
    #else
        Uint8  is_selected = 0 ; 
    if(BBU_Common_Utils_IsBatterySelected(index,&is_selected) == OK)
    {	
        if(is_selected == 1)
    #endif 
     /* #UnihanBU6#, Alisa_Gao, 2009/04/08 , TT1362:Battery display incorrectly,End*/
       {
            if(BBU_Common_Utils_GetBatteryCapacity(index,&bat_capacity) == OK)
            {
                websWrite(wp,"%ld mAH",bat_capacity);
                return 0;
            }
        }	
        else
        {
             websWrite(wp,"%s",BATTERY_STATUS_UNSELECTED);
             return 0;
        }
    }	
    websWrite(wp,"%s","unknow");
    return 0;
}

int BATTERYINFO_get_upsvoltage(webs_t wp,char_t *name)
{
    int index=get_code_index(name,"ups_voltage");
    Uint32 volt = 0;	
    /* #UnihanBU6#, Alisa_Gao, 2009/04/08 , TT1362:Battery display incorrectly,Start*/
    #if (CONFIG_FOR_NETGEAR_BATTERY_2S2P)
    Int32 bat_status = BBU_MIB_BATTERY_STATUS_UNKNOWN;
    if(BBU_Common_Utils_GetBatteryStatus(index,&bat_status) == OK)
    {
        if(bat_status != 0)/*0 is unknow*/
    #else
        Uint8  is_selected = 0;
    if(BBU_Common_Utils_IsBatterySelected(index,&is_selected) == OK)
    {	
        if(is_selected == 1)
    #endif 
    /* #UnihanBU6#, Alisa_Gao, 2009/04/08 , TT1362:Battery display incorrectly,End*/
        {
            if(BBU_Common_Utils_GetBatteryMeasuredVoltage(index,&volt) == OK)
            {
                websWrite(wp,"%d mV",volt);
                return 0;
            }
         }
        else 
        {
            websWrite(wp,"%s",BATTERY_STATUS_UNSELECTED);
            return 0;
        }
    }
    websWrite(wp,"%s","unknow");
    return 0;
}

int BATTERYINFO_get_upsremaintime(webs_t wp,char_t *name)
{	
     int index=get_code_index(name,"ups_remaintime");
     Uint32 remain_minutes = 0;
      /* #UnihanBU6#, Alisa_Gao, 2009/04/08 , TT1362:Battery display incorrectly,Start*/
     #if (CONFIG_FOR_NETGEAR_BATTERY_2S2P)
     Int32 bat_status = BBU_MIB_BATTERY_STATUS_UNKNOWN;
     if(BBU_Common_Utils_GetBatteryStatus(index,&bat_status) == OK)
     {
         if(bat_status != 0)/*0 is unknow*/
     #else
         Uint8  is_selected = 0 ;
         if(BBU_Common_Utils_IsBatterySelected(index,&is_selected) == OK)
         {
             if(is_selected == 1)
     #endif 
       /* #UnihanBU6#, Alisa_Gao, 2009/04/08 , TT1362:Battery display incorrectly,End*/
             {
                 if(BBU_Common_Utils_GetBatteryMinutesRemaining(index,&remain_minutes) == OK)
                 {
                     websWrite(wp,"%d minutes",remain_minutes);
                     return 0;
                 }
             }
            else
            {
                 websWrite(wp,"%s",BATTERY_STATUS_UNSELECTED);
                 return 0;
            }
     }
     websWrite(wp,"%s","unknow");
     return 0;
}

int BATTERYINFO_get_upsselected(webs_t wp,char_t *name)
{
    int index=get_code_index(name,"ups_selected");
    Uint8  is_selected = 0;
    
    if(BBU_Common_Utils_IsBatterySelected(index,&is_selected) == NOK)
    {
        websWrite(wp,"%s","unknow");
    }
    else
    {
        websWrite(wp,"%s",is_selected == 1 ? BATTERY_STATUS_SELECTED:BATTERY_STATUS_UNSELECTED);
    }
    
    return 0;
}

int BATTERYINFO_get_ifdelaycm(webs_t wp,char_t *name)
{
    Int32  delay_time = 0;
    if(BBU_Common_Utils_GetPowerMgmtCmDelay(&delay_time) == NOK)
    {
        websWrite(wp,"%s","unknow");
    }
    else
    {
        websWrite(wp,"%d",delay_time);
    }
    
    return 0;
}

int BATTERYINFO_get_ifdelayeth(webs_t wp,char_t *name)
{
    Int32  delay_time = 0;	
    if(BBU_Common_Utils_GetPowerMgmtEthDelay(&delay_time) == NOK)
    {
        websWrite(wp,"%s","unknow");
    }
    else
    {
        websWrite(wp,"%d",delay_time);
    }
    
    return 0;
}

int BATTERYINFO_get_ifdelayusb(webs_t wp,char_t *name)
{	
    Int32  delay_time = 0;	
    if(BBU_Common_Utils_GetPowerMgmtUsbDelay(&delay_time) == NOK)
    {
        websWrite(wp,"%s","unknow");
    }
    else
    {
        websWrite(wp,"%d",delay_time);
    }
    
    return 0;
}
