/*
 * CGI helper functions
 *
 * Copyright 2003, Broadcom Corporation
 * All Rights Reserved.                
 *                                     
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Broadcom Corporation;   
 * the contents of this file may not be disclosed to third parties, copied
 * or duplicated in any form, in whole or in part, without the prior      
 * written permission of Broadcom Corporation.                            
 *
 * $Id: cm.c,v 1.1.1.1 2010/06/30 06:49:33 eating Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "httpd.h"
#include "ipprov_db.h"
#include "vendor_common_utils.h"
#include "sys_dflt.h"

#include <arpa/inet.h>

/* #UnihanBU6#, Chenwin_Qiu, 20090407, Add. */
#include "pacm_prov_db.h"
#define MAX_REP_STR 256
#define DHCP_SECONDS (60*60*24)


int CM_Handle(int eid, webs_t wp, int argc, char_t **argv);
int CM_get_IpAddress(webs_t wp,char_t *name);
int CM_get_SubnetMask(webs_t wp,char_t *name);
int CM_get_Gateway(webs_t wp,char_t *name);
int CM_get_TftpServer(webs_t wp,char_t *name);
int CM_get_TimerServer(webs_t wp,char_t *name);
int CM_get_TimeOffset(webs_t wp,char_t *name);
int CM_get_ConfigFileName(webs_t wp,char_t *name);
int CM_get_DhcpLeaseTime(webs_t wp,char_t *name);
int CM_get_DhcpRebindTime(webs_t wp,char_t *name);
int CM_get_DhcpRenewTime(webs_t wp,char_t *name);
int CM_get_ConfigfileContent(webs_t wp,char_t *name);
/* #UnihanBU6#, Chenwin_Qiu, 20090407, Implement display CM option 1 and option 2. */
int CM_get_DhcpSuboption1(webs_t wp,char_t *name);
int CM_get_DhcpSuboption2(webs_t wp,char_t *name);


struct cgi_handler CM_handlers[]=
{
	{"ipaddr",CM_get_IpAddress},
	{"subnetmask",CM_get_SubnetMask},
	{"gateway",CM_get_Gateway},
	{"tftp_server",CM_get_TftpServer},
	{"time_server",CM_get_TimerServer},
	{"time_offset",CM_get_TimeOffset},
	{"bootfile",CM_get_ConfigFileName},
	{"lease",CM_get_DhcpLeaseTime},
	{"rebind",CM_get_DhcpRebindTime},
	{"renew",CM_get_DhcpRenewTime},
	{"configfile_content",CM_get_ConfigfileContent},
	/* #UnihanBU6#, Chenwin_Qiu, 20090407, Implement display CM option 1 and option 2. */
	{"subopt1",CM_get_DhcpSuboption1},
	{"subopt2",CM_get_DhcpSuboption2},
	{NULL,NULL}
};

void  get_CM_DaysAndSeconds(Uint32 *time,char *DaysSeconds)
{
	Uint32 days=(*time)/(DHCP_SECONDS);
	Uint32 seconds=(*time)%(DHCP_SECONDS);
	sprintf(DaysSeconds,"%u Days %u Seconds",days,seconds);
}

int CM_Handle(int eid, webs_t wp, int argc, char_t **argv)
{
	char_t*name;
	struct cgi_handler *handler;

	if(IpProv_DbRetrieveAccess() == NOK )
	{
		websWrite(wp, "unknown");
		return -1;
	}
	
	if(ejArgs(argc,argv,"%s",&name)<1)
	{
		websError(wp,400,"Insufficient args\n");
		return -1;
	}

	for(handler=&CM_handlers[0];handler->pattern;handler++)
	{
		if(strncmp(handler->pattern,name,strlen(handler->pattern))==0)
			handler->output(wp,name);
	}
	return 0;
}


int CM_get_IpAddress(webs_t wp,char_t *name)
{
	Uint8 ipAddress[MAX_INET_ADDRLEN];
	Uint8 Iptype;
	Uint32 ipLen;
	char addrStr[INET6_ADDRSTRLEN];
	memset(addrStr,0,INET6_ADDRSTRLEN);
	memset(ipAddress,0,MAX_INET_ADDRLEN);
	if (IpProv_GetCmIpType(INTERFACE_ID_CAB,&Iptype) == OK)
	{
		if (IpProv_GetCmIpAddress(INTERFACE_ID_CAB, ipAddress, &ipLen) == OK)
		{
			if (Iptype == INET_ADDR_TYPE_IPV4)
			{
				inet_ntop( AF_INET, ipAddress, addrStr, INET_ADDRSTRLEN);
			}
			else if (Iptype == INET_ADDR_TYPE_IPV6)
			{
				inet_ntop( AF_INET6, ipAddress, addrStr, INET6_ADDRSTRLEN);
			}
			else
			{
				sprintf(addrStr,"----");
			}
			websWrite(wp,"%s",addrStr);
		}
	}
	else
	{
		websWrite(wp,"----");
	}
	return 0;
}

int CM_get_SubnetMask(webs_t wp,char_t *name)
{
	InetAddr_t CmIpAddress;
	char subnetmask[INET6_ADDRSTRLEN];
	memset(subnetmask,0,INET6_ADDRSTRLEN);
	memset(&CmIpAddress,0,sizeof(InetAddr_t));
	if (IpProv_GetCmRouter(INTERFACE_ID_CAB, &CmIpAddress) == OK)
	{
		if (CmIpAddress.family == AF_INET)
		{
			inet_ntop( AF_INET, (Uint8*)GetInetAddrSubnetP( &CmIpAddress ), subnetmask, INET_ADDRSTRLEN);
			websWrite(wp,"%s",subnetmask);
		}
		else if (CmIpAddress.family == AF_INET6)
		{
			websWrite(wp,"%u",GetInetAddrPrefix( &CmIpAddress ));
		}
	}
	else
	{
		websWrite(wp,"----");
	}	
	return 0;
}

int CM_get_Gateway(webs_t wp,char_t *name)
{
	InetAddr_t gateway;
	char addrStr[INET6_ADDRSTRLEN];
	memset(addrStr,0,INET6_ADDRSTRLEN);
	memset(&gateway,0,sizeof(InetAddr_t));
	if (IpProv_GetCmRouter(INTERFACE_ID_CAB, &gateway) == OK)
	{
		if (gateway.family == AF_INET)
		{
			inet_ntop( AF_INET, gateway.addr, addrStr, INET_ADDRSTRLEN);
		}
		else if (gateway.family == AF_INET6)
		{
			inet_ntop( AF_INET6, gateway.addr, addrStr, INET6_ADDRSTRLEN);
		}	
		websWrite(wp,"%s",addrStr);
	}
	else
	{
		websWrite(wp,"----");
	}	
	return 0;
}

int CM_get_TftpServer(webs_t wp,char_t *name)
{
	Uint8 ipAddress[MAX_INET_ADDRLEN];
	Uint8 Iptype=1;
	Uint32 ipLen;
	char addrStr[INET6_ADDRSTRLEN];
	memset(addrStr,0,INET6_ADDRSTRLEN);
	memset(ipAddress,0,MAX_INET_ADDRLEN);

	if (IpProv_GetTftpIpType(INTERFACE_ID_CAB,&Iptype) == OK)
	{
		if (IpProv_GetTftpIpAddress(INTERFACE_ID_CAB, ipAddress, &ipLen) == OK)
		{
			if (Iptype == INET_ADDR_TYPE_IPV4)
			{
				inet_ntop( AF_INET, ipAddress, addrStr, INET_ADDRSTRLEN);
			}
			else
			{
				inet_ntop( AF_INET6, ipAddress, addrStr, INET6_ADDRSTRLEN);
			}
			websWrite(wp,"%s",addrStr);
		}
	}
	else
	{
		websWrite(wp,"----");
	}
return 0;
}

int CM_get_TimerServer(webs_t wp,char_t *name)
{
	Uint8 ipAddress[MAX_INET_ADDRLEN];
	Uint8 Iptype=1;
	Uint32 ipLen;
	char addrStr[INET6_ADDRSTRLEN];
	memset(addrStr,0,INET6_ADDRSTRLEN);
	memset(ipAddress,0,MAX_INET_ADDRLEN);

	if (IpProv_GetToDIpType(INTERFACE_ID_CAB,&Iptype) == OK)
	{
		if (IpProv_GetToDIpAddress(INTERFACE_ID_CAB,ipAddress,&ipLen) ==OK)
		{
			if (Iptype == INET_ADDR_TYPE_IPV4)
			{
				inet_ntop( AF_INET, ipAddress, addrStr, INET_ADDRSTRLEN);
			}
			else
			{
				inet_ntop( AF_INET6, ipAddress, addrStr, INET6_ADDRSTRLEN);
			}
			websWrite(wp,"%s",addrStr);
		}
	}
	else
	{
		websWrite(wp,"----");
	}
	return 0;
}
int CM_get_TimeOffset(webs_t wp,char_t *name)
{
	 Int32 TimeOffset = 0;	
	 if(Common_Utils_GetTimeOffset(&TimeOffset) == OK)
	 {
	 	websWrite(wp,"%d Seconds",TimeOffset);
	 }
	else
	{
		websWrite(wp,"%s","unknow");
	}
	return 0;
}
int CM_get_ConfigFileName(webs_t wp,char_t *name)
{
        /* #UnihanBU6#, Chenwin_Qiu, 20090317, Fix display ConfigFileName format*/
	 Char CfgFileName[IPDB_TFTP_FILE_NAME_LEN + 1];	   
	 char *partment;
	 char *ptr_tmp;	
	 memset(CfgFileName,0,IPDB_TFTP_FILE_NAME_LEN + 1);
	
	 if (IpProv_GetTftpConfigFileName(INTERFACE_ID_CAB, CfgFileName) == OK)
	 {
         	partment=CfgFileName;
	 	/*get the last '\' pointer postion and return the remanent content */  
	 	if((ptr_tmp=strrchr(partment,'\\'))!=NULL)
	 	{
			partment=ptr_tmp+1;
	  	}
	  	/*get the last '/' pointer postion and return the remanent content */    
	  	if((ptr_tmp=strrchr(partment,'/'))!=NULL)
	  	{
			partment=ptr_tmp+1;
	  	}
	  	websWrite(wp,"%s",partment);
	}
	else
	{
		websWrite(wp,"unknow");
	}
	return 0;
}
int CM_get_ConfigfileContent(webs_t wp,char_t *name)
{
        /* #UnihanBU6#, Alisa_Gao, 20080710, implement CGI to get MTA and CM config file content */
	char configFileName[IPDB_TFTP_FILE_NAME_LEN];
	char outputPath[128];
	char *p;
	FILE *out;
	char inputPath[128];
	memset(inputPath,0,sizeof(inputPath));
	memset(configFileName,0,IPDB_TFTP_FILE_NAME_LEN);
	memset(outputPath,0,sizeof(outputPath));
	if(IpProv_GetTftpConfigFileName(INTERFACE_ID_CAB,configFileName) == STATUS_OK)
	{  
	 /*after CM get the config name,we can get the config content,or return*/
		if(!strcmp(configFileName,""))
		{
			websWrite(wp,"%s","unknow");
			return 0;
		}
		/*delete the path from the config file name*/
	      while(1)
	   	{
			p = strstr(configFileName,"/");
			if (p == NULL)
				break;
			strcpy(configFileName,p+1);
	   	}
	    sprintf(inputPath,"/var/tmp/%s",configFileName);
	    sprintf(outputPath,SYS_DFLT_CM_CFGCONTENT_FILE_PATH);
	   /*change the config file to readable file,such as:/var/tmp/alisa_cm.BIN==>/var/tmp/cm_cfg.txt*/
  	    if(Common_Utils_GetConfigContent(inputPath,outputPath) == STATUS_OK)
  	    {
		char line[256];
		memset(line,0,sizeof(line));
		if((out=fopen(outputPath,"r"))!= NULL)
		{
			/*output the file content by line*/
		 	while(!feof(out)&&fgets(line,sizeof(line),out))
		 	{
		 		websWrite(wp,"%s",line);
				websWrite(wp,"<br>");
			}
			fclose(out);
			return 0;
		}
		else 
		{
		 	fprintf(stderr,"%s","Can't open the file,Error");
		}
	   }
	}
      	websWrite(wp,"%s","unknow");
	return 0;
}
int CM_get_DhcpLeaseTime(webs_t wp,char_t *name)
{
    Uint32 CMDhcpLeaseRemainTime = 0;
    double totaldays;
    int days;    
    int hours;    
    int minutes;    
    int seconds;    
    int remainder;    
    char time[MAX_REP_STR];	    
    memset(time,0,MAX_REP_STR);
    
    if(Common_Utils_GetCMDhcpLeaseRemainTime(&CMDhcpLeaseRemainTime)==STATUS_OK)
    {
        /* #UnihanBU6#, Chenwin_Qiu, 20090407, Fix display CMDhcpLeaseRemainTime format */
        totaldays = CMDhcpLeaseRemainTime/(24.0*60.0*60.0);
        days = CMDhcpLeaseRemainTime/(24*60*60);
        remainder = CMDhcpLeaseRemainTime % (24*60*60);
        hours = remainder/(60*60);
        remainder %= (60*60);
        minutes = remainder/(60);
        remainder %= (60);
        seconds = remainder;
        sprintf(time,"%1d days, %02dh, %02dm, %02ds",days,hours,minutes,seconds);
        websWrite(wp,"Days:%3.1f<br>&nbspSeconds: %u<br>&nbsp%s",totaldays,CMDhcpLeaseRemainTime,time);
    }
    else
    {
        websWrite(wp,"%s","unknow");
    }
    return 0;
}
int CM_get_DhcpRebindTime(webs_t wp,char_t *name)
{
    Uint32 CMDhcpRebindRemainTime = 0;
    double totaldays;
    int days;    
    int hours;    
    int minutes;    
    int seconds;    
    int remainder;    
    char time[MAX_REP_STR];	    
    memset(time,0,MAX_REP_STR);
    
    if(Common_Utils_GetCMDhcpRebindRemainTime(&CMDhcpRebindRemainTime) == STATUS_OK)
    {
        /* #UnihanBU6#, Chenwin_Qiu, 20090407, Fix display CMDhcpRebindRemainTime format */
        totaldays = CMDhcpRebindRemainTime/(24.0*60.0*60.0);
        days = CMDhcpRebindRemainTime/(24*60*60);
        remainder = CMDhcpRebindRemainTime % (24*60*60);
        hours = remainder/(60*60);
        remainder %= (60*60);
        minutes = remainder/(60);
        remainder %= (60);
        seconds = remainder;
        sprintf(time,"%1d days, %02dh, %02dm, %02ds",days,hours,minutes,seconds);
        websWrite(wp,"Days:%3.1f<br>&nbspSeconds: %u<br>&nbsp%s",totaldays,CMDhcpRebindRemainTime,time);
    }
    else
    {
        websWrite(wp,"%s","unknow");
    }
    return 0;
}
int CM_get_DhcpRenewTime(webs_t wp,char_t *name)
{
    Uint32 CMDhcpRenewRemainTime = 0;
    double totaldays;
    int days;    
    int hours;    
    int minutes;    
    int seconds;    
    int remainder;    
    char time[MAX_REP_STR];	    
    memset(time,0,MAX_REP_STR);
    
    if(Common_Utils_GetCMDhcpRenewRemainTime(&CMDhcpRenewRemainTime) == STATUS_OK)
    {
        /* #UnihanBU6#, Chenwin_Qiu, 20090407, Fix display CMDhcpRenewRemainTime format */
        totaldays = CMDhcpRenewRemainTime/(24.0*60.0*60.0);
        days = CMDhcpRenewRemainTime/(24*60*60);
        remainder = CMDhcpRenewRemainTime % (24*60*60);
        hours = remainder/(60*60);
        remainder %= (60*60);
        minutes = remainder/(60);
        remainder %= (60);
        seconds = remainder;
        sprintf(time,"%1d days, %02dh, %02dm, %02ds",days,hours,minutes,seconds);
        websWrite(wp,"Days:%3.1f<br>&nbspSeconds: %u<br>&nbsp%s",totaldays,CMDhcpRenewRemainTime,time);
    }
    else
    {
        websWrite(wp,"%s","unknow");
    }
    return 0;
}

/* #UnihanBU6#, Chenwin_Qiu, 20090407, Implement display CM DHCP option 1 and option 2, Start */
int CM_get_DhcpSuboption1(webs_t wp,char_t *name)
{
    InetAddr_t addr;
    char addrStr[INET6_ADDRSTRLEN];
    memset(&addr,0,sizeof(InetAddr_t));
    memset(addrStr,0,INET6_ADDRSTRLEN);
    
    if(PACM_Prov_GetServerIpAddress(&addr,PACM_MTA_DEV_SRV_DHCP1)==STATUS_OK)
    {	
        if(addr.family == AF_INET)
        {
            inet_ntop(AF_INET, addr.addr, addrStr, INET_ADDRSTRLEN);
        }
        else if(addr.family == AF_INET6)
        {
            inet_ntop(AF_INET, addr.addr, addrStr, INET6_ADDRSTRLEN);
        }
        
        websWrite(wp,"%s",addrStr);
    }
    
    else
    {
        websWrite(wp,"%s","------");
    }
    return 0;
}

int CM_get_DhcpSuboption2(webs_t wp,char_t *name)
{
    InetAddr_t addr;
    char addrStr[INET6_ADDRSTRLEN];
    memset(&addr,0,sizeof(InetAddr_t));
    memset(addrStr,0,INET6_ADDRSTRLEN);
    
    if(PACM_Prov_GetServerIpAddress(&addr,PACM_MTA_DEV_SRV_DHCP2)==STATUS_OK)
    {	
        if(addr.family == AF_INET)
        {
            inet_ntop(AF_INET, addr.addr, addrStr, INET_ADDRSTRLEN);
        }
        else if(addr.family == AF_INET6)
        {
            inet_ntop(AF_INET, addr.addr, addrStr, INET6_ADDRSTRLEN);
        }
		
        websWrite(wp,"%s",addrStr);
    }
    
    else
    {
        websWrite(wp,"%s","------");
    }
    return 0;
}
/* #UnihanBU6#, Chenwin_Qiu, 20090407, Implement display CM DHCP option 1 and option 2, End */

