/*
 * CGI helper functions
 *
 * Copyright 2003, Broadcom Corporation
 * All Rights Reserved.                
 *                                     
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Broadcom Corporation;   
 * the contents of this file may not be disclosed to third parties, copied
 * or duplicated in any form, in whole or in part, without the prior      
 * written permission of Broadcom Corporation.                            
 *
 * $Id: evenlog.c,v 1.1.1.1 2010/06/30 06:49:33 eating Exp $
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <syslog.h>
#include <arpa/inet.h>

#include "httpd.h"

#include "sys_types.h"
#include "event_mgr_main.h"
#include "event_mgr_if.h"
#include "pacm_event_mgr_if.h"
#include "pacm_event_mgr_main.h"
#include "pacm_config_utilities.h"

#define MAX_REP_STR		6*1024
#define MAC_ADDR_STRING_LEN 18
#define MDB_MAX_LOG_LEN    1024
#define MIDNIGHT_JAN_01_2000 (60L*60L*24L*7L*52L*30L+60L*60L*24L*37L)


int EVENTLOG_Handle(int eid, webs_t wp, int argc, char_t **argv);
int EVENTLOG_show_CmLog(webs_t wp,char_t *name);
int EVENTLOG_show_MtaLog(webs_t wp,char_t *name);

struct cgi_handler EVENTLOG_handlers[]=
{
    {"cm_log",EVENTLOG_show_CmLog},
    {"basic",EVENTLOG_show_MtaLog},
    {"advanced",EVENTLOG_show_MtaLog},
    {NULL,NULL}
};

/* #UnihanBU6#, Alisa_Gao, 2008/10/30 ,TT1056,Added for EventLevel,Start*/
char EventLevel[][32]=
{
    "None(0)",
    "Emergency (1)",
    "Alert (2)",
    "Critical (3)",
    "Error (4)",
    "Warning (5)",
    "Notice (6)",
    "Information (7)",
    "Debug (8)"
};
/* #UnihanBU6#, Alisa_Gao, 2008/10/30 ,End*/

int EVENTLOG_Handle(int eid, webs_t wp, int argc, char_t **argv)
{
    char_t*name;
    struct cgi_handler *handler;

    if(ejArgs(argc,argv,"%s",&name)<1)
    {
        websError(wp,400,"Insufficient args\n");
        return -1;
    }

    for(handler=&EVENTLOG_handlers[0];handler->pattern;handler++)
    {
        if(strncmp(handler->pattern,name,strlen(handler->pattern))==0)
            handler->output(wp,name);
    }
    return 0;
}

int EVENTLOG_show_MtaLog(webs_t wp,char_t *name)
{
    MtaEventLogEntry_t	newEntry;
    MtaEventLogEntry_t *ptr;
    Uint8 logEntry[MDB_MAX_LOG_LEN];
    Int logLen = 0;
    Uint logIndex = 0;
    Uint16 paramLen = MAC_ADDR_STRING_LEN;
    char macString[MAC_ADDR_STRING_LEN];
    char  TextStr[EVM_MAX_EVENT_TEXT];
    Uint32 uint32val;
    char *TimeStr;
    while(1)
    {	
        memset(&newEntry, 0, sizeof(MtaEventLogEntry_t));
        memset(logEntry,0,sizeof(logEntry));
        if(PACM_EVENT_MGR_GET_LOG_ENTRY_NEXT(logIndex,logEntry,&logLen) == NOK)
        {
            return -1;
        }
        memcpy(&newEntry,logEntry, sizeof(MtaEventLogEntry_t));
        logIndex=*(Uint*)logEntry;

        if(!logIndex)
        {
            printf("\n--- Reached last log ---\n\n");
            return -1;
        }
        ptr=&newEntry;
        uint32val = (Uint32)ptr->pktcDevEvLogTime;
        TimeStr = ctime (&uint32val);
        if(uint32val <= MIDNIGHT_JAN_01_2000)
        {      
            sprintf(TimeStr,"Time Not Established");
        }
        memset(TextStr, 0, sizeof(TextStr));
        if (strlen(ptr->pktcDevEvLogText) <=EVM_MAX_EVENT_TEXT)
        {      
            strncpy (TextStr, ptr->pktcDevEvLogText,EVM_MAX_EVENT_TEXT);
        }
        else
        {
            strncpy (TextStr, ptr->pktcDevEvLogText, EVM_MAX_EVENT_TEXT-3);
            strcat (TextStr, "...");
        }
        websWrite(wp,"<tr>");
        websWrite(wp,"<td>&nbsp;%s&nbsp;</td>",strtok(TimeStr, "\n"));

        /* #UnihanBU6#, Alisa_Gao, 2008/10/30 ,TT1056,Added for EventLevel,Start*/
        if(ptr->pktcDevEvLogLevel == PRI_LAST)
        {
            websWrite(wp,"<td>&nbsp;%s&nbsp;</td>",EventLevel[0]);
        }
        else
        {
            websWrite(wp,"<td>&nbsp;%s&nbsp;</td>",EventLevel[ptr->pktcDevEvLogLevel]);
        }
        /* #UnihanBU6#, Alisa_Gao, 2008/10/30 ,End*/

        websWrite(wp,"<td>&nbsp;%u&nbsp;</td>",ptr->pktcDevEvLogId);
        if(!strcmp(name,"basic"))
        {			
            websWrite(wp,"<td>&nbsp;%s&nbsp;</td>","----------------");
            websWrite(wp,"<td>&nbsp;%s&nbsp;</td>","----------------");
        }
        else if(!strcmp(name,"advanced"))
        {	
        /*get MTA MAC Address*/
            if(PACM_GetConfigurationValue(TI_PACM_MODULE_FILE_INIT, PACM_CONFIG_HW_ADDR, &paramLen, macString) == NOK)
            {
                strcpy(macString,"----------");
            }
            websWrite(wp,"<td>&nbsp;%s&nbsp;</td>",macString);
            websWrite(wp,"<td>&nbsp;%s&nbsp;</td>",ptr->pktcDevEvLogEndpointName);
        }
        websWrite(wp,"<td>&nbsp;%s&nbsp;</td>",TextStr);
        websWrite(wp,"</tr>"); 
    }
    return 0;
}


int EVENTLOG_show_CmLog(webs_t wp,char_t *name)
{
    Int resBufLen = 0;
    CmEventLogEntry_t *eventEntry;
    CmEventLogEntry_t EventLogEntry;
    Uint8 EventLogEntryResBuf[MDB_MAX_LOG_LEN];
    Uint index = 0; 
    char  TextStr[EVM_MAX_EVENT_TEXT];
    Uint32 uint32val ;
    char *TimeStr;
    Uint tmpindex;
    while(1)
    {
        memset(&EventLogEntry, 0, sizeof(CmEventLogEntry_t));
        memset(EventLogEntryResBuf, 0, sizeof(EventLogEntryResBuf));

        if( EVMIF_GetLogEntryNext(EVENT_MGR_CTRL_SOCK, index, EventLogEntryResBuf, &resBufLen) == NOK)
        {
            return -1;
        }
        memcpy(&EventLogEntry, EventLogEntryResBuf, sizeof(CmEventLogEntry_t));
        memcpy(&tmpindex,EventLogEntryResBuf,sizeof(Uint));
        index = tmpindex;
        eventEntry = &EventLogEntry;
        uint32val = (Uint32)eventEntry->docsDevEvLastTime.tv_sec;
        TimeStr = ctime (&uint32val); 
        if(uint32val <= MIDNIGHT_JAN_01_2000)
        {      
            sprintf(TimeStr,"Time Not Established");
        }

        memset (TextStr, 0, sizeof(TextStr));
        if (strlen(eventEntry->docsDevEvText) <= EVM_MAX_EVENT_TEXT)
        {      
            strncpy (TextStr, eventEntry->docsDevEvText, EVM_MAX_EVENT_TEXT);
        }
        else
        {
            strncpy (TextStr, eventEntry->docsDevEvText, EVM_MAX_EVENT_TEXT-3);
            strcat (TextStr, "...");
        }
        websWrite(wp,"<tr>");
        websWrite(wp,"<td>&nbsp;%s&nbsp;</td>",strtok(TimeStr, "\n"));

        if(eventEntry->docsDevEvLevel == PRI_LAST)
        {
            websWrite(wp,"<td>&nbsp;%s&nbsp;</td>",EventLevel[0]);
        }
        else
        {
            /* #UnihanBU6#, Alisa_Gao, 2008/10/30 ,simplify the old code*/
            websWrite(wp,"<td>&nbsp;%s&nbsp;</td>",EventLevel[eventEntry->docsDevEvLevel]);
        }
        /* #UnihanBU6#, Allen, 20081121, TT1089 DUT doesn't returned an appropriate event log to the CM event log when FW upgrade is inhibited */
        websWrite(wp,"<td>&nbsp;%u&nbsp;</td>",eventEntry->docsDevEvId);
        websWrite(wp,"<td>&nbsp;%s&nbsp;</td>",TextStr);
        websWrite(wp,"</tr>");	
    }
    return 0;
}
