    /* milli_httpd - pretty small HTTP server
** A combination of
** micro_httpd - really small HTTP server
** and
** mini_httpd - small HTTP server
**
** Copyright  1999,2000 by Jef Poskanzer <jef@acme.com>.
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions
** are met:
** 1. Redistributions of source code must retain the above copyright
**    notice, this list of conditions and the following disclaimer.
** 2. Redistributions in binary form must reproduce the above copyright
**    notice, this list of conditions and the following disclaimer in the
**    documentation and/or other materials provided with the distribution.
**
** THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
** ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
** ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
** FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
** OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
** HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
** LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
** OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
** SUCH DAMAGE.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <time.h>
#include <unistd.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <arpa/inet.h>
#include <getopt.h>
#include <stdarg.h>
#include <syslog.h>
#include "httpd.h"
#include "autoconf.h"
#include <sys/time.h>

#ifdef HAVE_OPENSSL
#include <openssl/ssl.h>
#endif

#ifdef HAVE_MATRIXSSL
# include <matrixSsl.h>
# include <matrixssl_xface.h>
#endif


#include <error.h>
#include <sys/signal.h>
#include "production_db.h" /* #UnihanBU6#,Rainy, 20080618, to get Lan IP address and mask.  */
#include "sys_dflt.h"         /* #UnihanBU6#,Alisa_Gao, 20080812, to get interface name */
#include "vendor_common_utils.h"
#ifdef CONFIG_FOR_NETGEAR_CVD31XT
#include "event_mgr_main.h"
#include "event_mgr_if.h"
#endif 
/*#UnihanBU6#,Ailsa_Cai,20090812,Add CLI & mib control for http session timeout ,set default 600 sec*/
#include "system_shmdb_util.h"

#define SERVER_NAME "httpd"
//#define SERVER_PORT 80
#define PROTOCOL "HTTP/1.0"
#define RFC1123FMT "%a, %d %b %Y %H:%M:%S GMT"
#define TIMEOUT	15

//#define DEBUG 0
//#define STIMEOUT 60*10   /* login timeout 1*10 minutes*/
/*#UnihanBU6#,Ailsa_Cai,20090812,Add CLI & mib control for http session timeout ,set default 600 sec*/
static unsigned int stimeout = 600;


static t_s *s_head = NULL; 
static char c_addr[20];
static char c_mac[20];

/* A multi-family sockaddr. */
typedef union
{
  struct sockaddr sa;
  struct sockaddr_in sa_in;
} usockaddr;

/* Globals. */
#ifdef HAVE_OPENSSL
static SSL *ssl;
#endif

#ifdef FILTER_DEBUG
FILE *debout;
#endif

#if defined(HAVE_OPENSSL) || defined(HAVE_MATRIXSSL)

#define DEFAULT_HTTPS_PORT 443
#define CERT_FILE "/etc/cert.pem"
#define KEY_FILE "/etc/key.pem"
#endif

#ifdef HAVE_MATRIXSSL
extern ssl_t *ssl;
extern sslKeys_t *keys;
#endif

#define DEFAULT_HTTP_PORT 80
#ifdef HAVE_HTTPS
extern int do_ssl;
#endif
int server_port;

char pid_file[80];
char *server_dir = NULL;
char *server_ip = NULL;
/* #UnihanBU6#,Alisa_Gao, 20080811, add interface to httpd server.Start */
char *server_interface=NULL ;
/* #UnihanBU6#,Alisa_Gao, 20080811. End*/

static webs_t conn_fp;		

/* #UnihanBU6#,Rainy, 20080618, implement httpd session and access level.  */
char auth_realm[AUTH_MAX];
char *advanced_page[]=
{
	"adv_product.asp",
	"adv_dhcp.asp",
	"adv_mta.asp",
	"adv_qos.asp",
	"adv_config.asp",
	"adv_evelog.asp",	
	"basic_management.asp",
	"techsupport.asp",
	"adv_entrance.asp",	/* #UnihanBU6#,Alisa_Gao, 20080717,Add the PODT page.*/
	"adv_linetest.asp",		/* #UnihanBU6#,Alisa_Gao, 20081113,Add the line test page.*/
	"adv_lastcall.asp",
	"help_adv_lastcall.html",
	"help_adv_product.html",	
	"help_adv_evelog.html",
	"help_adv_mta.html",
	"help_adv_qos.html",
	"help_adv_dhcp.html",
	"help_adv_config.html",
	"help_basic_management.html",
	NULL
};

/* #UnihanBU6#,Ailsa_Cai, 20030310,  access level for hidden page. Start */
char *hidden_page[]=
{
        "hidden_BackupSettings.asp",
        "hidden_ChangeScanPlan.asp",
        "hidden_CustomizeScanPlan.asp",
        "hidden_FirmwareUpgrade.asp",
        "hidden_GoDs.asp",
        "hidden_GoUs.asp",
        "hidden_PowerOnTimeKeeper.asp",
        "hidden_SetPassword.asp",
        "hidden_SetTelnet.asp",
        "hidden_SetVsif.asp",
        "hidindex.html",
        "hidden_contents.html",
        "hidden_SetTop.html",
        "hidden_Logout.html",
        "help_hidden_BackupSettings.html",
        "help_hidden_ChangeScanPlan.html",
        "help_hidden_CustomizeScanPlan.html",
        "help_hidden_FirmwareUpgrade.html",
        "help_hidden_GoDs.html",
        "help_hidden_GoUs.html",
        "help_hidden_PowerOnTimeKeeper.html",
        "help_hidden_SetPassword.html",
        "help_hidden_SetTelnet.html",
	 "help_hidden_SetVsif.html",
        "ds_frequency.cgi",
        "custom_scanplan.cgi",
        "change_scanplanlkf.cgi",
        "backup_setting.cgi",
        "goUs.cgi",
        "setPassword.cgi",
        "setTelnet.cgi",
        "firmwareUpgrade.cgi",
        "setVsif.cgi",
        "hidden_wait.asp",
	 NULL
};
/* #UnihanBU6#,Ailsa_Cai, 20030310,  End  */

int post;
int auth_fail = 0;
int httpd_level;
char http_client_ip[20];

typedef struct _HttpUserInfo
{
	char userpasswd[AUTH_MAX];
	char username[AUTH_MAX];
	unsigned int ip_stack;
	int accesslevel;
}HttpUserInfo;

int userlevel[4]={AuthLevelAdmin,AuthLevelUser,AuthLevelAdmin,AuthLevelAdmin};

static int get_slevel(t_s *h, char *addr);
static void set_slevel(t_s *h, char *addr, int lvl);

extern char *get_mac_from_ip (char *ip);

/* Forwards. */
static int initialize_listen_socket (usockaddr * usaP);
static int auth_check (char *dirname, char *authorization);
static void send_error (int status, char *title, char *extra_header,
			char *text);
static void send_headers (int status, char *title, char *extra_header,
			  char *mime_type);
static int b64_decode (const char *str, unsigned char *space, int size);
static int match (const char *pattern, const char *string);
static int match_one (const char *pattern, int patternlen,
		      const char *string);
static void handle_request (void);
static int set_interface(t_s *h,char*addr,unsigned int interface);

/* #UnihanBU6#,Ailsa_Cai, 20090310,add level for hidden pages  */
static int auth_hidden_check (char *dirname, char *authorization);
/* #UnihanBU6#,Ailsa_Cai, 20090310 , End */

static int
initialize_listen_socket (usockaddr * usaP)
{
    int listen_fd;
    int i=1,result=-1;
    /* #UnihanBU6#,Alisa_Gao, 20080812,Add,bind socket to special interface, Start  */
    listen_fd=socket(AF_INET,SOCK_STREAM,0);
    if (listen_fd < 0)
    {
        perror ("socket");
        return -1;
    }

    (void) fcntl (listen_fd, F_SETFD, 1);
    if (setsockopt(listen_fd, SOL_SOCKET, SO_REUSEADDR, (char *) &i, sizeof (i)) < 0)
    {
        perror ("setsockopt");
        return -1;
    }

    /* Bind the socket to the specified interface */
    result=setsockopt(listen_fd, SOL_SOCKET, SO_BINDTODEVICE,server_interface, strlen(server_interface) + 1);

    if(result<0)
    {
        close(listen_fd);
        perror("setsockopt failed");
        return 1;
    }

    memset (usaP, 0, sizeof (usockaddr));
    usaP->sa.sa_family = AF_INET;
    usaP->sa_in.sin_port = htons(server_port);
    /* #UnihanBU6#,Alisa_Gao, 20080812,Add, End  */
    if (bind (listen_fd, &usaP->sa, sizeof (struct sockaddr_in)) < 0)
    {
        perror ("bind");
        return -1;
    }
    if (listen (listen_fd, 1024) < 0)
    {
        perror ("listen");
        return -1;
    }
    return listen_fd;
}

static void
init_session(t_s *h)
{
	t_s *p=h;
	p->t = stimeout;  /* init time out */
	p->st = 0;
	p->lvl = 0;
      p->addr= NULL;
      p->mac= NULL;
      p->hiddenlevel = 0;  /* #UnihanBU6# , Ailsa_Cai , 20090317 , Add entrance for hidden pages */
      p->next = NULL;
}

static t_s*  find_session(t_s *s, char *addr)
{
   t_s *p=s->next;
   char *d=addr;
   while(p)
   {
      if(strcmp(p->addr,d) == 0)
      {
         return p;
      }
	else
	{
         p=p->next;
	}
   }
   return NULL;
}

static void
session_add(t_s *h, t_s *s)
{
  t_s *p=h;
	while(p->next)
	{
		p=p->next;
	}
	p->next=s;	
}

static void 
free_tsession(t_s *h)
{
	t_s *tmp=h;
	t_s *p=h;
	while(p)
	{
		tmp = p->next;
		free(p->addr);
		free(p->mac);
		free(p);
		p = tmp;
	}
}

static void
delete_session(t_s *h, char *addr)
{
   t_s *p=h;
   char *d= addr;
   t_s *tmp;
  
  while(p->next)
  {
      tmp = p->next;
      if(strcmp(tmp->addr,d) == 0)
      { 
    	  if(tmp->next)
    	  {  	
           p->next = tmp->next;
         }
         else
         {
           p->next = NULL;	
         }
         free(tmp);
         return;	
      }
      p = p->next;	    	
  }	

}

static void
check_dsession(t_s *h)
{
     t_s *p=h->next;
     int timeout = stimeout;
	
    Uint32 sysUpTime = 0; /* #UnihanBu6#, Ailsa_Cai ,20090415, Fixed Httpd hidden page shouldn't input the password twice. */	

    while(p)
    {
        /* #UnihanBu6#, Ailsa_Cai ,20090415, Fixed Httpd hidden page shouldn't input the password twice. Start*/	
        Common_Utils_GetSysUpTime(&sysUpTime);
        if (((sysUpTime) - (p->st)) > (timeout) && timeout != 0)   
        {
        /* #UnihanBu6#, Ailsa_Cai ,20090415, End */  
#if (EDEBUG)
     printf("== check_dsession delete addr:%s == \n",p->addr);
#endif //(EDEBUG)

           delete_session(h,p->addr);
	 }
	     p = p->next;
   }
}

static int
get_slevel(t_s *h, char *addr)
{
     t_s *f_p = find_session(h,addr);

     if (f_p)
     {
         return f_p->lvl;
     }
     else
     {
         return 0;
     }
}

/* #UnihanBU6# , Ailsa_Cai , 20090317 , Add entrance for hidden pages */
static int
get_hiddenlevel(t_s *h, char *addr)
{
     t_s *f_p = find_session(h,addr);

     if (f_p)
     {
         return f_p->hiddenlevel;
     }
     else
     {
         return 0;
     }
}

static void
set_hiddenlevel(t_s *h, char *addr, int lvl)
{

     t_s *f_p = find_session(h,addr);

     if (f_p)
     {
         f_p->hiddenlevel = lvl;
     }
     return ;
}
/* #UnihanBU6# , Ailsa_Cai , 20090317 ,End */

/* #UnihanBU6#,Alisa_Gao, 20080818,Add for access type,Start.*/
static int set_interface(t_s *h,char*addr,unsigned int interface)
{
     t_s *f_p = find_session(h,addr);
     if (f_p)
     {
         f_p->interface =interface;
	   return 1;
     }
     else
     {
         return 0;
     }
}
/* #UnihanBU6#,Alisa_Gao, 20080818,Add for access type,End. */



int
get_ulevel()
{
	   t_s *h = s_head;
	   char *addr = c_addr;
     t_s *f_p = find_session(h,addr);

     if (f_p)
     {
         return f_p->lvl;
     }
     else
     {
         return 0;
     }
}

static void
set_slevel(t_s *h, char *addr, int lvl)
{
#if (EDEBUG)
     printf("== set_slevel(1) addr:%s  lvl: %d== \n",addr,lvl);
#endif //(EDEBUG)

     t_s *f_p = find_session(h,addr);

     if (f_p)
     {
         f_p->lvl = lvl;
     }
     return ;
}

void
set_ulevel(int lvl)
{
#if (EDEBUG)
     printf("== set_slevel(1) addr:%s  lvl: %d== \n",c_addr,lvl);
#endif //(EDEBUG)
	   t_s *h = s_head;
	   char *addr = c_addr;
     t_s *f_p = find_session(h,addr);

     if (f_p)
     {
         f_p->lvl = lvl;
     }
     return ;
}

static int
auth_check( char* dirname, char* authorization )
{
    char authinfo[500];
    char* authpass;
    int l;
    /* #UnihanBU6#,Alisa_Gao, 20080818,Add, Start */
    unsigned int lanlevel=0;
    unsigned int wanlevel=0;
    HttpUserInfo userInfo[4];
    /* #UnihanBU6#,Alisa_Gao, 20080818,Add, End  */
  
    /*#UnihanBU6# Water Huang, 2009.2.3
     * Modify for Comcast security requirement.
     * EMTA-17: Password Encryption
     */
    char processed_authpass[AUTH_MAX];
    memset(authinfo,0,sizeof(authinfo));
    memset(processed_authpass,0,sizeof(processed_authpass));

    /* Basic authorization info? */
    if ( !authorization || strncmp( authorization, "Basic ", 6 ) != 0 ) 
    {
        return 0;
    }

    /* Decode it. */
    l = b64_decode( &(authorization[6]), (unsigned char*)authinfo, sizeof(authinfo) );
    authinfo[l] = '\0';
    /* Split into user and password. */
    authpass = strchr( authinfo, ':' );
    if ( authpass == (char*) 0 )
    {
	/* No colon?  Bogus auth info. */
        return 0;
    }
    *authpass++ = '\0';

  /* #UnihanBU6#,Alisa_Gao, 20080818,Add for access type, Start  */

/*	 Notes: 1. user_type: 
	 			SYS_DFLT_HTTP_USER_TYPE_LOCAL_MSO
 *                       SYS_DFLT_HTTP_USER_TYPE_LOCAL_ADMIN
 *                       SYS_DFLT_HTTP_USER_TYPE_REMOTE_MSO
 *                       SYS_DFLT_HTTP_USER_TYPE_REMOTE_ADMIN
 *         2. *ip_stack:
 *				 SYS_DFLT_HTTP_USER_IP_STACK_ID_NONE(disbale)
 *                        SYS_DFLT_HTTP_USER_IP_STACK_ID_LOCAL
 *                        SYS_DFLT_HTTP_USER_IP_STACK_ID_CM
 *                        SYS_DFLT_HTTP_USER_IP_STACK_ID_WAN
 */
    t_s *f_p=find_session(s_head,c_addr);
/* #UnihanBU6#, Eating Chen, 2009/12/03, System_ShmDb_RetrieveAccess() function 
 * shall be called once in each process/task 
 */
#if 0
    if(System_ShmDb_RetrieveAccess() == NOK)
    {
        fprintf(stderr,"%s","System Shmdb Retrieve Access Failed");
        return 0;
    }
#endif
    int i=0;
	/*get username,password,interface information*/
    for(i=0;i<4;i++)
    {
        if(Common_Utils_GetHttpUserIpStack ((i+1), &(userInfo[i].ip_stack)) == NOK)
        {
            userInfo[i].ip_stack = SYS_DFLT_HTTP_USER_IP_STACK_ID_NONE;
        }
    
        if(Common_Utils_GetHttpUserPassword((i+1),userInfo[i].userpasswd) == NOK) 
        {
            strcpy(userInfo[i].userpasswd,"");
        }
    
        if(Common_Utils_GetHttpUserName((i+1),userInfo[i].username) == NOK)
        {
            strcpy(userInfo[i].username,"");
        }

        userInfo[i].accesslevel = userlevel[i];
    }
    /*get lan/wan access type from techsupport */
    if (Common_Utils_GetHttpLanAccessType(&lanlevel) == NOK)
    {
        lanlevel=0;
    }
    
    if (Common_Utils_GetHttpWanAccessType(&wanlevel) == NOK)
    {
        wanlevel=0;
    }

/*#UnihanBU6# Water Huang, 2009.2.3
 * Modify for Comcast security requirement.
 * EMTA-17: Password Encryption
 */
#ifdef CONFIG_FOR_NETGEAR_CVD31XT
    MD5((const unsigned char *)authpass,strlen(authpass),(unsigned char *)processed_authpass);
#else
    memcpy(processed_authpass, authpass, strlen(authpass));
#endif    

    for(i=0;i<4;i++)
    {
	if((strcmp( userInfo[i].username, authinfo ) == 0) &&(strncmp(userInfo[i].userpasswd, processed_authpass,SYS_DFLT_HTTP_MSO_PASS_LEN) == 0))
       {	
            if(userInfo[i].ip_stack == f_p->interface)
            {
                if(userInfo[i].ip_stack == SYS_DFLT_HTTP_USER_IP_STACK_ID_LOCAL)
                {
                    return userInfo[i].accesslevel<lanlevel ? userInfo[i].accesslevel :lanlevel;/*return the smaller one*/
                }
                else if(userInfo[i].ip_stack == SYS_DFLT_HTTP_USER_IP_STACK_ID_NONE)
                {
                    return AuthLevelUnauthorized;
                }
                else
                {
                    return userInfo[i].accesslevel<wanlevel ? userInfo[i].accesslevel:wanlevel;
                }
            }
            else if ((userInfo[i].ip_stack > SYS_DFLT_HTTP_USER_IP_STACK_ID_LOCAL) && (f_p->interface == SYS_DFLT_HTTP_USER_IP_STACK_ID_WAN))
	    {	  
		  return userInfo[i].accesslevel<wanlevel ? userInfo[i].accesslevel:wanlevel;
	    }
        }
    }
   /* #UnihanBU6#,Alisa_Gao, 20080818,Add for access type, End  */
    return AuthLevelUnauthorized; 
}

void
send_authenticate (char *realm)
{
  char header[10000];

  (void) snprintf (header, sizeof (header),
		   "WWW-Authenticate: Basic realm=\"%s\"", realm);
  send_error (401, "Unauthorized", header,
	      "Authorization required.");
}


static void
send_error (int status, char *title, char *extra_header, char *text)
{

  // jimmy, https, 8/4/2003, fprintf -> wfprintf, fflush -> wfflush
  send_headers (status, title, extra_header, "text/html");
  (void) wfprintf (conn_fp,
		   "<HTML><HEAD><TITLE>%d %s</TITLE></HEAD>\n<BODY BGCOLOR=\"#cc9999\"><H4>%d %s</H4>\n",
		   status, title, status, title);
  (void) wfprintf (conn_fp, "%s\n", text);
  (void) wfprintf (conn_fp, "</BODY></HTML>\n");
  (void) wfflush (conn_fp);
}

static void
send_headers (int status, char *title, char *extra_header, char *mime_type)
{
  time_t now;
  char timebuf[100];

  // jimmy, https, 8/4/2003, fprintf -> wfprintf
  wfprintf (conn_fp, "%s %d %s\r\n", PROTOCOL, status, title);
  wfprintf (conn_fp, "Server: %s\r\n", SERVER_NAME);
  now = time ((time_t *) 0);
  strftime (timebuf, sizeof (timebuf), RFC1123FMT, gmtime (&now));
  wfprintf (conn_fp, "Date: %s\r\n", timebuf);
  if (extra_header != (char *) 0 && *extra_header)
    wfprintf (conn_fp, "%s\r\n", extra_header);
  if (mime_type != (char *) 0)
    wfprintf (conn_fp, "Content-Type: %s\r\n", mime_type);
  wfprintf (conn_fp, "Connection: close\r\n");
  wfprintf (conn_fp, "\r\n");
}


/* Base-64 decoding.  This represents binary data as printable ASCII
** characters.  Three 8-bit binary bytes are turned into four 6-bit
** values, like so:
**
**   [11111111]  [22222222]  [33333333]
**
**   [111111] [112222] [222233] [333333]
**
** Then the 6-bit values are represented using the characters "A-Za-z0-9+/".
*/

static int b64_decode_table[256] = {
  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,	/* 00-0F */
  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,	/* 10-1F */
  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, 62, -1, -1, -1, 63,	/* 20-2F */
  52, 53, 54, 55, 56, 57, 58, 59, 60, 61, -1, -1, -1, -1, -1, -1,	/* 30-3F */
  -1, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14,	/* 40-4F */
  15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, -1, -1, -1, -1, -1,	/* 50-5F */
  -1, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40,	/* 60-6F */
  41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, -1, -1, -1, -1, -1,	/* 70-7F */
  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,	/* 80-8F */
  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,	/* 90-9F */
  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,	/* A0-AF */
  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,	/* B0-BF */
  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,	/* C0-CF */
  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,	/* D0-DF */
  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,	/* E0-EF */
  -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1	/* F0-FF */
};

/* Do base-64 decoding on a string.  Ignore any non-base64 bytes.
** Return the actual number of bytes generated.  The decoded size will
** be at most 3/4 the size of the encoded, and may be smaller if there
** are padding characters (blanks, newlines).
*/
static int
b64_decode( const char* str, unsigned char* space, int size )
    {
    const char* cp;
    int space_idx, phase;
    int d, prev_d=0;
    unsigned char c;

    space_idx = 0;
    phase = 0;
    for ( cp = str; *cp != '\0'; ++cp )
	{
	d = b64_decode_table[(int)*cp];
	if ( d != -1 )
	    {
	    switch ( phase )
		{
		case 0:
		++phase;
		break;
		case 1:
		c = ( ( prev_d << 2 ) | ( ( d & 0x30 ) >> 4 ) );
		if ( space_idx < size )
		    space[space_idx++] = c;
		++phase;
		break;
		case 2:
		c = ( ( ( prev_d & 0xf ) << 4 ) | ( ( d & 0x3c ) >> 2 ) );
		if ( space_idx < size )
		    space[space_idx++] = c;
		++phase;
		break;
		case 3:
		c = ( ( ( prev_d & 0x03 ) << 6 ) | d );
		if ( space_idx < size )
		    space[space_idx++] = c;
		phase = 0;
		break;
		}
	    prev_d = d;
	    }
	}
    return space_idx;
    }


/* Simple shell-style filename matcher.  Only does ? * and **, and multiple
** patterns separated by |.  Returns 1 or 0.
*/
int
match (const char *pattern, const char *string)
{
  const char *or;
  for (;;)
    {
      or = strchr (pattern, '|');
      if (or == (char *) 0)
	return match_one (pattern, strlen (pattern), string);
      if (match_one (pattern, or - pattern, string))
	return 1;
      pattern = or + 1;
    }
}


static int
match_one (const char *pattern, int patternlen, const char *string)
{
  const char *p;

  for (p = pattern; p - pattern < patternlen; ++p, ++string)
    {
      if (*p == '?' && *string != '\0')
	continue;
      if (*p == '*')
	{
	  int i, pl;
	  ++p;
	  if (*p == '*')
	    {
	      /* Double-wildcard matches anything. */
	      ++p;
	      i = strlen (string);
	    }
	  else
	    /* Single-wildcard matches anything but slash. */
	    i = strcspn (string, "/");
	  pl = patternlen - (p - pattern);
	  for (; i >= 0; --i)
	    if (match_one (p, pl, &(string[i])))
	      return 1;
	  return 0;
	}
      if (*p != *string)
	return 0;
    }
  if (*string == '\0')
    return 1;
  return 0;
}


void
do_file(char *path, webs_t stream)
{
	FILE *fp;
	int c;

	if (!(fp = fopen(path, "r")))
		return;

	while ((c = getc(fp)) != EOF)
		{
			wfputc(c, stream);

		}
	fclose(fp);
}


int
containsstring (char *source, char *cmp)
{
  if (cmp == NULL || source == NULL)
    return 0;
  int slen=strlen(source);
  int clen=strlen(cmp);
  if (slen < clen)
    return 0;
  int i;
  
  for (i = 0; i < slen - clen; i++)
    {
      int a;
      int r = 0;
      for (a = 0; a < clen; a++)
	if (source[i + a] != cmp[a])
	  r++;
      if (!r)
	return 1;
    }
  return 0;
}

#define LINE_LEN 10000


static void
handle_request(void)
{
    char line[10000], *cur;
    char *method, *path, *protocol, *authorization, *boundary;
    char *cp;
    char *file;
    int hiddenFlag = 0;     //hiddenpages: 1  other : 0  /* #UnihanBU6# , Ailsa_Cai , 20090317 , Add for check if hidden pages */
    int len;
    struct mime_handler *handler;
    int cl = 0, flags;
    /* Initialize the request variables. */
    authorization = boundary = NULL;
    bzero( line, sizeof line );

    /* Parse the first line of the request. */
    if ( wfgets( line, sizeof(line), conn_fp ) == (char*) 0 ) {
	send_error( 400, "Bad Request", (char*) 0, "No request found." );
	return;
    }
    /* #UnihanBU6# , Alisa_Gao , 20090417 ,TT1363:User can't access GUI from remote side anymore */
   /* To prevent http receive https packets, cause http crash (by honor 2003/09/02) */
    if (strncasecmp (line, "GET", 3) && strncasecmp (line, "POST", 4))
    {
       send_error( 400, "Bad Request", (char*) 0, "No request found." );
       return;
    }
   /* #UnihanBU6# , Alisa_Gao , 20090417,Add End*/
    method = path = line;
    strsep(&path, " ");
    while (*path == ' ') path++;
    protocol = path;
    strsep(&protocol, " ");
    while (*protocol == ' ') protocol++;
    cp = protocol;
    strsep(&cp, " ");
    if ( !method || !path || !protocol ) {
	send_error( 400, "Bad Request", (char*) 0, "Can't parse request." );
	return;
    }
    cur = protocol + strlen(protocol) + 1;

    /* Parse the rest of the request headers. */
    while ( wfgets( cur, line + sizeof(line) - cur, conn_fp ) != (char*) 0 )
	{
	if ( strcmp( cur, "\n" ) == 0 || strcmp( cur, "\r\n" ) == 0 ){
	    break;
	}
	else if ( strncasecmp( cur, "Authorization:", 14 ) == 0 )
	    {
	    cp = &cur[14];
	    cp += strspn( cp, " \t" );
	    authorization = cp;
	    cur = cp + strlen(cp) + 1;
	    }
	else if (strncasecmp( cur, "Content-Length:", 15 ) == 0) {
		cp = &cur[15];
		cp += strspn( cp, " \t" );
		cl = strtoul( cp, NULL, 0 );
	}
	else if ((cp = strstr( cur, "boundary=" ))) {
            boundary = &cp[9];
	    for( cp = cp + 9; *cp && *cp != '\r' && *cp != '\n'; cp++ );
	    *cp = '\0';
	    cur = ++cp;
	}

	}

    if ( strcasecmp( method, "get" ) != 0 && strcasecmp(method, "post") != 0 ) {
	send_error( 501, "Not Implemented", (char*) 0, "That method is not implemented." );
	return;
    }
	
    if ( path[0] != '/' ) {
	send_error( 400, "Bad Request", (char*) 0, "Bad filename." );
	return;
    }
    file = &(path[1]);
    len = strlen( file );
    if ( file[0] == '/' || strcmp( file, ".." ) == 0 || strncmp( file, "../", 3 ) == 0 || strstr( file, "/../" ) != (char*) 0 || strcmp( &(file[len-3]), "/.." ) == 0 ) {
	send_error( 400, "Bad Request", (char*) 0, "Illegal filename." );
	return;
    }
    if ( file[0] == '\0' || file[len-1] == '/' )
    {
	    file = "index.html";
    }
    Uint32 uint32Var;
    char  LanIp[32],LanMask[32];
    memset(LanIp, 0, sizeof(LanIp));
    memset(LanMask, 0, sizeof(LanIp));
    if(ProductionDb_RetrieveAccess() == NOK)
    {
        return;
    }	
    if (ProdDb_GetLanIpAddr(&uint32Var) == OK)
    {
        sprintf(LanIp,"%d.%d.%d.%d ",(unsigned int)(uint32Var >> 24) & 0xFF,(unsigned int)(uint32Var >> 16) & 0xFF,(unsigned int)(uint32Var >>  8) & 0xFF,(unsigned int)(uint32Var) & 0xFF);
    }
    if (ProdDb_GetLanIpMask(&uint32Var) == OK)
    {
        sprintf(LanMask,"%d.%d.%d.%d ",(unsigned int)(uint32Var >> 24) & 0xFF,(unsigned int)(uint32Var >> 16) & 0xFF,(unsigned int)(uint32Var >>  8) & 0xFF,(unsigned int)(uint32Var) & 0xFF);
    }

    for (handler = &mime_handlers[0]; handler->pattern; handler++) {
        if (match(handler->pattern, file)) {
            if (handler->auth) 
            {	
		/* #UnihanBU6#,Rainy, 20080618, implement httpd session and access level.  */
                int accesslevel=0;
                int lvl_t = 0;
	         int hidden_lvl = 0;  //* #UnihanBU6# , Ailsa_Cai , 20090317 , Add level for hidden pages */

		/* #UnihanBU6#,Alisa_Gao, 20080818, add for access type,Start  */
				 
                lvl_t = get_slevel(s_head,c_addr);

                 /* #UnihanBU6#,Alisa_Gao, 200090319*/
                /*Fix bug:When CPE is dynamic,Access Lan0,also can access advance pages,It is not correct*/
                /*When Access by lan0,User just can access basic pages*/
                int level_t = SYS_DFLT_HTTP_USER_IP_STACK_ID_NONE;
                if(strcmp(server_interface,SYS_DFLT_IFNAME_LOCAL) == 0)
                {
                	level_t = SYS_DFLT_HTTP_USER_IP_STACK_ID_LOCAL;
                }
                else 
                {
                	level_t = SYS_DFLT_HTTP_USER_IP_STACK_ID_WAN;/*mta0 and wan0 is the same now*/
                }
                set_interface(s_head,c_addr,level_t);
               /* #UnihanBU6#,Alisa_Gao, 200090319,Fix bug:End*/ 
				
	         /* #UnihanBU6# , Ailsa_Cai , 20090317 , Add entrance for hidden pages ,Start */
                int temp = 0;
                hidden_lvl = get_hiddenlevel(s_head , c_addr);

                while(hidden_page[temp] != NULL)
                {
                    if(strcmp(file,(char *)hidden_page[temp] ) == 0)
                    {
                        hiddenFlag = 1;						
                        break;
                    }
                    temp++ ;
                }

                if(hiddenFlag == 1)
                {
                    if(hidden_lvl == 0)
                    {
                        set_hiddenlevel(s_head,c_addr,1);    //hidden pages have login 
                        send_authenticate(auth_realm);
                        return ;
                    }
                    accesslevel = auth_hidden_check(auth_realm, authorization);
					
                    if(accesslevel == -2)   //interface is not lan
                    {
                        send_error(403, "Access Forbidden", NULL , "This Interface is Forbidden !" );
                        return ;
                    }  

                    if ( accesslevel == AuthLevelUnauthorized )  //Unauthorized
                    {
                        send_authenticate (auth_realm);
                        return ;
                    }

                    if(!strcmp(file,"hidden_Logout.html"))
                    { 
                        set_hiddenlevel(s_head, c_addr, 0); 
                        file="hidden_Logout.html";
                    }
                }
                else
                {   /* #UnihanBU6# , Ailsa_Cai , 20090317 , End */
                /* #UnihanBU6# , Ailsa_Cai , 20090806 , Fixed TT1517: session timeout dosen't work */
                    if(lvl_t == 0)
                    {
                        set_slevel(s_head,c_addr,-1);
                        send_authenticate (auth_realm);                    	
                        return;
                    }
                /* #UnihanBU6# , Ailsa_Cai , 20090806 ,End */
                    accesslevel = auth_check(auth_realm, authorization);

                    if(lvl_t<accesslevel)
                    {
                        set_slevel(s_head,c_addr,accesslevel); 
                    }
			
                    lvl_t = get_slevel(s_head,c_addr);	

                    if (!lvl_t)
                    {
                        #ifdef CONFIG_FOR_NETGEAR_CVD31XT
                            if( OK != EVMIF_EventSend(EVENT_MGR_CTRL_SOCK, VENDOR_EVM_WEB_LOGIN_FAIL, 1, NULL, NULL)) 
                            {
                                printf("Send WEB Login Error Event Fail!");
                            }
                        #endif 
                        send_authenticate (auth_realm);
                        return;
                    }

                    int index = 0;
                    int expectlevel=AuthLevelUser;
                    while(advanced_page[index] != NULL)
                    {
                        if(strcmp(file,(char *)advanced_page[index] ) == 0)
                        {
                            expectlevel =  AuthLevelAdminAdvanced;
                            break;
                        }
                        index++ ;
                    }
                    if (lvl_t < expectlevel)
                    {   
                        if(lvl_t == AuthLevelAdmin)
                        {
                            /* #UnihanBU6#,Alisa_Gao, 20080717, jump to POTD page  */
                            file="adv_entrance.asp";
                        }
                        else 
                        {
                            send_authenticate (auth_realm);
                            return;
                        }
                    }
                    /* #UnihanBU6#,Alisa_Gao, 20080717, after logout,set the access level to 0  */
                    if(!strcmp(file,"RgLogout.html"))
                    { 
                        set_slevel(s_head, c_addr, 0); 
                        file="RgLogout.html";
                    }	
                    /* #UnihanBU6#,Alisa_Gao, 20080717, End*/
                }
            }
            if (strcasecmp(method, "post") == 0 && !handler->input) 
            {
                send_error(501, "Not Implemented", NULL, "That method is not implemented.");
                return;
            }			    
	    /* #UnihanBU6#,Alisa_Gao, 20080715,implement POST method for httpd server  */
            if (strcasecmp (method, "post") == 0)
            {
                post = 1;
            }
            /*for GET method*/
            else
            {
                post =0;
            }
            /* #UnihanBU6#,Alisa_Gao, 20080715,End*/
            if (handler->input) 
            {
                handler->input(file, conn_fp, cl, boundary);
                #if defined(linux)
                    #ifdef HAVE_HTTPS
    	                 if (!do_ssl && (flags = fcntl (fileno (conn_fp), F_GETFL)) != -1 &&
                            fcntl (fileno (conn_fp), F_SETFL, flags | O_NONBLOCK) != -1)
                        {
                        /* Read up to two more characters */
                        if (fgetc (conn_fp) != EOF)
                            (void) fgetc (conn_fp);
                        fcntl (fileno (conn_fp), F_SETFL, flags);
                    }
                   #else
	                 if ((flags = fcntl (fileno (conn_fp), F_GETFL)) != -1 &&
	                      fcntl (fileno (conn_fp), F_SETFL, flags | O_NONBLOCK) != -1)
                        {
                        /* Read up to two more characters */
                            if (fgetc (conn_fp) != EOF)
                                (void) fgetc (conn_fp);
                            fcntl (fileno (conn_fp), F_SETFL, flags);
                    }
                  #endif
              #endif
            }
            send_headers( 200, "Ok", handler->extra_header, handler->mime_type );
            if (handler->output)
                handler->output(file, conn_fp);

            break;
        }
    }

    if (!handler->pattern) 
	    send_error( 404, "Not Found", (char*) 0, "File not found." );
}


void				
get_client_ip_mac_am (int conn_fp,char *c_addr,char *c_mac)
{
  struct sockaddr_in sa;
  int len = sizeof (struct sockaddr_in);
  getpeername (conn_fp, (struct sockaddr *) &sa, &len);
  sprintf (c_addr, inet_ntoa (sa.sin_addr));
}

static void
handle_server_sig_int (int sig)
{
#ifdef HAVE_HTTPS
 // ct_syslog (LOG_INFO, httpd_level, "httpd server %sshutdown",
	    // do_ssl ? "(ssl support) " : "");
#else
 // ct_syslog (LOG_INFO, httpd_level, "httpd server shutdown");
#endif
  exit (0);
}

void
settimeouts (int sock, int secs)
{
  struct timeval tv;

  tv.tv_sec = secs;
  tv.tv_usec = 0;
  if (setsockopt (sock, SOL_SOCKET, SO_SNDTIMEO, &tv, sizeof (tv)) < 0)
    perror ("setsockopt(SO_SNDTIMEO)");
  if (setsockopt (sock, SOL_SOCKET, SO_RCVTIMEO, &tv, sizeof (tv)) < 0)
    perror ("setsockopt(SO_RCVTIMEO)");
}

/****************************************************************************
 *
 > $Function: decodeString()
 *
 * $Description: Given a URL encoded string, convert it to plain ascii.
 *   Since decoding always makes strings smaller, the decode is done in-place.
 *   Thus, callers should strdup() the argument if they do not want the
 *   argument modified.  The return is the original pointer, allowing this
 *   function to be easily used as arguments to other functions.
 *
 * $Parameters:
 *      (char *) string . . . The first string to decode.
 *      (int)    flag   . . . 1 if require decode '+' as ' ' for CGI
 *
 * $Return: (char *)  . . . . A pointer to the decoded string (same as input).
 *
 * $Errors: None
 *
 ****************************************************************************/
static char *
decodeString (char *orig, int flag_plus_to_space)
{
  /* note that decoded string is always shorter than original */
  char *string = orig;
  char *ptr = string;

  while (*ptr)
    {
      if (*ptr == '+' && flag_plus_to_space)
	{
	  *string++ = ' ';
	  ptr++;
	}
      else if (*ptr != '%')
	*string++ = *ptr++;
      else
	{
	  unsigned int value;
	  sscanf (ptr + 1, "%2X", &value);
	  *string++ = value;
	  ptr += 3;
	}
    }
  *string = '\0';
  return orig;
}

/****************************************************************************
 *
 > $Function: encodeString()
 *
 * $Description: Given a string, html encode special characters.
 *   This is used for the -e command line option to provide an easy way
 *   for scripts to encode result data without confusing browsers.  The
 *   returned string pointer is memory allocated by malloc().
 *
 * $Parameters:
 *      (const char *) string . . The first string to encode.
 *
 * $Return: (char *) . . . .. . . A pointer to the encoded string.
 *
 * $Errors: Returns a null string ("") if memory is not available.
 *
 ****************************************************************************/
static char *
encodeString (const char *string)
{
  /* take the simple route and encode everything */
  /* could possibly scan once to get length.     */
  int len = strlen (string);
  char *out = malloc (len * 5 + 1);
  char *p = out;
  char ch;

  if (!out)
    return "";
  while ((ch = *string++))
    {
      // very simple check for what to encode

      if (ch > '0' - 1 && ch < '9' + 1)
	*p++ = ch;
      else if (ch > 'a' - 1 && ch < 'z' + 1)
	*p++ = ch;
      else if (ch > 'A' - 1 && ch < 'Z' + 1)
	*p++ = ch;
      else
	p += sprintf (p, "&#%d;", (unsigned char) ch);
    }
  *p = 0;
  return out;
}


int
main (int argc, char **argv)
{
#if (EDEBUG)
     printf("== http main()!!!! == \n");
#endif //(EDEBUG)

    usockaddr usa;
    int listen_fd;
    int conn_fd;
    socklen_t sz = sizeof (usa);
    int c;
    int timeout = TIMEOUT;
    int isSetHttps = 0;
/*#UnihanBU6#,Ailsa_Cai,20090812,Add CLI & mib control for http session timeout ,set default 600 sec*/
    unsigned int httpSessionTimeout;

  struct stat stat_dir;
//cprintf("init nvram tab\n");
/* SEG addition */
  //Initnvramtab ();
//cprintf("done()\n");
#ifdef HAVE_OPENSSL
  BIO *sbio;
  SSL_CTX *ctx = NULL;
  int r;
  BIO *ssl_bio;
#endif

  server_port = DEFAULT_HTTP_PORT;
  server_interface = SYS_DFLT_IFNAME_LOCAL;
   while ((c = getopt (argc, argv, "Sih:p:d:t:s:g:e:I:")) != -1)
    switch (c)
      {
#ifdef HAVE_HTTPS
      case 'S':
	#if defined(HAVE_OPENSSL) || defined(HAVE_MATRIXSSL)
		isSetHttps = 1;
		do_ssl = 1;
		server_port = DEFAULT_HTTPS_PORT;
	#else
		fprintf (stderr, "No SSL support available\n");
		exit (0);
	#endif
	break;
#endif
      case 'h':
	server_dir = optarg;
	break;

      case 'a':
	server_ip = optarg;
	break;      
      case 'p':
      	server_port = atoi (optarg);
      	#ifdef HAVE_HTTPS
      	if(isSetHttps == 1)
      	{
      	    if(server_port != DEFAULT_HTTPS_PORT)
      	    {
      	        fprintf(stderr,"Error,Https Support Port is 443!");
      		  exit(0);
      	    }
       }
      	#endif 	
	break;
	 /* #UnihanBU6#,Alisa_Gao, 20080812,add interface, Start  */
	case 'I':
	server_interface = optarg;
	break;
	 /* #UnihanBU6#,Alisa_Gao, 20080812, End  */
      case 't':
	timeout = atoi (optarg);
	break;
#ifdef DEBUG_CIPHER
      case 's':
	set_ciphers = optarg;
	break;
      case 'g':
	get_ciphers = 1;
	break;
#endif
      case 'e':
	printf ("%s", encodeString (optarg));
	exit (0);
	break;
      case 'd':
	printf ("%s", decodeString (optarg, 1));
	exit (0);
	break;
      case 'i':
	fprintf (stderr, "Usage: %s [-S] [-p port] [-I interface]\n"
#ifdef HAVE_HTTPS
		 "	-S : Support https (port 443)\n"
#endif
		 "	-p port : Which port to listen?\n"
		 " 	-I:which interface to bind to?\n"
		 "	-t secs : How many seconds to wait before timing out?\n"
		 "	-s ciphers: set cipher lists\n"
		 "	-g: get cipher lists\n"
		 "	-h: home directory: use directory\n"
		 "	-d: decode string\n"
		 "	-e: encode string\n", argv[0]);
	exit (0);
	break;
      default:
	exit(0);
	//break;
      }

  /* Ignore broken pipes */
  signal (SIGPIPE, SIG_IGN);
  signal (SIGTERM, handle_server_sig_int);	// kill

  if (server_dir && stat (server_dir, &stat_dir) == 0)
    chdir (server_dir);

  /* Build our SSL context */
#ifdef HAVE_HTTPS
  if (do_ssl)
    {
#ifdef HAVE_OPENSSL
      SSLeay_add_ssl_algorithms ();
      SSL_load_error_strings ();
      ctx = SSL_CTX_new (SSLv23_server_method ());
      
      /* #UnihanBU6 Water Huang 2008.8.27#
       * Set SSL bug workaround options.
       * Refer http://www.openssl.org/docs/ssl/SSL_CTX_set_options.html 
       * to detail explain.
       */
      SSL_CTX_set_options(ctx,SSL_OP_ALL);
      
      if (SSL_CTX_use_certificate_file (ctx, CERT_FILE, SSL_FILETYPE_PEM) ==
	  0)
	{
	  //cprintf ("Can't read %s\n", CERT_FILE);
	  ERR_print_errors_fp (stderr);
	  exit (1);

	}
      if (SSL_CTX_use_PrivateKey_file (ctx, KEY_FILE, SSL_FILETYPE_PEM) == 0)
	{
	  //cprintf ("Can't read %s\n", KEY_FILE);
	  ERR_print_errors_fp (stderr);
	  exit (1);
	}
      if (SSL_CTX_check_private_key (ctx) == 0)
	{
	  //cprintf ("Check private key fail\n");
	  ERR_print_errors_fp (stderr);
	  exit (1);
	}
#endif

#ifdef HAVE_MATRIXSSL
      matrixssl_init ();
      if (0 != matrixSslReadKeys (&keys, CERT_FILE, KEY_FILE, NULL, NULL))
	{
	  fprintf (stderr, "Error reading or parsing %s.\n", KEY_FILE);
	  exit (0);
	}
#endif
    }
#endif

  /* Initialize listen socket */
  if ((listen_fd = initialize_listen_socket (&usa)) < 0)
    {
      //ct_syslog (LOG_ERR, httpd_level, "Can't bind to any address");
      exit (errno);
    }
    #if !defined(DEBUG)
    {
       FILE *pid_fp;
       if(strlen(pid_file) ==0)
       {
   	     if(isSetHttps == 1)
   	     {
   	         sprintf(pid_file,"/var/run/https_%s.pid",server_interface);
   	     }
   	     else/*It is not https*/
   	     {
   	   	   sprintf(pid_file,"/var/run/httpd_%s.pid",server_interface);
   	     }
        }
        /* Daemonize and log PID */
        if (daemon (1, 1) == -1)
        {
            perror ("daemon");
            exit (errno);
        }
        if (!(pid_fp = fopen (pid_file, "w")))
        {
            perror (pid_file);
            return errno;
        }
        fprintf (pid_fp, "%d", getpid ());
        fclose (pid_fp);
    }
    #endif

    t_s *t_p=NULL;
    s_head = (t_s *)malloc(sizeof(t_s));
    init_session(s_head);
    
    memset(c_addr,0,strlen(c_addr));
    memset(c_mac,0,strlen(c_mac));

    /* #UnihanBU6#, Eating Chen, 2009/12/02, System_ShmDb_RetrieveAccess() function 
     * shall be called once in each process/task 
     */
    if (System_ShmDb_RetrieveAccess()!=OK)
    {
       printf("Snmp_ShmDb_RetrieveAccess() fail (httpd process)\n");
       return -1;
    }

  /* Loop forever handling requests */
  for (;;)
    {
        /* #UnihanBU6#,Ailsa_Cai,20090810, Add CLI & mib control for http/telnet/ssh session timeout ,set default 600 secd,Start */
/* #UnihanBU6#, Eating Chen, 2009/12/02, System_ShmDb_RetrieveAccess() function 
 * shall be called once in each process/task 
 */
#if 0
        if( System_ShmDb_RetrieveAccess() != OK ||           
#endif
        if( System_ShmDb_GetHTTPInactivityTimeout( &httpSessionTimeout ) != OK)           
        {            
            printf( "Failed to access System DB!\n" );            
            return -1;        
         }
	 stimeout = httpSessionTimeout;
/* #UnihanBU6#,Ailsa_Cai,20090810, End */     
      if ((conn_fd = accept (listen_fd, &usa.sa, &sz)) < 0)
	{
	  perror ("accept");
	  return errno;
	}

      /* Make sure we don't linger a long time if the other end disappears */
      settimeouts (conn_fd, timeout);

#ifdef HAVE_HTTPS
      if (do_ssl)
	{
#ifdef HAVE_OPENSSL
	  sbio = BIO_new_socket (conn_fd, BIO_NOCLOSE);
	  ssl = SSL_new (ctx);

//#ifdef DEBUG_CIPHER
//              check_cipher();
//#endif
	  SSL_set_bio (ssl, sbio, sbio);

	  if ((r = SSL_accept (ssl) <= 0))
	    {
	      //berr_exit("SSL accept error");
	     // ct_syslog (LOG_ERR, httpd_level, "SSL accept error");
	      close (conn_fd);
	      continue;
	    }

	  conn_fp = (webs_t) BIO_new (BIO_f_buffer ());
	  ssl_bio = BIO_new (BIO_f_ssl ());
	  BIO_set_ssl (ssl_bio, ssl, BIO_CLOSE);
	  BIO_push ((BIO *) conn_fp, ssl_bio);
#elif defined(HAVE_MATRIXSSL)
	  matrixssl_new_session (conn_fd);
	  conn_fp = (FILE *) conn_fd;
#endif
	}
      else
#endif
	{
#ifdef HAVE_HTTPS
/*
	  if (check_action () == ACT_WEBS_UPGRADE)
	    {			// We don't want user to use web (http) during web (https) upgrade.
	      fprintf (stderr, "httpd: nothing to do...\n");
	      return -1;
	    }*/
#endif
	  if (!(conn_fp = fdopen (conn_fd, "r+")))
	    {
	      perror ("fdopen");
	      return errno;
	    }
	}

      //get_client_ip_mac (conn_fd);
     get_client_ip_mac_am(conn_fd,c_addr,c_mac);

     #if (EDEBUG)
     printf("client ip: %s \n",c_addr);
     printf("client mac: %s \n",c_mac);
     #endif //EDEBUG

 /* #UnihanBu6#, Ailsa_Cai ,20090415, Fixed Httpd hidden page shouldn't input the password twice. */
        Uint32 sysUpTime = 0;
        Common_Utils_GetSysUpTime(&sysUpTime);
/* #UnihanBu6#, Ailsa_Cai ,20090415, End. */ 
    
     check_dsession(s_head);  
  
     t_p=find_session(s_head,c_addr);
     if(t_p)
     {     
        t_p->st = sysUpTime; /* #UnihanBu6#, Ailsa_Cai ,20090415, Fixed Httpd hidden page shouldn't input the password twice. */
     }	    
     else
     {
        t_s *new_s = (t_s *)malloc(sizeof(t_s));
        char *new_addr = (char *)malloc(20*sizeof(char));
        char *new_mac = (char *)malloc(20*sizeof(char));
        init_session(new_s);
        sprintf(new_addr,"%s",c_addr);
        sprintf(new_mac,"%s",c_mac);
        new_s->addr = new_addr;
        new_s->mac = new_mac;
        new_s->st = sysUpTime;    /* #UnihanBu6#, Ailsa_Cai ,20090415, Fixed Httpd hidden page shouldn't input the password twice. */    
        session_add(s_head,new_s);
     }
  
    #if (EDEBUG)
    {
        t_s *p=s_head;
        int i = 0;
	  int timeout = stimeout;
        while(p)
        {
  	       printf("[%d] addr:%s mac:%s timeout:%d session time:%d user_lvl:%d  nvram t:%d\n",i++, p->addr,p->mac,p->t,p->st,p->lvl,timeout*60);
  	       p = p->next;
        }	
    }
    #endif //(EDEBUG)

		  
      handle_request ();

      wfflush (conn_fp);	// jimmy, https, 8/4/2003
      wfclose (conn_fp);	// jimmy, https, 8/4/2003
      close (conn_fd);
    }

    free_tsession(s_head);

  shutdown (listen_fd, 2);
  close (listen_fd);

  return 0;
}

char *
wfgets (char *buf, int len, FILE * fp)
{
#ifdef HAVE_HTTPS
#ifdef HAVE_OPENSSL
  if (do_ssl)
    return (char *) BIO_gets ((BIO *) fp, buf, len);
  else
#elif defined(HAVE_MATRIXSSL)
  if (do_ssl)
    return (char *) matrixssl_gets (fp, buf, len);
  else
#endif
#endif
    return fgets (buf, len, fp);
}

int
wfputc (char c, FILE * fp)
{

#ifdef HAVE_HTTPS
#ifdef HAVE_OPENSSL

  if (do_ssl)
    return BIO_printf ((BIO *) fp, "%c", c);
  else
#elif defined(HAVE_MATRIXSSL)
  if (do_ssl)
    return matrixssl_putc (fp, c);
  else
#endif
#endif
    return fputc (c, fp);
}

int
wfputs (char *buf, FILE * fp)
{
#ifdef HAVE_HTTPS
#ifdef HAVE_OPENSSL
  if (do_ssl)
    return BIO_puts ((BIO *) fp, buf);
  else
#elif defined(HAVE_MATRIXSSL)
  if (do_ssl)
    return matrixssl_puts (fp, buf);
  else
#endif
#endif
    return fputs (buf, fp);
}

int
wfprintf (FILE * fp, char *fmt, ...)
{
  va_list args;
  char buf[1024];
  int ret;
  //buf = (char*)malloc(1024);
  va_start (args, fmt);
  vsnprintf (buf, sizeof (buf), fmt, args);
#ifdef HAVE_HTTPS
#ifdef HAVE_OPENSSL
  if (do_ssl)
    ret = BIO_printf ((BIO *) fp, "%s", buf);
  else
#elif defined(HAVE_MATRIXSSL)
  if (do_ssl)
    ret = matrixssl_printf (fp, "%s", buf);
  else
#endif
#endif
    ret = fprintf (fp, "%s", buf);
  va_end (args);
  //free(buf);
  return ret;
}

int
websWrite (webs_t wp, char *fmt, ...)
{
  va_list args;
  char buf[1024];
  int ret;
  FILE *fp = wp;
  if (!wp || !fmt)
    return -1;
  //buf = (char*)malloc(1024);
  va_start (args, fmt);
  vsnprintf (buf, sizeof (buf), fmt, args);
#ifdef HAVE_HTTPS
#ifdef HAVE_OPENSSL
  if (do_ssl)
    ret = BIO_printf ((BIO *) fp, "%s", buf);
  else
#elif defined(HAVE_MATRIXSSL)
  if (do_ssl)
    ret = matrixssl_printf (fp, "%s", buf);
  else
#endif
#endif
    ret = fprintf (fp, "%s", buf);
  va_end (args);
  wfflush (wp);
  return ret;
}

size_t
wfwrite (char *buf, int size, int n, FILE * fp)
{
#ifdef HAVE_HTTPS
#ifdef HAVE_OPENSSL
  if (do_ssl)
    return BIO_write ((BIO *) fp, buf, n * size);
  else
#elif defined(HAVE_MATRIXSSL)
  if (do_ssl)
    return matrixssl_write (fp, buf, n * size);
  else
#endif
#endif
    return fwrite (buf, size, n, fp);
}

size_t
wfread (char *buf, int size, int n, FILE * fp)
{
#ifdef HAVE_HTTPS
#ifdef HAVE_OPENSSL
  if (do_ssl)
    return BIO_read ((BIO *) fp, buf, n * size);
  else
#elif defined(HAVE_MATRIXSSL)
  if (do_ssl)
    {
      //do it in chains
      int cnt = (size * n) / 0x4000;
      int i;
      int len = 0;
      for (i = 0; i < cnt; i++)
	{
	  len += matrixssl_read (fp, buf, 0x4000);
	  *buf += 0x4000;
	}
      len += matrixssl_read (fp, buf, (size * n) % 0x4000);

      return len;
    }
  else
#endif
#endif
    return fread (buf, size, n, fp);
}

int
wfflush (FILE * fp)
{
#ifdef HAVE_HTTPS
#ifdef HAVE_OPENSSL
  if (do_ssl)
    {
      BIO_flush ((BIO *) fp);
      return 1;
    }
  else
#elif defined(HAVE_MATRIXSSL)
  if (do_ssl)
    return matrixssl_flush (fp);
  else
#endif
#endif
    return fflush (fp);
}

int
wfclose (FILE * fp)
{
#ifdef HAVE_HTTPS
#ifdef HAVE_OPENSSL
  if (do_ssl)
    {
      BIO_free_all ((BIO *) fp);
      return 1;
    }
  else
#elif defined(HAVE_MATRIXSSL)
  if (do_ssl)
    return matrixssl_free_session (fp);
  else
#endif
#endif
    return fclose (fp);
}

#ifdef DEBUG_CIPHER
void
check_cipher (void)
{
  STACK_OF (SSL_CIPHER) * sk;
  char buf[512];
  BIO *STDout = NULL;
  int i;
  static BIO *bio_stdout = NULL;
  X509 *peer;
  static BIO *bio_s_out = NULL;
  SSL_CIPHER *ciph;

  if (set_ciphers)
    {
      /* Set supported cipher lists */
      SSL_set_cipher_list (ssl, set_ciphers);
    }
  if (get_ciphers)
    {
      /* Show supported cipher lists */
      sk = SSL_get_ciphers (ssl);

      for (i = 0; i < sk_SSL_CIPHER_num (sk); i++)
	{
	  BIO_puts (STDout,
		    SSL_CIPHER_description (sk_SSL_CIPHER_value (sk, i), buf,
					    512));
	  printf ("%d: %s", i, buf);
	}
      if (STDout != NULL)
	BIO_free_all (STDout);
    }
  peer = SSL_get_peer_certificate (ssl);
  if (peer != NULL)
    {
      BIO_printf (bio_s_out, "Client certificate\n");
      PEM_write_bio_X509 (bio_s_out, peer);
      X509_NAME_oneline (X509_get_subject_name (peer), buf, sizeof buf);
      BIO_printf (bio_s_out, "subject=%s\n", buf);
      X509_NAME_oneline (X509_get_issuer_name (peer), buf, sizeof buf);
      BIO_printf (bio_s_out, "issuer=%s\n", buf);
      X509_free (peer);
    }

  if (SSL_get_shared_ciphers (ssl, buf, sizeof buf) != NULL)
    BIO_printf (bio_s_out, "Shared ciphers:%s\n", buf);

  bio_stdout = BIO_new_fp (stdout, BIO_NOCLOSE);
  ciph = SSL_get_current_cipher (ssl);
  BIO_printf (bio_stdout, "%s%s, cipher %s %s\n",
	      "",
	      SSL_get_version (ssl),
	      SSL_CIPHER_get_version (ciph), SSL_CIPHER_get_name (ciph));
}
#endif



char *
websGetVar (webs_t wp, char *var, char *d)
{
char *ret = get_cgi (var) ? : d;
return ret;
}

/* #UnihanBU6#,Ailsa_Cai, 20090310,add entrance for hidden pages  */

static int auth_hidden_check (char *dirname, char *authorization)
{
    char authinfo[500];
    char* authUser;
    char* authpass;
    int l;
    memset(authinfo,0,sizeof(authinfo));

    if ( !authorization || strncmp( authorization, "Basic ", 6 ) != 0 ) 
    {
        return 0;
    }
    l = b64_decode( &(authorization[6]), (unsigned char*)authinfo, sizeof(authinfo) );
    authinfo[l] = '\0';

   /* Split into user and password. */
    authUser = strtok(authinfo, ":" );
    authpass = strtok( NULL, ":");
    if ( authUser == (char*) 0  || authpass == (char*) 0 )
    {
        return 0;
    }
	
    t_s *f_p=find_session(s_head,c_addr);

    if(f_p->interface != SYS_DFLT_HTTP_USER_IP_STACK_ID_LOCAL ) //check interface
    {
        return -2;   // not lan
    }
/* #UnihanBU6#, Eating Chen, 2009/12/02, System_ShmDb_RetrieveAccess() function 
 * shall be called once in each process/task 
 */
#if 0
    if(System_ShmDb_RetrieveAccess() == NOK)
    {
        fprintf(stderr,"%s","System Shmdb Retrieve Access Failed");
        return 0;
    }
#endif
    if(NvramDb_RetrieveAccess() == NOK)
    {
        fprintf(stderr,"%s","NvramDb Retrieve Access Failed");
        return 0;
    }
   
    char username[SYS_DFLT_HTTP_HIDDEN_NAME_SIZE];	
    char password[SYS_DFLT_HTTP_HIDDEN_PASS_SIZE];   
    memset(username,0,sizeof(username));
    memset(password,0,sizeof(password));
	
    if(System_ShmDb_GetHttpHiddenUserName(username) == NOK 
        || System_ShmDb_GetHttpHiddenPassword(password) == NOK )
    {
        printf("Failed to access System DB!\n" );
        return 0;        
    }
	
    if ( strcmp( authUser, username ) == 0 && strcmp( authpass, password ) == 0 )
        return AuthLevelUser;
	
    return AuthLevelUnauthorized; 
}
/* #UnihanBU6#,Ailsa_Cai, 20090310 , End */
