/*
 * CGI helper functions
 *
 * Copyright 2003, Broadcom Corporation
 * All Rights Reserved.                
 *                                     
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Broadcom Corporation;   
 * the contents of this file may not be disclosed to third parties, copied
 * or duplicated in any form, in whole or in part, without the prior      
 * written permission of Broadcom Corporation.                            
 *
 * $Id: provision.c,v 1.1.1.1 2010/06/30 06:49:33 eating Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "sys_types.h"

#include "httpd.h"
#include "docsis_db.h"
#include "pacm_prov_db.h"
#include "vendor_common_utils.h"

int PROVISION_Handle(int eid, webs_t wp, int argc, char_t **argv);
int PROVISION_get_DocsisDssacn(webs_t wp,char_t*name);
int PROVISION_get_DocsisRang(webs_t wp,char_t*name);
int PROVISION_get_CmDhcp(webs_t wp,char_t*name);
int PROVISION_get_CmDL(webs_t wp,char_t*name);
int PROVISION_get_CmReg(webs_t wp,char_t*name);
int PROVISION_get_MtaDhcp(webs_t wp,char_t *name);
int PROVISION_get_MtaSecurity(webs_t wp,char_t *name);
int PROVISION_get_MtaDL(webs_t wp,char_t *name);
int PROVISION_get_MtaReg(webs_t wp,char_t *name);
int PROVISION_get_MtaState(webs_t wp,char_t *name);
int  PROVISION_get_ProvisionMethod(webs_t wp,char_t *name);

struct cgi_handler PROVISION_handlers[]={
    {"docsis_dsscan",PROVISION_get_DocsisDssacn},
    {"docsis_range",PROVISION_get_DocsisRang},
    {"cm_dhcp",PROVISION_get_CmDhcp},
    {"cm_dl", PROVISION_get_CmDL},
    {"cm_reg",PROVISION_get_CmReg},
    {"mta_dhcp",PROVISION_get_MtaDhcp},
    {"mta_security",PROVISION_get_MtaSecurity},
    {"mta_dl",PROVISION_get_MtaDL},
    {"mta_reg",PROVISION_get_MtaReg},
    {"mta_prov",PROVISION_get_MtaState},
    {"prov_method",PROVISION_get_ProvisionMethod},
    {NULL,NULL}
};
char FlowType[][8]=
{
    "Basic",
    "Hybrid",
    "Secure",
    "Other"
};
char MtaMsg[][64]=
{
    "Complete",
    "In Progress",
    "Fail Config File Error",
    "Pass With Warnings",
    "Pass With Incomplete Parsing",
    "Fail Internal Error",
    "Fail Other Reason"
};

int PROVISION_Handle(int eid, webs_t wp, int argc, char_t **argv)
{
    char_t*name;
    struct cgi_handler *handler;

    if(DocsisParamsDb_RetrieveAccess() == NOK )
    {
        websWrite(wp, "unknown");
        return -1;
    }
    if(PACM_Prov_DbRetrieveAccess() == NOK )
    {
        websWrite(wp, "unknown");
        return -1;
    }
	
    if(Common_Utils_init() == NOK )
    {
        websWrite(wp, "unknown");
        return -1;
    }
    if(ejArgs(argc,argv,"%s",&name)<1)
    {
        websError(wp,400,"Insufficient args\n");
        return -1;
    }

    for(handler=&PROVISION_handlers[0];handler->pattern;handler++)
    {
        if(strncmp(handler->pattern,name,strlen(handler->pattern))==0)
        handler->output(wp,name);
    }
    return 0;
}


int PROVISION_get_DocsisDssacn(webs_t wp,char_t*name)
{
    static CmMacParams_t cmMacPar;
	
    if (Cm_DocsisDb_GetCmMacParams(&cmMacPar) != OK)
    {	
        printf("Can't get CM status\n");
        websWrite(wp,"%s","unknow");
        return -1;
    }
    else
    {
        switch ( cmMacPar.macStatus )
        {
            case DOCSIS_CM_STAT_PARAMS_OTHER:
            case DOCSIS_CM_STAT_PARAMS_NOT_READY:
                websWrite(wp,"------");
                break;
				
            case DOCSIS_CM_STAT_PARAMS_NOT_SYNCHRONIZED:
                websWrite(wp,"In Progress");
                break;
				
            case DOCSIS_CM_STAT_PARAMS_PHY_SYNCHRONIZED:
            case DOCSIS_CM_STAT_PARAMS_DS_TOPOLOGY_RESOLUTION_IN_PROGRESS:
            case DOCSIS_CM_STAT_PARAMS_US_PARAMETERS_ACQUIRED:	
            case DOCSIS_CM_STAT_PARAMS_RANGING_IN_PROGRESS:			
            case DOCSIS_CM_STAT_PARAMS_RANGING_COMPLETE:	
            case DOCSIS_CM_STAT_PARAMS_EAE_IN_PROGRESS:
            case DOCSIS_CM_STAT_PARAMS_DHCPV4_IN_PROGRESS:
            case DOCSIS_CM_STAT_PARAMS_DHCPV6_IN_PROGRESS:
            case DOCSIS_CM_STAT_PARAMS_DHCPV4_COMPLETE:
            case DOCSIS_CM_STAT_PARAMS_DHCPV6_COMPLETE:
            case DOCSIS_CM_STAT_PARAMS_TOD_ESTABLISHED:
            case DOCSIS_CM_STAT_PARAMS_SECURITY_ESTABLISHED:
            case DOCSIS_CM_STAT_PARAMS_CONFIG_FILE_DOWNLOAD_COMPLETE:
            case DOCSIS_CM_STAT_PARAMS_REGISTRATION_IN_PROGRESS:
            case DOCSIS_CM_STAT_PARAMS_REGISTRATION_COMPLETE:
            case DOCSIS_CM_STAT_PARAMS_ACCESS_DENIED:
            case DOCSIS_CM_STAT_PARAMS_OPERATIONAL:
                websWrite(wp,"Complete");
                break;
				
            default:
                websWrite(wp,"------");
                break;
        }
    }
    return 0;
}

int PROVISION_get_DocsisRang(webs_t wp,char_t*name)
{
    static CmMacParams_t cmMacPar;
	
    if (Cm_DocsisDb_GetCmMacParams(&cmMacPar) != OK)
    {	
        printf("Can't get CM status\n");
        websWrite(wp,"%s","unknow");
        return -1;
    }
    else
    {
        switch ( cmMacPar.macStatus )
        {
            case DOCSIS_CM_STAT_PARAMS_OTHER:
            case DOCSIS_CM_STAT_PARAMS_NOT_READY:
            case DOCSIS_CM_STAT_PARAMS_NOT_SYNCHRONIZED:
            case DOCSIS_CM_STAT_PARAMS_PHY_SYNCHRONIZED:
            case DOCSIS_CM_STAT_PARAMS_DS_TOPOLOGY_RESOLUTION_IN_PROGRESS:
            case DOCSIS_CM_STAT_PARAMS_US_PARAMETERS_ACQUIRED:
                websWrite(wp,"------");
                break;
				
            case DOCSIS_CM_STAT_PARAMS_RANGING_IN_PROGRESS:
                websWrite(wp,"In Progress");
                break;
				
            case DOCSIS_CM_STAT_PARAMS_RANGING_COMPLETE:	
            case DOCSIS_CM_STAT_PARAMS_EAE_IN_PROGRESS:
            case DOCSIS_CM_STAT_PARAMS_DHCPV4_IN_PROGRESS:
            case DOCSIS_CM_STAT_PARAMS_DHCPV6_IN_PROGRESS:
            case DOCSIS_CM_STAT_PARAMS_DHCPV4_COMPLETE:
            case DOCSIS_CM_STAT_PARAMS_DHCPV6_COMPLETE:
            case DOCSIS_CM_STAT_PARAMS_TOD_ESTABLISHED:
            case DOCSIS_CM_STAT_PARAMS_SECURITY_ESTABLISHED:
            case DOCSIS_CM_STAT_PARAMS_CONFIG_FILE_DOWNLOAD_COMPLETE:
            case DOCSIS_CM_STAT_PARAMS_REGISTRATION_IN_PROGRESS:
            case DOCSIS_CM_STAT_PARAMS_REGISTRATION_COMPLETE:
            case DOCSIS_CM_STAT_PARAMS_ACCESS_DENIED:
            case DOCSIS_CM_STAT_PARAMS_OPERATIONAL:
                websWrite(wp,"Complete");
                break;
				
            default:
                websWrite(wp,"------");
                break;
        }
    }
    return 0;
}
int PROVISION_get_CmDhcp(webs_t wp,char_t*name)
{
    static CmMacParams_t cmMacPar;
	
    if (Cm_DocsisDb_GetCmMacParams(&cmMacPar) != OK)
    {	
        printf("Can't get CM status\n");
        websWrite(wp,"%s","unknow");
        return -1;
    }
    else
    {
        switch ( cmMacPar.macStatus )
        {
            case DOCSIS_CM_STAT_PARAMS_OTHER:
            case DOCSIS_CM_STAT_PARAMS_NOT_READY:
            case DOCSIS_CM_STAT_PARAMS_NOT_SYNCHRONIZED:
            case DOCSIS_CM_STAT_PARAMS_PHY_SYNCHRONIZED:
            case DOCSIS_CM_STAT_PARAMS_DS_TOPOLOGY_RESOLUTION_IN_PROGRESS:
            case DOCSIS_CM_STAT_PARAMS_US_PARAMETERS_ACQUIRED:
            case DOCSIS_CM_STAT_PARAMS_RANGING_IN_PROGRESS:
            case DOCSIS_CM_STAT_PARAMS_RANGING_COMPLETE:	
            case DOCSIS_CM_STAT_PARAMS_EAE_IN_PROGRESS:
                websWrite(wp,"------");
                break;
				
            case DOCSIS_CM_STAT_PARAMS_DHCPV4_IN_PROGRESS:
            case DOCSIS_CM_STAT_PARAMS_DHCPV6_IN_PROGRESS:
                websWrite(wp,"In Progress");
                break;
				
            case DOCSIS_CM_STAT_PARAMS_DHCPV4_COMPLETE:
            case DOCSIS_CM_STAT_PARAMS_DHCPV6_COMPLETE:
            case DOCSIS_CM_STAT_PARAMS_TOD_ESTABLISHED:
            case DOCSIS_CM_STAT_PARAMS_SECURITY_ESTABLISHED:
            case DOCSIS_CM_STAT_PARAMS_CONFIG_FILE_DOWNLOAD_COMPLETE:
            case DOCSIS_CM_STAT_PARAMS_REGISTRATION_IN_PROGRESS:
            case DOCSIS_CM_STAT_PARAMS_REGISTRATION_COMPLETE:
            case DOCSIS_CM_STAT_PARAMS_ACCESS_DENIED:
            case DOCSIS_CM_STAT_PARAMS_OPERATIONAL:
                websWrite(wp,"Complete");
                break;
				
            default:
                websWrite(wp,"------");
                break;
        }
    }
    return 0;
}

int PROVISION_get_CmDL(webs_t wp,char_t*name)
{
    static CmMacParams_t cmMacPar;
	
    if (Cm_DocsisDb_GetCmMacParams(&cmMacPar) != OK)
    {	
        printf("Can't get CM status\n");
        websWrite(wp,"%s","unknow");
        return -1;
    }
    else
    {
        switch ( cmMacPar.macStatus )
        {
            case DOCSIS_CM_STAT_PARAMS_OTHER:
            case DOCSIS_CM_STAT_PARAMS_NOT_READY:
            case DOCSIS_CM_STAT_PARAMS_NOT_SYNCHRONIZED:
            case DOCSIS_CM_STAT_PARAMS_PHY_SYNCHRONIZED:
            case DOCSIS_CM_STAT_PARAMS_DS_TOPOLOGY_RESOLUTION_IN_PROGRESS:
            case DOCSIS_CM_STAT_PARAMS_US_PARAMETERS_ACQUIRED:
            case DOCSIS_CM_STAT_PARAMS_RANGING_IN_PROGRESS:
            case DOCSIS_CM_STAT_PARAMS_RANGING_COMPLETE:	
            case DOCSIS_CM_STAT_PARAMS_EAE_IN_PROGRESS:			
            case DOCSIS_CM_STAT_PARAMS_DHCPV4_IN_PROGRESS:
            case DOCSIS_CM_STAT_PARAMS_DHCPV6_IN_PROGRESS:				
            case DOCSIS_CM_STAT_PARAMS_DHCPV4_COMPLETE:
            case DOCSIS_CM_STAT_PARAMS_DHCPV6_COMPLETE:
                websWrite(wp,"------");
                break;
            case DOCSIS_CM_STAT_PARAMS_TOD_ESTABLISHED:
            case DOCSIS_CM_STAT_PARAMS_SECURITY_ESTABLISHED:
                websWrite(wp,"In Progress");
                break;
            case DOCSIS_CM_STAT_PARAMS_CONFIG_FILE_DOWNLOAD_COMPLETE:
            case DOCSIS_CM_STAT_PARAMS_REGISTRATION_IN_PROGRESS:
            case DOCSIS_CM_STAT_PARAMS_REGISTRATION_COMPLETE:
            case DOCSIS_CM_STAT_PARAMS_ACCESS_DENIED:
            case DOCSIS_CM_STAT_PARAMS_OPERATIONAL:
                websWrite(wp,"Complete");
                break;
				
            default:
                websWrite(wp,"------");
                break;
        }
    }
    return 0;
}

int PROVISION_get_CmReg(webs_t wp,char_t*name)
{
    static CmMacParams_t cmMacPar;
	
    if (Cm_DocsisDb_GetCmMacParams(&cmMacPar) != OK)
    {	
        printf("Can't get CM status\n");
        websWrite(wp,"%s","unknow");
        return -1;
    }
    else
    {
        switch ( cmMacPar.macStatus )
        {
            case DOCSIS_CM_STAT_PARAMS_OTHER:
            case DOCSIS_CM_STAT_PARAMS_NOT_READY:
            case DOCSIS_CM_STAT_PARAMS_NOT_SYNCHRONIZED:
            case DOCSIS_CM_STAT_PARAMS_PHY_SYNCHRONIZED:
            case DOCSIS_CM_STAT_PARAMS_DS_TOPOLOGY_RESOLUTION_IN_PROGRESS:
            case DOCSIS_CM_STAT_PARAMS_US_PARAMETERS_ACQUIRED:
            case DOCSIS_CM_STAT_PARAMS_RANGING_IN_PROGRESS:
            case DOCSIS_CM_STAT_PARAMS_RANGING_COMPLETE:	
            case DOCSIS_CM_STAT_PARAMS_EAE_IN_PROGRESS: 		
            case DOCSIS_CM_STAT_PARAMS_DHCPV4_IN_PROGRESS:
            case DOCSIS_CM_STAT_PARAMS_DHCPV6_IN_PROGRESS:				
            case DOCSIS_CM_STAT_PARAMS_DHCPV4_COMPLETE:
            case DOCSIS_CM_STAT_PARAMS_DHCPV6_COMPLETE:
            case DOCSIS_CM_STAT_PARAMS_TOD_ESTABLISHED:
            case DOCSIS_CM_STAT_PARAMS_SECURITY_ESTABLISHED:
            case DOCSIS_CM_STAT_PARAMS_CONFIG_FILE_DOWNLOAD_COMPLETE:
                websWrite(wp,"------");
                break;
            case DOCSIS_CM_STAT_PARAMS_REGISTRATION_IN_PROGRESS:
                websWrite(wp,"In Progress");
                break;
            case DOCSIS_CM_STAT_PARAMS_REGISTRATION_COMPLETE:
            case DOCSIS_CM_STAT_PARAMS_ACCESS_DENIED:
            case DOCSIS_CM_STAT_PARAMS_OPERATIONAL:
                websWrite(wp,"Complete");
                break;
				
            default:
                websWrite(wp,"------");
                break;
        }
    }
    return 0;
}

int PROVISION_get_MtaDhcp(webs_t wp,char_t *name)
{
    PktcMTAProvState_e MTAProvState;
    if (Common_Utils_GetPktcMTAProvState(&MTAProvState) == OK)
    {
        if ( MTAProvState == PACM_PROV_STATE_DHCP_DISCOVER)
        {
            websWrite(wp,"In Progress");
        }
        else if ( MTAProvState >= PACM_PROV_STATE_DHCP_COMPLETE)
        {
            websWrite(wp,"Complete");
        }
        else
        {
            websWrite(wp,"------");
        }
    }
    else
    {
        websWrite(wp,"unknow");
    }
    return 0;
}
int PROVISION_get_MtaSecurity(webs_t wp,char_t *name)
{
    PktcMTAProvState_e MTAProvState;
    PACM_Prov_Flow_e provFlow;
    if (Common_Utils_GetPktcMTAProvState(&MTAProvState) == OK)
    {
        if ( MTAProvState <= PACM_PROV_STATE_DHCP_COMPLETE)
        {
            websWrite(wp,"------");
            return 0;
        }

        if (PACM_Prov_GetProvFlow(&provFlow) == STATUS_OK)
        {
            if ((provFlow ==PACM_PROV_FLOW_BASIC) ||(provFlow ==PACM_PROV_FLOW_HYBRID) )
            {
                websWrite(wp,"Disable");
                return 0;
            }
            else
            {
                if ( MTAProvState == PACM_PROV_STATE_SECURUTY_START)
                    websWrite(wp,"In Progress");
				
                else if ( MTAProvState >= PACM_PROV_STATE_SECURUTY_ESTABLISHED)
                    websWrite(wp,"Complete");
            }
        }
    }
    else
    {
        websWrite(wp,"unknow");
    }
    return 0;
}
int PROVISION_get_MtaDL(webs_t wp,char_t *name)
{
    PktcMTAProvState_e MTAProvState;
	
    if (Common_Utils_GetPktcMTAProvState(&MTAProvState) == OK)
    {
        if ( MTAProvState == PACM_PROV_STATE_TFTP_START)
        {
            websWrite(wp,"In Progress");
        }
        else if ( MTAProvState >= PACM_PROV_STATE_TFTP_COMPLETE)
        {
            websWrite(wp,"Complete");
        }
        else
        {
            websWrite(wp,"------");
        }
    }
    else
    {
        websWrite(wp,"unknow");
    }
    return 0;
}

int PROVISION_get_MtaReg(webs_t wp,char_t *name)
{	
    char line1Status[32];
    char line2Status[32];
    char  buffer[64];
    char tmp[32];
    memset(line1Status,0,sizeof(line1Status));
    memset(line2Status,0,sizeof(line2Status));
    memset(tmp,0,sizeof(tmp));
    memset(buffer,0,sizeof(buffer));
	
    if(Common_Utils_GetEndptStatus(0,line1Status)==STATUS_OK)
    {
        sprintf(buffer,"Line1:%s",line1Status);
    }
    else
    {
        sprintf(buffer,"Line1:unknow");
    }
	
    if(Common_Utils_GetEndptStatus(1,line2Status)==STATUS_OK)
    {
        sprintf(tmp,"/Line2:%s",line2Status);
        strcat(buffer,tmp);
    }
    else
    {
        strcat(buffer,"/Line2:unknow");
    }
    websWrite(wp,"%s",buffer);
    return 0;	
}


int PROVISION_get_MtaState(webs_t wp,char_t *name)
{
    PacmProvCompletionState_e Mtastate;
    PktcMTAProvState_e MTAProvState;
	
    if (Common_Utils_GetPktcMTAProvState(&MTAProvState) == OK)
    {
        if ( MTAProvState < PACM_PROV_STATE_DHCP_DISCOVER)
        {
            websWrite(wp,"------");
            return 0;
        }
    }
    if (PACM_Prov_GetMtaDevProvisioningState(&Mtastate) == OK)
    {
        websWrite(wp,"%s",MtaMsg[Mtastate-1]);
    }
    else
    {
        websWrite(wp,"unknow");
    }
    return 0;
}

int  PROVISION_get_ProvisionMethod(webs_t wp,char_t *name)
{
    PACM_Prov_Flow_e provFlow;
    PACM_Prov_Flow_e x1x2;
    if (PACM_Prov_GetProvFlow(&provFlow) == STATUS_OK && PACM_Prov_GetProvFlowFlavor(&x1x2) == STATUS_OK)
    {
        websWrite(wp,"%s.%d",FlowType[provFlow],x1x2);
    }
    else
    {
        websWrite(wp,"unknow");
    }   
    return 0;
}
