/*
 * CGI helper functions
 *
 * Copyright 2003, Broadcom Corporation
 * All Rights Reserved.                
 *                                     
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Broadcom Corporation;   
 * the contents of this file may not be disclosed to third parties, copied
 * or duplicated in any form, in whole or in part, without the prior      
 * written permission of Broadcom Corporation.                            
 *
 * $Id: signal.c,v 1.1.1.1 2010/06/30 06:49:33 eating Exp $
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <syslog.h>
#include <arpa/inet.h>
#include <time.h>

#include "httpd.h"

#include "docsis_db.h"
#include "production_db.h"
#include "dbridge_conf.h"
#include "bpi.h"
#include "bpi_db.h"
#include "ds_db.h"
#include "us_db.h"

#include "hal_phy_dnstream.h"
#include "hal_phy_upstream.h"
#include "hal_reg_access.h"
#include "hal_phy_regs.h"
#include "ucd_db.h"
#include "hal_db.h"
#include "hal_global.h"

/* #UnihanBU6#, Alisa_Gao, 20080724, Start*/
#include "chdb_main.h"
#include "chdb_rx.h"
#include "nvram_db.h"
#include "icc.h"
/* #UnihanBU6#, Alisa_Gao, 20080724*/

#define RATE_MULTIPLIER     160
#define MAX_REP_STR         128
static float frequencyList[4];

char ucdModStr[][8]=
    {"",
     "QPSK",
    "8QAM",
    "16QAM",
    "32QAM",
    "64QAM",
    "128QAM"
    };

typedef enum {
    eCmDownstreamLockStatus0, /* for Ds0 */
    eCmDownstreamModulation0,
    eCmDownstreamChannelId0,
    eCmDownstreamSymbolRate0,
    eCmDownstreamFrequency0,
    eCmDownstreamPower0,
    eCmDownstreamSnr0,
    eCmDownstreamLockStatus1, /* for Ds1 */
    eCmDownstreamModulation1,
    eCmDownstreamChannelId1,
    eCmDownstreamSymbolRate1,
    eCmDownstreamFrequency1,
    eCmDownstreamPower1,
    eCmDownstreamSnr1,
    eCmDownstreamLockStatus2, /* for Ds2 */
    eCmDownstreamModulation2,
    eCmDownstreamChannelId2,
    eCmDownstreamSymbolRate2,
    eCmDownstreamFrequency2,
    eCmDownstreamPower2,
    eCmDownstreamSnr2,
    eCmDownstreamLockStatus3, /* for Ds3 */
    eCmDownstreamModulation3,
    eCmDownstreamChannelId3,
    eCmDownstreamSymbolRate3,
    eCmDownstreamFrequency3,
    eCmDownstreamPower3,
    eCmDownstreamSnr3,
    eCmUpstreamLockStatus0, /* for Us0 */
    eCmUpstreamModulation0,
    eCmUpstreamChannelId0,
    eCmUpstreamSymbolRate0,
    eCmUpstreamFrequency0,
    eCmUpstreamPower0,
    eCmUpstreamLockStatus1, /* for Us1 */
    eCmUpstreamModulation1,
    eCmUpstreamChannelId1,
    eCmUpstreamSymbolRate1,
    eCmUpstreamFrequency1,
    eCmUpstreamPower1,
    eCmUpstreamLockStatus2, /* for Us2 */
    eCmUpstreamModulation2,
    eCmUpstreamChannelId2,
    eCmUpstreamSymbolRate2,
    eCmUpstreamFrequency2,
    eCmUpstreamPower2,
    eCmUpstreamLockStatus3, /* for Us3 */
    eCmUpstreamModulation3,
    eCmUpstreamChannelId3,
    eCmUpstreamSymbolRate3,
    eCmUpstreamFrequency3,
    eCmUpstreamPower3,
    eSystemTime,
    eNumOfConnectionWebStr
}eWebStr;

static char emptyStr[eNumOfConnectionWebStr][MAX_REP_STR];


int SIGNAL_Handle(int eid, webs_t wp, int argc, char_t **argv);

int SIGNAL_get_DsFrequeny(webs_t wp, char_t *name);
int SIGNAL_get_DsPower(webs_t wp, char_t *name);
int SIGNAL_get_DsSNR(webs_t wp, char_t *name);
int SIGNAL_get_DsModulation(webs_t wp, char_t *name);
int SIGNAL_get_UsFrequeny(webs_t wp, char_t *name);
int SIGNAL_get_UsPower(webs_t wp, char_t *name);
int SIGNAL_get_UsDataRate(webs_t wp, char_t *name);
int SIGNAL_get_UsModulation(webs_t wp, char_t *name);

/* #UnihanBU6#, Alisa_Gao, 20090220,Add for MFG test,when access UI,Ds/Us Freq should display*/
int SIGNAL_get_DsFrequenyTest(webs_t wp, char_t *name);
int SIGNAL_get_UsFrequenyTest(webs_t wp, char_t *name);
/* #UnihanBU6#, Alisa_Gao, 20090220,Add End*/

/* #UnihanBU6#,Chenwin_Qiu, 20090316 Add for get Hidden page DsFreqency */
int SIGNAL_get_HidDsFrequeny(webs_t wp, char_t *name);
/* #UnihanBU6#, Chenwin_Qiu, 20090316, Add End*/

int get_UsChannelParms(int channel);
int get_DsChannelParms(int channel);
struct cgi_handler SIGNAL_handlers[]={
    {"ds0_freq",SIGNAL_get_DsFrequeny},
    {"ds0_power",SIGNAL_get_DsPower},
    {"ds0_snr",SIGNAL_get_DsSNR},
    {"ds0_modulation",SIGNAL_get_DsModulation},
    {"ds1_freq",SIGNAL_get_DsFrequeny},
    {"ds1_power",SIGNAL_get_DsPower},
    {"ds1_snr",SIGNAL_get_DsSNR},
    {"ds1_modulation",SIGNAL_get_DsModulation},
    {"ds2_freq",SIGNAL_get_DsFrequeny},
    {"ds2_power",SIGNAL_get_DsPower},
    {"ds2_snr",SIGNAL_get_DsSNR},
    {"ds2_modulation",SIGNAL_get_DsModulation},
    {"ds3_freq",SIGNAL_get_DsFrequeny},
    {"ds3_power",SIGNAL_get_DsPower},
    {"ds3_snr",SIGNAL_get_DsSNR},
    {"ds3_modulation",SIGNAL_get_DsModulation},
    {"us0_freq",SIGNAL_get_UsFrequeny},
    {"us0_power",SIGNAL_get_UsPower},
    {"us0_datarate",SIGNAL_get_UsDataRate},
    {"us0_modulation",SIGNAL_get_UsModulation},
    {"us1_freq",SIGNAL_get_UsFrequeny},
    {"us1_power",SIGNAL_get_UsPower},
    {"us1_datarate",SIGNAL_get_UsDataRate},
    {"us1_modulation",SIGNAL_get_UsModulation},
    {"us2_freq",SIGNAL_get_UsFrequeny},
    {"us2_power",SIGNAL_get_UsPower},
    {"us2_datarate",SIGNAL_get_UsDataRate},
    {"us2_modulation",SIGNAL_get_UsModulation},
    {"us3_freq",SIGNAL_get_UsFrequeny},
    {"us3_power",SIGNAL_get_UsPower},
    {"us3_datarate",SIGNAL_get_UsDataRate},
    {"us3_modulation",SIGNAL_get_UsModulation},
    /* #UnihanBU6#, Alisa_Gao, 20090220,Add for MFG test,when access UI,Ds/Us Freq should display*/
    {"ds0_testfreq",SIGNAL_get_DsFrequenyTest},
    {"ds1_testfreq",SIGNAL_get_DsFrequenyTest},
    {"ds2_testfreq",SIGNAL_get_DsFrequenyTest},
    {"ds3_testfreq",SIGNAL_get_DsFrequenyTest},
    {"us0_testfreq",SIGNAL_get_UsFrequenyTest},
    {"us1_testfreq",SIGNAL_get_UsFrequenyTest},
    {"us2_testfreq",SIGNAL_get_UsFrequenyTest},
    {"us3_testfreq",SIGNAL_get_UsFrequenyTest},
    /* #UnihanBU6#, Alisa_Gao, 20090220,Add End*/
	
    /* #UnihanBU6#,Chenwin_Qiu, 20090316 Add for get Hidden page DsFreqency */
    {"ds_hidfreq",SIGNAL_get_HidDsFrequeny},	
    /* #UnihanBU6#, Chenwin_Qiu, 20090316,Add End*/
    {NULL,NULL}
};

int SIGNAL_Handle(int eid, webs_t wp, int argc, char_t **argv)
{
	char *name;
	struct cgi_handler *handler;

	 /* Initial all DB connections*/
    if ( Ucd_DbRetrieveAccess() == NOK )
    {
        websWrite(wp, "unknown");
        return -1;
    }
    if ( ProductionDb_RetrieveAccess() == NOK )
    {
        websWrite(wp, "unknown");
        return -1;
    }
    if ( NvramDb_RetrieveAccess() == NOK )
    {
        websWrite(wp, "unknown");
        return -1;
    }
    if ( DocsisParamsDb_RetrieveAccess() == NOK )
    {
        websWrite(wp, "unknown");
        return -1;
    }
    if ( UpstreamParamsDb_RetrieveAccess() == NOK )
    {
        websWrite(wp, "unknown");
        return -1;
    }
   if(CHDB_Main_RetrieveAccess() == NOK)
   {
   	websWrite(wp,"unknown");
	return -1;
   }
    if ( HalDb_RetrieveAccess() == NOK )
    {
        websWrite(wp, "unknown");
        return -1;
    }
    if ( ICC_init() == NOK )
    {
        websWrite(wp, "unknown");
        return -1;
    }
    if ( HAL_GlobalRegistration(1) == NOK )
    {
        websWrite(wp, "unknown");
        return -1;
    }
    if(ejArgs(argc,argv,"%s",&name)<1)
    {
        websError(wp,400,"Insufficient args\n");
        return -1;
    }
    
    for(handler=&SIGNAL_handlers[0];handler->pattern;handler++)
    {
    	if(strncmp(handler->pattern,name,strlen(handler->pattern))==0)
    	{	
    		handler->output(wp,name);
    	}
    }    
	return 0;
}

int get_channel_index(char *name, char *prex)
{
    char num[4];
    memset( num, 0, 4 );
    strncpy(num, name+strlen(prex), 1);
    return atoi(num);
}

/* #UnihanBU6#, Rainy, 20080618, if not locked, show 0 MHz*/
int SIGNAL_get_DsFrequeny(webs_t wp, char_t *name)
{
    int channel=0;
    int level=0;
    channel=get_channel_index(name,"ds");
    get_DsChannelParms(channel);
    HAL_DsCalGetRFFrequency(frequencyList);
    level = get_ulevel();
    /* #UnihanBU6#, Chenwin_Qiu, 20090407, Fix display frequency format. */
    int tmp[channel];
 /* #UnihanBU6#, Rainy_yuan, 20090525, for TD#506, TD#614.All the frequencies, power range should be shown on the BASIC webpage */
#if 0	
    if (level < AuthLevelAdmin)
    {
        websWrite(wp, "%s", "------");
        return 0;
    }
#endif
    if (strcmp(emptyStr[eCmDownstreamLockStatus0+channel*7],"Locked"))
    {
        websWrite(wp, "%s", "0 MHz");
    }
    else
    {
        /* #UnihanBU6#, Chenwin_Qiu, 20090407, Fix display frequency format, Start */
        tmp[channel]=ceil(frequencyList[channel]-0.5);
	 websWrite(wp, "%d MHz",tmp[channel]);
	 /* #UnihanBU6#, Chenwin_Qiu, 20090407, Fix display frequency format. End */
    }
    return 0;
}

/* #UnihanBU6#, Alisa_Gao, 20090220,Add for MFG test,when access UI,Ds/Us Freq should display*/
int SIGNAL_get_DsFrequenyTest(webs_t wp, char_t *name)
{
    int channel=0;
    int level=0;
    channel=get_channel_index(name,"ds");
    get_DsChannelParms(channel);
    HAL_DsCalGetRFFrequency(frequencyList);
    if (strcmp(emptyStr[eCmDownstreamLockStatus0+channel*7],"Locked"))
    {
        websWrite(wp, "%s", "0 MHz");
    }
    else
    {
        websWrite(wp, "%.3f MHz", frequencyList[channel]);
    }
    return 0;
}
/* #UnihanBU6#, Alisa_Gao, 20090220,Add End*/

/* #UnihanBU6#,Chenwin_Qiu, 20090313, implement CGI for get DsFreqency, Start*/
int SIGNAL_get_HidDsFrequeny(webs_t wp, char_t *name)
{
    int channel;
    int  channelNUM=4;
	
    for (channel=0;channel<channelNUM;channel++)
    {
        get_DsChannelParms(channel);

        HAL_DsCalGetRFFrequency(frequencyList);	 

	if (strcmp(emptyStr[eCmDownstreamLockStatus0+channel*7],"Locked"))
        {
             websWrite(wp, "<tr align=center><td>%d</td><td>%s</td></tr>", channel,"0 MHz");
	}
        else
        {
             websWrite(wp, "<tr align=center><td>%d</td><td>%.3f MHz</td></tr>",channel, frequencyList[channel]);
        }       
    }
    return 0;
}
/* #UnihanBU6#,Chenwin_Qiu, 20090313, End*/

int SIGNAL_get_DsPower(webs_t wp, char_t *name)
{
    int channel=0;
    channel=get_channel_index(name,"ds");
    get_DsChannelParms(channel);
    websWrite(wp,"%s", emptyStr[eCmDownstreamPower0+channel*7]);  
    return 0;
}
int SIGNAL_get_DsSNR(webs_t wp, char_t *name)
{
    int channel=0;
    channel=get_channel_index(name,"ds");
    get_DsChannelParms(channel);
    websWrite(wp, "%s", emptyStr[eCmDownstreamSnr0+channel*7]);
    return 0;
}
int SIGNAL_get_DsModulation(webs_t wp, char_t *name)
{
    int channel=0;
    channel=get_channel_index(name,"ds");
    get_DsChannelParms(channel);
     websWrite(wp, "%s", emptyStr[eCmDownstreamModulation0+channel*7]);
    return 0;
}
int SIGNAL_get_UsFrequeny(webs_t wp, char_t *name)
{
    int channel=0;
    int level=0;
    channel=get_channel_index(name,"us");
    get_UsChannelParms(channel);
 /* #UnihanBU6#, Rainy_yuan, 20090525, for TD#506, TD#614.All the frequencies, power range should be shown on the BASIC webpage */
#if 0	
    level = get_ulevel();
    if (level < AuthLevelAdmin)
    {
        websWrite(wp, "%s", "------");
        return 0;
    }
#endif
    websWrite(wp, "%s", emptyStr[eCmUpstreamFrequency0+channel*6]);
    return 0;
}

/* #UnihanBU6#, Alisa_Gao, 20090220,Add for MFG test,when access UI,Ds/Us Freq should display*/
int SIGNAL_get_UsFrequenyTest(webs_t wp, char_t *name)
{
    int channel=0;
    int level=0;
    channel=get_channel_index(name,"us");
    get_UsChannelParms(channel);
    websWrite(wp, "%s", emptyStr[eCmUpstreamFrequency0+channel*6]);
    return 0;
}
/* #UnihanBU6#, Alisa_Gao, 20090220,Add End*/

int SIGNAL_get_UsPower(webs_t wp, char_t *name)
{
    int channel=0;
    channel=get_channel_index(name,"us");
    get_UsChannelParms(channel);
    websWrite(wp, "%s", emptyStr[eCmUpstreamPower0+channel*6]);
    return 0;
}
int SIGNAL_get_UsDataRate(webs_t wp, char_t *name)
{
    int channel=0;
    channel=get_channel_index(name,"us");
    get_UsChannelParms(channel);
    websWrite(wp, "%s", emptyStr[eCmUpstreamSymbolRate0+channel*6]);
    return 0;
}

int SIGNAL_get_UsModulation(webs_t wp, char_t *name)
{
    int channel=0;
    channel=get_channel_index(name,"us");
    get_UsChannelParms(channel);
    websWrite(wp, "%s", emptyStr[eCmUpstreamModulation0+channel*6]);
    return 0;
}

Uint8 GetDsChParams(Int dsPort,CHDB_Rx_Channel_t*pDsChannelParamsEntry)
{
    PHY_DS_MODULATION_TYPE_e qamType = 0;
    ANNEX_e annex;
    PHY_DS_INTERLEAVINGL_e interleaverLength;

    if ( !pDsChannelParamsEntry )
        return 0;


    /*Get DS Channel modulation type from Phy*/
    if ( PHY_Single_DS_Get_Qam_Type(dsPort, &qamType) != PHY_DS_OP_OK )
    {
        printf(" PHY_Single_DS_Get_Qam_Type not OK" );
        return 0;
    }
    else
    {
        if ( qamType == PHY_DS_64QAM_MODULATION )
            pDsChannelParamsEntry->modulation = DS_CH_MODULATION_QAM64;
        else if ( qamType == PHY_DS_256QAM_MODULATION )
            pDsChannelParamsEntry->modulation = DS_CH_MODULATION_QAM256;
        else
            pDsChannelParamsEntry->modulation = DS_CH_MODULATION_OTHER;
    }

    /*Get DS Channel Annex type from Phy*/
    if ( PHY_Single_DS_Get_Annex_Type(dsPort, &annex) != PHY_DS_OP_OK )
    {
        printf(" PHY_Single_DS_Get_Annex_Type not OK" );
        return 0;
    }
    else
    {
        if ( annex == PHY_ANNEX_A )
            pDsChannelParamsEntry->annex = DS_CH_ANNEX_ANNEX_A;
        else if ( annex == PHY_ANNEX_B )
            pDsChannelParamsEntry->annex = DS_CH_ANNEX_ANNEX_B;
        else
            pDsChannelParamsEntry->annex = DS_CH_ANNEX_UNKNOWN;
    }

    static float power [ PHY_RECEIVER_COUNT];
    //HAL_DsCalGetCalcPower(power, 1);

    if( dsPort == PHY_RECEIVER_CHANNEL_1 )
    {
        memset( power, 0, sizeof( power ) );
        /* #UnihanBU6#, Allen, 20081021, Porting from VSDK 1.0.0.26 */
        if ( HalDb_DsCalGetReportedPower(power, dsPort) )
        {
            printf(" HalDb_DsCalGetReportedPower not OK" );
        }
    }

    pDsChannelParamsEntry->power= power[dsPort];

    /*Get the interleaver*/

    if ( PHY_Single_DS_Get_Interleaver_Length(dsPort, annex, &interleaverLength) != PHY_DS_OP_OK )
    {
        printf(" PHY_Single_DS_Get_Interleaver_Length not OK" );
        return 0;
    }
    else
    {
        switch ( interleaverLength )
        {
            case DS_INTERLEAVING_8_16:
                pDsChannelParamsEntry->interleaver = DS_CH_INTER_TAP8_INCR16;
                break;
            case DS_INTERLEAVING_16_8:
                pDsChannelParamsEntry->interleaver = DS_CH_INTER_TAP16_INCR8;
                break;
            case DS_INTERLEAVING_32_4:
                pDsChannelParamsEntry->interleaver = DS_CH_INTER_TAP32_INCR4;
                break;
            case DS_INTERLEAVING_64_2:
                pDsChannelParamsEntry->interleaver = DS_CH_INTER_TAP64_INCR2;
                break;
            case DS_INTERLEAVING_128_1_A:
            case DS_INTERLEAVING_128_1_B:
                pDsChannelParamsEntry->interleaver = DS_CH_INTER_TAP128_INCR1;
                break;
            case DS_INTERLEAVING_12_17:
                pDsChannelParamsEntry->interleaver = DS_CH_INTER_TAP12_INCR17;
                break;
            default:
                pDsChannelParamsEntry->interleaver = DS_CH_INTER_UNKNOWN;
                break;

        }
    }

    return 1;

}


int get_DsChannelParms(int channel)
{
	 CHDB_Rx_Channel_t  *pDsChannelParamsEntry=NULL;
	 pDsChannelParamsEntry=( CHDB_Rx_Channel_t *)CHDB_Main_GetDsChannelParamsEntry();

	if ( pDsChannelParamsEntry )
        {
            //HAL_Connect();
                pDsChannelParamsEntry = &pDsChannelParamsEntry[channel];
                Uint8 dcid = 0;
                Docsis_DocsMode_e opMode;
                if(DocsisDb_GetOperationalMode(&opMode) == OK)
                {
                    if(opMode == DOCSIS_MODE_30)
                    {
                        if(DocsisDb_GetPortDcid(channel+1, &dcid) != OK)
                            return 0;
                    }
                    else
                    {
                        if(Ucd_DbGetUcdDsId(channel+1, &dcid) != OK)
                        {
                            printf("Ucd_DbGetUcdDsId not OK");
                            return 0;
                        }
                    }
                    GetDsChParams(channel, pDsChannelParamsEntry);
                
                    uint8 DsLock = 0;
                 if(pDsChannelParamsEntry->annex == DS_CH_ANNEX_ANNEX_A )
                    {							
				if( pDsChannelParamsEntry->modulation == DS_CH_MODULATION_QAM64 || pDsChannelParamsEntry->modulation == DS_CH_MODULATION_QAM256 )
					 sprintf(emptyStr[eCmDownstreamSymbolRate0+channel*7],"6952000 Ksym/sec");
                            else
                                sprintf(emptyStr[eCmDownstreamSymbolRate0+channel*7],"unknown");                    
                        DsLock = PHY_IsFecLocked(channel,1);
                    }
               if( pDsChannelParamsEntry->annex ==DS_CH_ANNEX_ANNEX_B )
                    {
                            if( pDsChannelParamsEntry->modulation== DS_CH_MODULATION_QAM64 )
                                sprintf(emptyStr[eCmDownstreamSymbolRate0+channel*7],"5059641 Ksym/sec");
                            else if( pDsChannelParamsEntry->modulation == DS_CH_MODULATION_QAM256 )
                                sprintf(emptyStr[eCmDownstreamSymbolRate0+channel*7],"5360537 Ksym/sec");
                            else
                                sprintf(emptyStr[eCmDownstreamSymbolRate0+channel*7],"unknown");
                        DsLock = PHY_IsFecLocked(channel,0);
                    }
                
                    if( channel==0 )
                        sprintf(emptyStr[eCmDownstreamLockStatus0+channel*7],"%s",DsLock?"Locked":"In Progress");
                    else
                        sprintf(emptyStr[eCmDownstreamLockStatus0+channel*7],"%s",DsLock?"Locked":"Not Locked");
                    
                    if( DsLock )
                    {
                        sprintf(emptyStr[eCmDownstreamModulation0+channel*7],"%s", (pDsChannelParamsEntry->modulation==DS_CH_MODULATION_QAM256)?"256 QAM":
                                                                  ((pDsChannelParamsEntry->modulation==DS_CH_MODULATION_QAM64)?"64 QAM":
                                                                  ((pDsChannelParamsEntry->modulation==DS_CH_MODULATION_OTHER)?"Other":"unknown")));
						
                        sprintf(emptyStr[eCmDownstreamChannelId0+channel*7],"%d",channel);	

                        sprintf(emptyStr[eCmDownstreamPower0+channel*7],"%3.2f dBmV",pDsChannelParamsEntry->power);

                        PHY_DS_MODULATION_TYPE_e qamType = 0;
                        float mseVal=0.0;
                        PHY_Single_DS_Get_Qam_Type(channel, &qamType);
                        PHY_Single_DS_Get_Mse(channel,qamType,&mseVal);
                        sprintf(emptyStr[eCmDownstreamSnr0+channel*7],"%3.2f dB",mseVal);
                    }
                    else
                    {
                        sprintf(emptyStr[eCmDownstreamModulation0+channel*7],"unknown");                       
                        sprintf(emptyStr[eCmDownstreamChannelId0+channel*7],"0");                        
                        sprintf(emptyStr[eCmDownstreamSymbolRate0+channel*7],"0 Ksym/sec");                                               
                        sprintf(emptyStr[eCmDownstreamPower0+channel*7],"0 dBmV");
                        sprintf(emptyStr[eCmDownstreamSnr0+channel*7],"0 dBmV");
                    }
                }
        }
		return 0;
}

int get_UsChannelParms(int channel)
{
        Ucd_IfUsChannelEntry_t usChProperty;
        Ucd_IfUsChannelEntry_t *usChPropertyP = &usChProperty;
            if ( Ucd_UcdDb_GetUsChannelProperty(channel+1, usChPropertyP) == OK )
            {
                uint8 islock = 0;
                if( usChPropertyP->frequency!=0 )
                    islock = 1;
				
                if( islock )
                {

                    int iindex;
                    Ucd_burstDescriptor_t *pBurstDes=NULL;
                    Uint8 us_modulation = 0;
                    for (iindex=0;iindex<UCDDB_IUC_MAX_NUMBER -2;iindex++)
                    {
                        pBurstDes = GetUcdPropertyBurstDescriptorP(usChPropertyP,iindex);
                        if (IsBurstDescriptorValid(pBurstDes))
                        {
                            if( GetBurstModulationType(pBurstDes) > us_modulation )
                                us_modulation = GetBurstModulationType(pBurstDes);
                        }
                    }            
                    sprintf(emptyStr[eCmUpstreamLockStatus0+channel*6],"Locked");
                    sprintf(emptyStr[eCmUpstreamModulation0+channel*6],"%s",ucdModStr[us_modulation]);
                    sprintf(emptyStr[eCmUpstreamChannelId0+channel*6],"%d",usChPropertyP->usId);
                    sprintf(emptyStr[eCmUpstreamSymbolRate0+channel*6],"%d Ksym/sec",usChPropertyP->symbolRate*RATE_MULTIPLIER);
				 /* #UnihanBU6#, Rainy, 20080618,frequency format ###.### MHz*/
                    sprintf(emptyStr[eCmUpstreamFrequency0+channel*6],"%.2f MHz",(float)usChPropertyP->frequency/MEGA);
                    float reportPower[4];
                    HalDb_UsCalGetReportedPower(0,&(reportPower[0]));
                    sprintf(emptyStr[eCmUpstreamPower0+channel*6],"%3.2f dBmV",reportPower[0]);
                }
                else
                {
                    sprintf(emptyStr[eCmUpstreamLockStatus0+channel*6],"Not Locked");
                    sprintf(emptyStr[eCmUpstreamModulation0+channel*6],"unknown");
                    sprintf(emptyStr[eCmUpstreamChannelId0+channel*6],"0");
                    sprintf(emptyStr[eCmUpstreamSymbolRate0+channel*6],"0 Ksym/sec");
                    sprintf(emptyStr[eCmUpstreamFrequency0+channel*6],"0 Hz");
                    sprintf(emptyStr[eCmUpstreamPower0+channel*6],"0 dBmV");
                }
            }
		return 0;
}
