/*
 * CGI helper functions
 *
 * Copyright 2003, Broadcom Corporation
 * All Rights Reserved.                
 *                                     
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Broadcom Corporation;   
 * the contents of this file may not be disclosed to third parties, copied
 * or duplicated in any form, in whole or in part, without the prior      
 * written permission of Broadcom Corporation.                            
 *
 * $Id: system.c,v 1.1.1.1 2010/06/30 06:49:33 eating Exp $
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include "httpd.h"
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <fcntl.h>
#include <stdlib.h>
#include <syslog.h>
#include <arpa/inet.h>
#include <linux/kernel.h>

#include "release.h"
#include "docsis_db.h"
#include "production_db.h"
#include "pacm_base.h"
#include "nvram_db.h"
/* #UnihanBU6#,Rainy, 20080902, show BPI/BPI+ status.  */
#include "bpi.h"
#include "bpi_db.h"
#include "sw_dl_db.h"
#include "utils_sys.h"
#include "vendor_common_utils.h"
#include "dbridge_conf.h"
/* #UnihanBU6#, Chenwin_Qiu, 20090323, show sw_revision. */
#include "system_shmdb_util.h"
#define MAX_REP_STR 256
#define MIDNIGHT_JAN_01_2000 (60L*60L*24L*7L*52L*30L+60L*60L*24L*37L)


int SYSTEM_Handle(int eid, webs_t wp, int argc, char_t ** argv);
int SYSTEM_get_IFState(webs_t wp,char_t*name);
int SYSTEM_get_SwFileName(webs_t wp,char_t*name);
int SYSTEM_get_CpeNumber(webs_t wp,char_t*name);
int SYSTEM_get_FlashPart(webs_t wp,char_t*name);
int SYSTEM_get_MtaTypeId(webs_t wp,char_t*name);
int SYSTEM_get_FirmwareName(webs_t wp,char_t*name);
int SYSTEM_get_FirmwareBuildTime(webs_t wp,char_t*name);
int SYSTEM_get_CmState(webs_t wp,char_t*name);
int SYSTEM_get_HwRevision(webs_t wp,char_t*name);
int SYSTEM_get_SwVersion(webs_t wp,char_t*name);
int SYSTEM_get_SysUpTime(webs_t wp,char_t*name);
int SYSTEM_get_CurrentTime(webs_t wp,char_t*name);
int SYSTEM_get_VendorName(webs_t wp,char_t*name);
int SYSTEM_get_ModelName(webs_t wp,char_t*name);
int SYSTEM_get_ProductType(webs_t wp,char_t*name);
int SYSTEM_get_DownloadRevision(webs_t wp,char_t*name);
int SYSTEM_get_BootRevision(webs_t wp,char_t*name);
int SYSTEM_get_CmSerialNumber(webs_t wp,char_t*name);
int SYSTEM_get_MtaSerialNumber(webs_t wp,char_t*name);
int SYSTEM_get_SwRevision(webs_t wp,char_t*name);
int SYSTEM_get_WanIsolation(webs_t wp,char_t*name);
/* #UnihanBU6#,Rainy, 20080902, show BPI/BPI+ status.  */
int SYSTEM_get_BpiStatus(webs_t wp,char_t*name);
int SYSTEM_get_BpiComment(webs_t wp,char_t*name);

struct cgi_handler SYSTEM_handlers[]={
{"sys_uptime",SYSTEM_get_SysUpTime},
{"cpe",SYSTEM_get_CpeNumber},
{"cm_state",SYSTEM_get_CmState},
{"wan_isolation",SYSTEM_get_WanIsolation},
{"currenttime",SYSTEM_get_CurrentTime},
{"hw_rev",SYSTEM_get_HwRevision},
{"vendor",SYSTEM_get_VendorName},
{"boot_rev",SYSTEM_get_BootRevision},
{"sw_rev",SYSTEM_get_SwRevision},
{"model",SYSTEM_get_ModelName},
{"product_type",SYSTEM_get_ProductType},
{"flash_part",SYSTEM_get_FlashPart},
{"dl_rev",SYSTEM_get_DownloadRevision},
{"sw_ver",SYSTEM_get_SwVersion},
{"cm_serialnum",SYSTEM_get_CmSerialNumber},
{"mta_serialnum",SYSTEM_get_MtaSerialNumber},
{"mta_typeid",SYSTEM_get_MtaTypeId},
{"firmware_name",SYSTEM_get_FirmwareName},
{"buildtime",SYSTEM_get_FirmwareBuildTime},
{"usb0",SYSTEM_get_IFState},
{"eth0",SYSTEM_get_IFState},
{"sw_filename",SYSTEM_get_SwFileName},
/* #UnihanBU6#,Rainy, 20080902, show BPI/BPI+ status.  */
{"bpi_status",SYSTEM_get_BpiStatus},
{"bpi_comment",SYSTEM_get_BpiComment},
{NULL,NULL}
};


int SYSTEM_Handle(int eid,webs_t wp,int argc,char_t**argv)
{
	char_t*name;
	struct cgi_handler *handler;
	/* #UnihanBU6#,Rainy, 20080902, show BPI/BPI+ status.  */
        if ( Bpi_DbRetrieveAccess() == NOK )
        {
		websWrite(wp, "%s","unknown");
		return -1;
        }

        /* #UnihanBU6#, Chenwin_Qiu, 2009/03/23, Implement sw_revision utility. Start */
/* #UnihanBU6#, Eating Chen, 2009/12/02, System_ShmDb_RetrieveAccess() function 
 * shall be called once in each process/task 
 */
#if 0
        if( System_ShmDb_RetrieveAccess() == NOK)
        {
              websWrite(wp,"%s","unknown");
              return -1;
        }
#endif
        /* #UnihanBU6#, Chenwin_Qiu, 2009/03/23, Implement sw_revision utility. End */
        
	if( ProductionDb_RetrieveAccess() == NOK )
	{
		websWrite(wp, "%s","unknown");
		return -1;
	}
	if(DocsisParamsDb_RetrieveAccess() == NOK )
	{
		websWrite(wp, "%s","unknown");
		return -1;
	}
	if(Common_Utils_init()==NOK)
	{
		websWrite(wp,"%s","unknow");
		return -1;
	}
	if(ejArgs(argc,argv,"%s",&name)<1)
	{
		websError(wp,400,"Insufficient args\n");
		return -1;
	}
	
	for(handler=&SYSTEM_handlers[0];handler->pattern;handler++)
	{
		if(strncmp(handler->pattern,name,strlen(handler->pattern))==0)
			handler->output(wp,name);
	}
	return 0;
}

/* #UnihanBU6#, Chenwin_Qiu, 2009/03/23, Implement sw_revision utility. Start */
int SYSTEM_get_SwRevision(webs_t wp,char_t*name)
{
    char swRev[SYS_DFLT_SW_REVISION_SIZE];
    memset(swRev,0,SYS_DFLT_SW_REVISION_SIZE);    
	
    if(System_ShmDb_GetSwRevision(swRev)==STATUS_OK)
    {
        websWrite(wp,"%s",swRev);    
    }	
    else 
    {
        websWrite(wp,"%s","unknow");
    }	
	
    return 0;
}
/* #UnihanBU6#, Chenwin_Qiu, 2009/03/23, Implement sw_revision utility. End */

int SYSTEM_get_SwVersion(webs_t wp,char_t*name)
{
	char swVer[MAX_REP_STR];
	memset(swVer,0,MAX_REP_STR);
	if(utils_getSwVersion(swVer,MAX_REP_STR)==STATUS_OK)
	{
		websWrite(wp,"%s",swVer);
	}
	else
	{
		websWrite(wp,"%s","unknow");
	}
	return 0;
}

int SYSTEM_get_SwFileName(webs_t wp,char_t*name)
{
	 char SWName[COMMON_UTILS_MAX_LEN_OF_FILE_NAME];
	 memset(SWName,0,COMMON_UTILS_MAX_LEN_OF_FILE_NAME);
	if(Common_Utils_GetSwFileName(SWName)==STATUS_OK)
	{
	 	websWrite(wp,"%s",SWName);
	}
	else 
	{
		websWrite(wp,"%s","unknow");
	}
	return 0;
}

int SYSTEM_get_HwRevision(webs_t wp,char_t*name)
{
	char hwVer[MAX_REP_STR];
	memset(hwVer,0,MAX_REP_STR);
	Uint16 HwVer0=0, HwVer1=0;
	/* #UnihanBU6# shadow 2009/03/12 hw version display 1.02b error, cant display "b" */
	char hwVersion[MAX_REP_STR] = {0};

	ProdDb_GetPbHwRevision(&HwVer0);
    /* #UnihanBU6# shadow 2009/03/12 hw version display 1.02b error, cant display "b" */
#if 0
	ProdDb_GetPbHwPatch(&HwVer1);
	sprintf(hwVer,"%d.%d",HwVer0, HwVer1);
#else
	if (ProdDb_GetHwRevPatch(hwVersion) != STATUS_OK)
	{
		/* in ti/docsis/src/common/managementsnmp/src/sys.c  function BuildSysDescr()  
		* use 3 if cant not read from production db
		*/
	    sprintf(hwVersion,"03");     
	}
	sprintf(hwVer,"%d.%s",HwVer0, hwVersion);
#endif
	websWrite(wp, "%s", hwVer); 	
	return 0;
}

/* #UnihanBU6#, Rainy_yuan, 2008/09/03 * to get image build time */
int SYSTEM_get_FirmwareBuildTime(webs_t wp,char_t*name)
{
	char buildtime[MAX_REP_STR];
	int time_len=4;
	char time_temp[6][time_len+1];
	
	/* #UnihanBU6#, Chenwin_Qiu, 2009/03/17 fix the format of display image build time. */
	int week_int;
	char *week_str;
	char *month_str;
	int timearray [6];

        char *week_array[]={"Mon","Tue","Wed","Thu","Fri","Sat","Sun"};
        char *month_array[]={"Jan","Feb","Mar","Apr","May","June","July","Aug","Sept","Oct","Nov","Dec"};
	
	memset(buildtime,0,MAX_REP_STR);
	memset(time_temp,0,sizeof(time_temp));

	if (Common_Utils_GetSwBuildTime(buildtime) == OK)
	{
		int i,offset=4;
		int lenth_buildtime= strlen(buildtime);
		strncpy(time_temp[0],buildtime,4);
		timearray[0] = atoi(time_temp[0]);
			
                for (i=1;i<6;i++)
                {
              	     if (i==1)
                     {
                            strncpy(time_temp[i],buildtime+offset,2);
                            timearray[i] = atoi(time_temp[i]);
                     }
                     else
                     {
                            offset=4+(i-1)*2;
                            strncpy(time_temp[i],buildtime+offset,2);
                            timearray[i] = atoi(time_temp[i]);
                     }
                }
                /*to count week day*/
                week_int= (timearray[2]+2*timearray[1]+3*(timearray[1]+1)/5+timearray[0]+timearray[0]/4-timearray[0]/100+timearray[0]/400)%7;
                week_str=week_array[week_int];
		/*to count month*/	  
                month_str=month_array[timearray[1]-1];	  
                websWrite(wp,"%s %s %s %s:%s:%s %s",week_str,month_str,time_temp[2],time_temp[3],time_temp[4],time_temp[5],time_temp[0]);      		
	        /* #UnihanBU6#, Chenwin_Qiu, 2009/03/17 , End*/
        }
	else
	{
		websWrite(wp, "%s", "unknow"); 	
	}
	return 0;
}

int SYSTEM_get_SysUpTime(webs_t wp,char_t*name)
{
	char systemtime[MAX_REP_STR];
	memset(systemtime,0,MAX_REP_STR);
	struct sysinfo info;
	sysinfo(&info);
	long uptime = info.uptime;
	int days, hours, minutes, seconds, remainder;
	days = uptime/(24*60*60);
	remainder = uptime % (24*60*60);
	hours = remainder/(60*60);
	remainder %= (60*60);
	minutes = remainder/(60);
	remainder %= (60);
	seconds = remainder;
	sprintf(systemtime,"%1d days %02dh:%02dm:%02ds",days,hours,minutes,seconds);
	websWrite(wp, "%s", systemtime); 	
	return 0;
}


int SYSTEM_get_WanIsolation(webs_t wp,char_t*name)
{
	Uint8 value;
	if(Dbridge_GetDbrige_Mode(&value) == OK)
	{
		if(value == DBR_STANDBY_MODE)
		{
			websWrite(wp,"%s","ON");
		}
		else
		{
			websWrite(wp,"%s","OFF");
		}
	}
	else
	{
		websWrite(wp,"%s","unknow");
	}
	return 0;
}

int SYSTEM_get_IFState(webs_t wp,char_t*name)
{
	IfDb_Entry_t *data = NULL;
	int idx;
	char buffer[32];
	memset(buffer,0,sizeof(buffer));
	
	for ( idx = 0; idx < IF_INDEXES_MAX_NUM; idx++ )
	{
		 data = Common_Utils_GetIfTableByIndex(idx) ;
		 if (data != NULL)
		 {
			if ( strcmp( (*data).pIfDescr2Index->ifName, name) == 0)
			{
				switch((*data).ifAdminStatus)
				{
					case 1: // up
						sprintf(buffer,"Enabled");
						break;
				
					case 2: // down
					default:
						sprintf(buffer,"Disabled");
						break;
				
					case 3: // testing
						sprintf(buffer,"Testing");
						break;
				}
				websWrite(wp,"%s",buffer);
			}
		 }
	}
	return 0;
}

int SYSTEM_get_CpeNumber(webs_t wp,char_t*name)
{
	if(Common_Utils_GetCpeNumber()== -1)
	{
		websWrite(wp,"%s","unknow");
	}
	else
	{
		websWrite(wp,"%d",Common_Utils_GetCpeNumber());
	}
	return 0;
}

int SYSTEM_get_FlashPart(webs_t wp,char_t*name)
{
	int activeimage=0;
	if(Common_Utils_GetActiveFlashPart(&activeimage)==STATUS_OK)
	{
		websWrite(wp,"%d",activeimage);
	}
	else
	{
		websWrite(wp,"%s","unknow");
	}
	return 0;
}

int SYSTEM_get_MtaTypeId(webs_t wp,char_t*name)
{
	/* #UnihanBU6#,Alisa_Gao, 20080630, need to implement */
	websWrite(wp,"%s","MGCP");
	return 0;
}

int SYSTEM_get_FirmwareName(webs_t wp,char_t*name)
{
	char FirmName[COMMON_UTILS_MAX_LEN_OF_FILE_NAME];
	memset(FirmName,0,COMMON_UTILS_MAX_LEN_OF_FILE_NAME);
	if(Common_Utils_GetFirmwareName(FirmName)==STATUS_OK)
	{	
		websWrite(wp,"%s",FirmName);
	}
	else
	{
		websWrite(wp,"%s","unknow");
	}
	return 0;
}


int SYSTEM_get_CmState(webs_t wp,char_t*name)
{
	static CmMacParams_t cmMacPar;
	if (Cm_DocsisDb_GetCmMacParams(&cmMacPar) != STATUS_OK)
	{
		printf("Can't get CM status\n");
		websWrite(wp,"%s","unknow");
		return -1;
	}
	else 
	{
		switch(cmMacPar.macStatus)
		{
			case DOCSIS_CM_STAT_PARAMS_NOT_READY:
				 websWrite(wp,"Not Ready");
				break;
			case DOCSIS_CM_STAT_PARAMS_NOT_SYNCHRONIZED:
				 websWrite(wp,"Not Synchronized");
				break;
			case DOCSIS_CM_STAT_PARAMS_PHY_SYNCHRONIZED:
				websWrite(wp,"Phy Synchronized");
				break;
			case  DOCSIS_CM_STAT_PARAMS_DS_TOPOLOGY_RESOLUTION_IN_PROGRESS:
				websWrite(wp,"Ds Topology Resolution In Progress");
				break;
			case DOCSIS_CM_STAT_PARAMS_US_PARAMETERS_ACQUIRED:
				websWrite(wp,"Us Parameters Acquired");
				break;
			case DOCSIS_CM_STAT_PARAMS_RANGING_IN_PROGRESS:
				websWrite(wp,"Ranging In Progress");
				break;
			case DOCSIS_CM_STAT_PARAMS_RANGING_COMPLETE:
				websWrite(wp,"Ranging Complete");
				break;
			case DOCSIS_CM_STAT_PARAMS_EAE_IN_PROGRESS:
				websWrite(wp,"Eae In Progress");
				break;
			case DOCSIS_CM_STAT_PARAMS_DHCPV4_IN_PROGRESS:
				websWrite(wp,"DHCPv4 In Progress");
				break;
			case DOCSIS_CM_STAT_PARAMS_DHCPV6_IN_PROGRESS:
				websWrite(wp,"DHCPv6 In Progress");
				break;
			case  DOCSIS_CM_STAT_PARAMS_DHCPV4_COMPLETE:
				websWrite(wp,"DHCPv4 Complete");
				break;
			case DOCSIS_CM_STAT_PARAMS_DHCPV6_COMPLETE:
				websWrite(wp,"DHCPv6 Complete");
				break;
			case DOCSIS_CM_STAT_PARAMS_TOD_ESTABLISHED:
				websWrite(wp,"ToD Established");
				break;
			case DOCSIS_CM_STAT_PARAMS_SECURITY_ESTABLISHED:
				websWrite(wp,"Security Established");
				break;
			case DOCSIS_CM_STAT_PARAMS_CONFIG_FILE_DOWNLOAD_COMPLETE:
				websWrite(wp,"Config File Download Complete");
				break;
			case DOCSIS_CM_STAT_PARAMS_REGISTRATION_IN_PROGRESS:
				websWrite(wp,"Registration In Progress");
				break;
			case DOCSIS_CM_STAT_PARAMS_REGISTRATION_COMPLETE:
				websWrite(wp,"Registration Complete");
				break;
			case DOCSIS_CM_STAT_PARAMS_ACCESS_DENIED:
				websWrite(wp,"Access Denied");
				break;
			case DOCSIS_CM_STAT_PARAMS_OPERATIONAL:
				websWrite(wp,"Operational");
				break;
			case DOCSIS_CM_STAT_PARAMS_BPI_INIT:
				websWrite(wp,"BPI init");
				break;
			case DOCSIS_CM_STAT_PARAMS_FORWARDING_DISABLED:
				websWrite(wp,"Forwarding Disabled");
				break;
			case  DOCSIS_CM_STAT_PARAMS_RF_MUTE_ALL:
				websWrite(wp,"RF mute all");
				break;
			case  DOCSIS_CM_STAT_PARAMS_OTHER:
				websWrite(wp,"other");
				break;
			#if(CONFIG_FOR_NETGEAR)
			case DOCSIS_CM_STAT_PARAMS_DS1:
				websWrite(wp,"Downstream channel 1");
				break;
				case DOCSIS_CM_STAT_PARAMS_DS2:
				websWrite(wp,"Downstream channel 2");
				break;
			case DOCSIS_CM_STAT_PARAMS_DS3:
				websWrite(wp,"Downstream channel 3");
				break;
			case DOCSIS_CM_STAT_PARAMS_DS4:
				websWrite(wp,"Downstream channel 4");
				break;
			case DOCSIS_CM_STAT_PARAMS_US1:
				websWrite(wp,"Upstream channel 1");
				break;
			case DOCSIS_CM_STAT_PARAMS_US2:
				websWrite(wp,"Upstream channel 2");
				break;
			case DOCSIS_CM_STAT_PARAMS_US3:
				websWrite(wp,"Upstream channel 3");
				break;
			case DOCSIS_CM_STAT_PARAMS_US4:
				websWrite(wp,"Upstream channel 4");
				break;
			#endif
			default:
				websWrite(wp,"unknow");
				break;
		}
	}	
	return 0;
}


int SYSTEM_get_CurrentTime(webs_t wp,char_t*name)
{
	time_t now;
	time(&now);
	if(now<MIDNIGHT_JAN_01_2000)
	{
		websWrite(wp,"%s","--:--:--:--:--:--");
	}
	else
	{
		websWrite(wp,"%s",ctime(&now));
	}
	return 0;	 
}

int SYSTEM_get_VendorName(webs_t wp,char_t*name)
{
	char vendor[MAX_REP_STR];
	memset(vendor,0,MAX_REP_STR);
	sprintf(vendor,"%s",CONFIG_VENDOR_NAME);
	websWrite(wp,"%s",vendor);
	return 0;
}


int SYSTEM_get_ModelName(webs_t wp,char_t*name)
{
	char model[MAX_REP_STR];
	memset(model,0,MAX_REP_STR);
	sprintf(model,"%s",CONFIG_VENDOR_MODEL);
	websWrite(wp,"%s",model);
	return 0;
}

int SYSTEM_get_ProductType(webs_t wp,char_t*name)
{
	char product[MAX_REP_STR];
	memset(product,0,MAX_REP_STR);
	sprintf(product,"%s",CONFIG_VENDOR_MODEL);
	websWrite(wp,"%s",product);
	return 0;
}

int SYSTEM_get_DownloadRevision(webs_t wp,char_t*name)
{  
	char swFileName[MAX_REP_STR];
	memset(swFileName,0,MAX_REP_STR);
	if(NvramDb_GetNewFileName(swFileName)==STATUS_OK)
	{
		if(!strcmp(swFileName,""))
		{
			websWrite(wp,"%s","------");
		}
		else
		{
			websWrite(wp,"%s",swFileName);
		}
	}
	else
	{
		websWrite(wp,"%s","unknow");
	}
	return 0;
}

int SYSTEM_get_CmSerialNumber(webs_t wp,char_t*name)
{
	char serialnumber[MAX_REP_STR];
	memset(serialnumber,0,MAX_REP_STR);
	if(ProdDb_GetSerialNumber(serialnumber)==STATUS_OK)
	{
		websWrite(wp,"%s",serialnumber);
	}
	else
	{
		websWrite(wp,"%s","unknow");
	}
	return 0;
}

int SYSTEM_get_MtaSerialNumber(webs_t wp,char_t*name)
{
	char serialnumber[MAX_REP_STR];
	memset(serialnumber,0,MAX_REP_STR);
	if(ProdDb_GetSerialNumber(serialnumber)==STATUS_OK)
	{
		websWrite(wp,"%s",serialnumber);
	}
	else
	{
		websWrite(wp,"%s","unknow");
	}
	return 0;
}

int SYSTEM_get_BootRevision(webs_t wp,char_t*name)
{
	Char bootVersion[MAX_REP_STR];
	memset(bootVersion,0,MAX_REP_STR);
	if(utils_getUbootVersion(bootVersion, sizeof(bootVersion))==STATUS_OK)
	{
		websWrite(wp,"%s",bootVersion);
	}
	else
	{
		websWrite(wp,"%s","unknow");
	}
	return 0;
}

/* #UnihanBU6#,Rainy, 20080902, show BPI/BPI+ status.  */
int SYSTEM_get_BpiStatus(webs_t wp,char_t*name)
{
	Int16 bpienable=0;

	if (Bpi_BpiDb_GetBpiEnable(&bpienable) == STATUS_OK)    
	{
		websWrite(wp, "%s", bpienable?"Enabled":"Disabled");    
	}
	else
	{
		websWrite(wp,"%s","unknow");
	}

	return 0;
}

int SYSTEM_get_BpiComment(webs_t wp,char_t*name)
{
   	Uint32 bpienableplus=0;    
	Int16 bpienable=0;

	if ((Bpi_BpiDb_GetBpiEnable(&bpienable) == OK) && (Bpi_BpiDb_GetBpiPlusMode(&bpienableplus) == OK))
	{
		if( bpienable )        
		{            
			if(bpienableplus)            
			{		  
			    websWrite(wp, "%s","BPI+");            
			}            
			else            
			{		  
				websWrite(wp, "%s","BPI");            
			}        
		}        
		else        
		{		  
			websWrite(wp, "%s","Disabled");        
		}    
	}
	else
	{
		websWrite(wp,"%s","unknow");
	}

	return 0;
}
