/*
 * CGI helper functions
 *
 * Copyright 2003, Broadcom Corporation
 * All Rights Reserved.                
 *                                     
 * This is UNPUBLISHED PROPRIETARY SOURCE CODE of Broadcom Corporation;   
 * the contents of this file may not be disclosed to third parties, copied
 * or duplicated in any form, in whole or in part, without the prior      
 * written permission of Broadcom Corporation.                            
 *
 * $Id: tech.c,v 1.1.1.1 2010/06/30 06:49:33 eating Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "httpd.h"
#include "vendor_common_utils.h"
#include "vendornvm_common.h"
#include "services.h"
#ifdef CONFIG_FOR_NETGEAR_CVD31XT
#include "event_mgr_main.h"
#include "event_mgr_if.h"
#endif 

extern void set_ulevel(int lvl);
extern STATUS Services_Util_SendIccMessage(ServicesStatEvType_e Type, unsigned char Action, unsigned long Parameter);
int QOS_Handle(int eid, webs_t wp, int argc, char_t **argv);
int TECH_get_TelnetStatus(webs_t wp, char_t *name);
int TECH_get_SshStatus(webs_t wp, char_t *name);
int TECH_get_HttpLanAccess(webs_t wp, char_t *name);
int TECH_get_HttpWanAccess(webs_t wp, char_t *name);
STATUS TECH_set_PasswordSeed(webs_t wp, char_t *name);
STATUS TECH_set_TelnetStatus(webs_t wp, char_t *name);
STATUS TECH_set_SshStatus(webs_t wp, char_t *name);
STATUS TECH_set_HttpWanAccessType(webs_t wp, char_t *name);
STATUS TECH_set_HttpLanAccessType(webs_t wp, char_t *name);
 
int apply_techsupport_cgi(webs_t wp, char_t *urlPrefix, char_t *webDir, int arg,char_t *url, char_t *path, char_t *query);
int  apply_adventrance_cgi(webs_t wp, char_t *urlPrefix, char_t *webDir, int arg,char_t *url, char_t *path, char_t *query);
int apply_management_cgi(webs_t wp, char_t *urlPrefix, char_t *webDir, int arg,char_t *url, char_t *path, char_t *query);


struct cgi_handler TECH_handlers[]=
{
    {"tel_state",TECH_get_TelnetStatus},
    {"ssh_state",TECH_get_SshStatus},
    {"lan_access",TECH_get_HttpLanAccess},
    {"wan_access",TECH_get_HttpWanAccess},
    {NULL,NULL}
};

struct cgi_handler TECH_SET_ChangePassword_Handlers[]=
{
    {"passwordseed",TECH_set_PasswordSeed},
    {NULL,NULL}
};

struct cgi_handler TECH_SET_UserAccess_Handlers[]=
{	
    {"telnetStatus",TECH_set_TelnetStatus},
    {"sshStatus",TECH_set_SshStatus},
    {"enableHttpLan",TECH_set_HttpLanAccessType},
    {"enableHttpWan",TECH_set_HttpWanAccessType},
    {NULL,NULL}
};

int TECH_Handle(int eid, webs_t wp, int argc, char_t **argv)
{
    char_t*name;
    struct cgi_handler *handler;
    
    if( VendorNvm_Common_RetrieveAccess() == STATUS_NOK)
    {
        fprintf(stderr,"%s","VendorNvm init Error!");
        return -1;
    }
    if(Common_Utils_init() == STATUS_NOK)
    {
        fprintf(stderr,"%s","DB init Error!");
        return -1;
    } 
    if(ejArgs(argc,argv,"%s",&name)<1)
    {
        websError(wp,400,"Insufficient args\n");
        return -1;
    }
    
    for(handler=&TECH_handlers[0];handler->pattern;handler++)
    {
        if(strncmp(handler->pattern,name,strlen(handler->pattern))==0)
            handler->output(wp,name);
    }
    return 0;
}

/* #UnihanBU6#,Alisa_Gao, 20080714,implement CGI to support techsupport page*/
int apply_techsupport_cgi(webs_t wp, char_t *urlPrefix, char_t *webDir, int arg, 
	  char_t *url, char_t *path, char_t *query)
{
    char *userbuttontype;
    userbuttontype=websGetVar(wp,"buttontype","");
    struct cgi_handler *handler;
    /*init DB*/
    if( VendorNvm_Common_RetrieveAccess() == STATUS_NOK)
    {
        fprintf(stderr,"%s","VendorNvm init Fail!");
        return -1;
    }
	
    if(Common_Utils_init() == STATUS_NOK)
    {
        fprintf(stderr,"%s","DB init Fail");
        return -1;
    } 
    /*different button differnt handler method*/
    if(!strcmp(userbuttontype,"changepassword"))
    {
        for(handler=&TECH_SET_ChangePassword_Handlers[0];handler->pattern;handler++)
        { 
             handler->output(wp,NULL);
        }
    }
    else if(!strcmp(userbuttontype,"saveuseraccess"))
    {
        for(handler=&TECH_SET_UserAccess_Handlers[0];handler->pattern;handler++)
        {
             handler->output(wp,NULL);
        } 
    } 
     do_ej("techsupport.asp", wp);
     return 0;
}
/*#UnihanBU6#,Alisa_Gao, 20080714 End*/

int apply_adventrance_cgi(webs_t wp, char_t *urlPrefix, char_t *webDir, int arg,char_t *url, char_t *path, char_t *query)
{
/* #UnihanBU6#, Energy_Yang, 2009/02/20 * Update the CVD31XT seed potd function and potd tools, ao all netgear project can use the same tool. */
    char seed[SYS_DFLT_HTTP_PASS_SEED_SIZE];
    char buf[SYS_DFLT_HTTP_PASS_SEED_POTD_SIZE];
    char*webpassword;
    memset(buf,0,sizeof(buf));
    memset(seed,0,sizeof(seed));
    webpassword=websGetVar(wp,"PasswordOfTheDay","");
    if(Common_Utils_GetHttpPasswordSeed(seed) == STATUS_OK)
    {	
        if(Common_Utils_GetNetgearPasswordOfTheDay(seed,strlen(seed),buf,sizeof(buf)) == STATUS_OK)
        {
            if(!strcmp(webpassword,buf))
            {
                set_ulevel(AuthLevelAdminAdvanced);
                do_ej("adv_product.asp",wp);/*the first page of advanced*/
                return 0;
            }
        }
    }			
    do_ej("adv_entrance.asp",wp);/* when error,return potd page*/
    return 0;
}

int apply_management_cgi(webs_t wp, char_t *urlPrefix, char_t *webDir, int arg,char_t *url, char_t *path, char_t *query)
{
    char *isrestoredef;
    char *isreboot;
    char command[128];

    memset(command,0,sizeof(command));
    isrestoredef=websGetVar(wp,"RestoreFactory","");
    isreboot=websGetVar(wp,"Reboot","");
    
    if(atoi(isrestoredef) == 1)
    {
        sprintf(command,"rm -rf %s",SYS_DFLT_VENDOR_NVMRAM_PATH );
        system(command);
        #ifdef CONFIG_FOR_NETGEAR_CVD31XT
        if( OK != EVMIF_EventSend(EVENT_MGR_CTRL_SOCK, VENDOR_EVM_RESET_TO_DEFAULT, 1, NULL, NULL) ) 
	  {
	      printf("Send Change Password Event Fail!");
	  }
        #endif
        do_ej("basic_wait.asp",wp);
	/*after remove the nvram file,you need reboot*/
        memset(command,0,sizeof(command));
        sprintf(command,"reboot");
        system(command);
    }
	
    if(atoi(isreboot) == 1)
    {
        memset(command,0,sizeof(command));
	 #ifdef CONFIG_FOR_NETGEAR_CVD31XT
        if( OK != EVMIF_EventSend(EVENT_MGR_CTRL_SOCK, VENDOR_EVM_REBOOT, 1, NULL, NULL) ) 
	  {
	      printf("Send Change Password Event Fail!");
	  }
        #endif
        do_ej("basic_wait.asp",wp);
        sprintf(command,"reboot");
        system(command);
    }
	return 0;
}

/* #UnihanBU6#, Alisa_Gao, 2008/09/09, get telnet status in techsupport page */
/**************************************************************************/
/*! \fn STATUS STATUS TECH_get_TelnetStatus( webs_t wp,char_t *name)
 **************************************************************************
 *  \brief get:get the telnet status,telnet status is enable or disable,the default status is enable.
 *  \param[in] webstream wp,item name in page.
 *  \param[out] no output.
 *  \return NOK if error,else return OK.
 */
int TECH_get_TelnetStatus(webs_t wp, char_t *name)
{
    unsigned char telstate=SYS_DFLT_VSIF_TELNET_ENABLE;/*telent default status:enable*/
    if(System_ShmDb_GetTelnetEnabled(&telstate) == NOK)
    {
        fprintf(stderr,"%s","Telenet Status Get Fail!");
    }
    websWrite(wp,"%d",telstate);
    return 0;
}


/* #UnihanBU6#, Alisa_Gao, 2008/09/09, set telnet status in techsupport page */
/**************************************************************************/
/*! \fn STATUS STATUS TECH_set_TelnetStatus( webs_t wp,char_t *name)
 **************************************************************************
 *  \brief set telnet status,and then start or stop telnet service.
 *  \param[in] webstream wp,item name in page.
 *  \param[out] no output.
 *  \return NOK if error,else return OK.
 */
STATUS TECH_set_TelnetStatus(webs_t wp, char_t *name)
{
    char *telnetenable = websGetVar(wp,"telnetStatus","");
    unsigned char enable=atoi(telnetenable); 
    unsigned char old_enable = SYS_DFLT_VSIF_TELNET_ENABLE;
    if(System_ShmDb_GetTelnetEnabled(&old_enable)!= OK)
    {
        fprintf(stderr,"%s","Telnet status Get FaIl\n");
        return NOK;
    }
    if(old_enable != enable)
    {
    if(enable == 1)/*1==>enable,0==>disable*/
    {
         Services_Util_SendIccMessage(SERVICES_SINGLE_PROCESS,START,SERVICES_STATEV_TELNET);
    }
    else 
    {
        Services_Util_SendIccMessage(SERVICES_SINGLE_PROCESS,STOP,SERVICES_STATEV_TELNET);
    }
	    #ifdef CONFIG_FOR_NETGEAR_CVD31XT
           if( OK != EVMIF_EventSend(EVENT_MGR_CTRL_SOCK, VENDOR_EVM_CHANGE_TELNET_STATUS, 1, NULL, NULL)) 
           {
                 printf("Send Change Telnet Status Event Fail!");
           }
           #endif 
	    
    }
    if(System_ShmDb_SetTelnetEnabled(enable) == NOK)
    {	
        fprintf(stderr,"%s","Telenet status Save Fail!");
        return NOK;
    }
    return OK;
}


/* #UnihanBU6#, Alisa_Gao, 2008/09/09, get ssh status in techsupport page */
/**************************************************************************/
/*! \fn STATUS STATUS TECH_get_SshStatus( webs_t wp,char_t *name)
 **************************************************************************
 *  \brief get:get the ssh status,ssh status is enable or disable,the default status is enable.
 *  \param[in] webstream wp,item name in page.
 *  \param[out] no output.
 *  \return NOK if error,else return OK.
 */
int TECH_get_SshStatus( webs_t wp,char_t *name)
{
    unsigned char sshstate=SYS_DFLT_SSH_ENABLE;/*ssh default status is enable*/
    if(System_ShmDb_GetSshEnabled(&sshstate) == NOK)
    {
        fprintf(stderr,"%s","SSH status Get Fail!");
    }
    websWrite(wp,"%d",sshstate);
    return 0;
}


/* #UnihanBU6#, Alisa_Gao, 2008/09/09, set ssh status in techsupport page */
/**************************************************************************/
/*! \fn STATUS STATUS TECH_set_SshStatus( webs_t wp,char_t *name)
 **************************************************************************
 *  \brief set ssh status,and then start or stop ssh service.
 *  \param[in] webstream wp,item name in page.
 *  \param[out] no output.
 *  \return NOK if error,else return OK.
 */
STATUS TECH_set_SshStatus( webs_t wp,char_t *name)
{
    char *sshenable = websGetVar(wp,"sshStatus","");
    unsigned char enable=atoi(sshenable);
    unsigned char old_enable = SYS_DFLT_SSH_ENABLE;
    if(System_ShmDb_GetSshEnabled(&old_enable) != OK)
    {
        fprintf(stderr,"%s","SSH status Get FaIl\n");
        return NOK;
    }
    if(enable != old_enable)
    {
    if(enable == 1)
    {
        Services_Util_SendIccMessage(SERVICES_SINGLE_PROCESS, START,SERVICES_STATEV_SSH);
    }
    else 
    {
        Services_Util_SendIccMessage(SERVICES_SINGLE_PROCESS, STOP,SERVICES_STATEV_SSH);
    }
    if(System_ShmDb_SetSshEnabled(enable) == NOK)
    {
        fprintf(stderr,"%s","SSH status Save Fail!");
        return NOK;
    }
        #ifdef CONFIG_FOR_NETGEAR_CVD31XT
        if( OK != EVMIF_EventSend(EVENT_MGR_CTRL_SOCK, VENDOR_EVM_CHANGE_SSH_STATUS, 1, NULL, NULL)) 
        {
            printf("Send Change Telnet Status Event Fail!");
        }
        #endif     
    }
    return OK;
}


/* #UnihanBU6#, Alisa_Gao, 2008/09/09, get Http Lan Access Type in techsupport page */
/**************************************************************************/
/*! \fn STATUS TECH_get_HttpLanAccessType(webs_t wp, char_t  *name)
 **************************************************************************
 *  \brief get Http Lan Access Type:lan access has three types:off,basic,advanced,the default value is basic.
 *  \param[in] webstream wp,item name in page.
 *  \param[out] no output.
 *  \return NOK if error,else return OK.
 */
int TECH_get_HttpLanAccess(webs_t wp, char_t *name)
{
    unsigned int accesstype=SYS_DFLT_HTTP_LAN_ACCESS_TYPE;/*lan default is basic*/
    if(Common_Utils_GetHttpLanAccessType(&accesstype) == NOK)
    {
        fprintf(stderr,"%s","Http Lan Access Type Get Fail!");
    }
    websWrite(wp,"%d",accesstype);	
    return 0;
}


/* #UnihanBU6#, Alisa_Gao, 2008/09/09, set Http Lan Access Type in techsupport page */
/**************************************************************************/
/*! \fn STATUS TECH_set_HttpLanAccessType(webs_t wp, char_t  *name)
 **************************************************************************
 *  \brief set Http Lan Access Type:wan access type :off,basic,advanced
 *  \param[in] webstream wp,item name in page.
 *  \param[out] no output.
 *  \return NOK if error,else return OK.
 */
STATUS TECH_set_HttpLanAccessType(webs_t wp, char_t *name)
{
    char *lanaccess = websGetVar(wp,"enableHttpLan","");
    unsigned int access=atoi(lanaccess);
    unsigned int old_access = SYS_DFLT_HTTP_LAN_ACCESS_TYPE;
    if(Common_Utils_GetHttpLanAccessType(&old_access) != OK)
    {
    	  fprintf(stderr,"%s","Http Lan Access Type Get FaIl\n");
        return NOK;
    }
    if(old_access != access)
    {
    if(Common_Utils_SetHttpLanAccessType(access,1) == NOK)
    {
        return NOK;
    }
    if(VendorNvm_Common_SaveToNvram() == NOK)
    {
        fprintf(stderr,"%s","Http Lan Access Type Save to NVM Fail!");
        return NOK;
    }
        /* #UnihanBU6#, Alisa_Gao, 2009/04/14,TT1364:GUI must log below events to Event Log Webpage.Add Start*/
        #ifdef CONFIG_FOR_NETGEAR_CVD31XT
        if( OK != EVMIF_EventSend(EVENT_MGR_CTRL_SOCK, VENDOR_EVM_CHANGE_HTTP_LAN_ACCESS_LEVEL, 1, NULL, NULL)) 
        {
            printf("Send Change Telnet Status Event Fail!");
        }
        /* #UnihanBU6,Alisa_Gao,2009/04/14,Add End*/
        #endif     
    }
    return OK;
 }


/* #UnihanBU6#, Alisa_Gao, 2008/09/09, get Http Wan Access Type in techsupport page */
/**************************************************************************/
/*! \fn STATUS TECH_get_HttpWanAccessType(webs_t wp, char_t  *name)
 **************************************************************************
 *  \brief get Http Wan Access Type:wan access has three types:off,basic,advanced,the default type is advanced.
 *  \param[in] webstream wp,item name in page.
 *  \param[out] no output.
 *  \return NOK if error,else return OK.
 */
int TECH_get_HttpWanAccess(webs_t wp, char_t *name)
{
    unsigned int accesstype=SYS_DFLT_HTTP_WAN_ACCESS_TYPE;/*wan default is advanced*/
    if(Common_Utils_GetHttpWanAccessType(&accesstype) == NOK)
    {
        fprintf(stderr,"%s","Http Wan Access Get Fail!");
    }
    websWrite(wp,"%d",accesstype);
    return 0;
}


/* #UnihanBU6#, Alisa_Gao, 2008/09/09, set Http Wan Access Type in techsupport page */
/**************************************************************************/
/*! \fn STATUS TECH_set_HttpWanAccessType(webs_t wp, char_t  *name)
 **************************************************************************
 *  \brief set Http Wan Access Type:wan access type :off,basic,advanced
 *  \param[in] webstream wp,item name in page.
 *  \param[out] no output.
 *  \return NOK if error,else return OK.
 */
STATUS TECH_set_HttpWanAccessType(webs_t wp, char_t *name)
{
    char *wanaccess = websGetVar(wp,"enableHttpWan","");
    unsigned int access=atoi(wanaccess);
    unsigned int old_access = SYS_DFLT_HTTP_WAN_ACCESS_TYPE;
    if(Common_Utils_GetHttpWanAccessType(&old_access) != OK)
    {
         fprintf(stderr,"%s","Http Wan Access Get Fail!");
         return NOK;
    }
    if(old_access != access)
    {
    if(Common_Utils_SetHttpWanAccessType(access,1) == NOK)
    {
        return NOK;
    }
    if(VendorNvm_Common_SaveToNvram() == NOK)
    {
        fprintf(stderr,"%s","Http Wan Access Type Save to NVM Fail!");
        return NOK;
    }
        /* #UnihanBU6#, Alisa_Gao, 2009/04/14,TT1364:GUI must log below events to Event Log Webpage.Add Start*/
        #ifdef CONFIG_FOR_NETGEAR_CVD31XT
        if( OK != EVMIF_EventSend(EVENT_MGR_CTRL_SOCK, VENDOR_EVM_CHANGE_HTTP_WAN_ACCESS_LEVEL, 1, NULL, NULL)) 
        {
            printf("Send Change Telnet Status Event Fail!");
        }
        /* #UnihanBU6,Alisa_Gao,2009/04/14,Add End*/
        #endif
    }
    return OK;
}


/* #UnihanBU6#, Alisa_Gao, 2008/09/09, set password seed in techsupport page */
/**************************************************************************/
/*! \fn STATUS TECH_set_PasswordSeed(webs_t wp, char_t  *name)
 **************************************************************************
 *  \brief set password seed 
 *  \param[in] webstream wp,item name in page.
 *  \param[out] no output.
 *  \return NOK if error,else return OK.
 */
STATUS TECH_set_PasswordSeed(webs_t wp, char_t  *name)
{
    char *seed= websGetVar(wp, "passwordSeed", "");
    char processed_seed[SYS_DFLT_HTTP_PASS_SEED_SIZE];
    char old_pwdseed[SYS_DFLT_HTTP_PASS_SEED_SIZE];
    int isNvramChanged = 0;
    memset(processed_seed,0,sizeof(processed_seed));
    memset(old_pwdseed,0,sizeof(old_pwdseed));
/* #UnihanBU6#, Eating Chen, 2009/12/09, System_ShmDb_RetrieveAccess() function 
 * shall be called once in each process/task 
 */
    if((VendorNvm_Common_RetrieveAccess()!=OK)	||
#if 0
    	(System_ShmDb_RetrieveAccess()!= OK) ||
#endif
	(System_ShmDb_GetHttpPasswordSeed(old_pwdseed) != OK))
   {
    	 printf("Fail to Access DB!\n");
    	 return NOK;
    }
    /* #UnihanBU6#, Alisa_Gao 20090218, EMTA-17,Add for Comcast Security Require,Password Encrytion ,Start*/
    #ifdef CONFIG_FOR_NETGEAR_CVD31XT
    	 if(strlen(seed)<8||strlen(seed)>16)
    	 {
    	 	printf("Seed Length Error(8-16)");
    	 	return NOK;
    	 }
        MD5((const unsigned char *)seed,strlen(seed),(unsigned char *)processed_seed);
    #else
        memcpy(processed_seed, seed, sizeof(processed_seed));
    #endif 
    /* #UnihanBU6#, Alisa_Gao 20090218, EMTA-17,Add End*/
    if(strcmp(old_pwdseed,processed_seed))
    {
	    if(System_ShmDb_SetHttpPasswordSeed(processed_seed,strlen(processed_seed),1) == OK)
	    {
	    	 isNvramChanged = 1;
	        #ifdef CONFIG_FOR_NETGEAR_CVD31XT
	        if( OK != EVMIF_EventSend(EVENT_MGR_CTRL_SOCK, VENDOR_EVM_CHANGE_PASSWORD_SEED, 1, NULL, NULL) ) 
	        {
	        	printf("Send Change Password Event Fail!");
	        }
	        #endif 
	    	}
    	}
    if(isNvramChanged == 1)
    {
	   if(VendorNvm_Common_SaveToNvram() == NOK)
	   {
	       fprintf(stderr,"%s","PasswordSeed Save to NVM Fail!");
	       return NOK;
	   }
    }
    return OK;
}

