/* MODULE NAME: STANDBY_BUTTON_DRIVER_C
 *
 * CREATER: Water Huang         2009/12/3
 *
 * Copyright (C)    ASUSTeK Computer Inc.   2006-2009
 */

/**********************************************************/
/*                                                        */  
/* INCLUDE FILE DECLARATIONS                              */
/*                                                        */
/**********************************************************/
#include <linux/module.h>
#include <linux/types.h>
#include <linux/device.h>	/* for struct device */
#include <linux/kernel.h>
#include <linux/delay.h>
#include <asm/uaccess.h>
#include <asm/io.h>
#include <linux/ioctl.h>
#include <linux/cdev.h>
#include <asm/arch-avalanche/generic/pal.h>
#include <linux/kthread.h>
#include <linux/proc_fs.h> 

/**********************************************************/
/*                                                        */  
/* NAMING CONSTANT DECLARATIONS                           */  
/* {Constants define for LOCAL reference ONLY.}           */  
/*                                                        */  
/**********************************************************/

#define MODE_BUF_SIZE 256

/* for button control */
#define STANDBY_BUTTON_GPIO 42
/* proc entry.
 */
#define STANDBY_BUTTON_DRIVER_PROC_ENTRY "avalanche/gpio42_ctrl"

/**********************************************************/
/*                                                        */
/*  FUNCTION DECLARATIONS                                 */
/*                                                        */
/**********************************************************/
/* high level init function (called by insmod when driver is loaded) */
int STANDBY_BUTTON_DRIVER_init_module(void);

/* high level remove function (called by rmmod when driver is unloaded) */
void STANDBY_BUTTON_DRIVER_exit_module(void);

/* Declare external kernel service */
extern INT32 PAL_sysGpioInBit(UINT32 gpio_pin);
extern INT32 PAL_sysGpioCtrl(UINT32 gpio_pin, PAL_SYS_GPIO_PIN_MODE_T pin_mode, PAL_SYS_GPIO_PIN_DIRECTION_T pin_direction);

/**********************************************************/
/*                                                        */
/* DATA TYPE DECLARATIONS                                 */
/* {DATA TYPE define for LOCAL reference ONLY.}           */
/*                                                        */  
/**********************************************************/

/**********************************************************/
/*                                                        */
/* LOCAL SUBPROGRAM DECLARATIONS                          */
/* {Function routines define for LOCAL reference ONLY.}   */
/*                                                        */
/**********************************************************/

/* local function */
static int STANDBY_BUTTON_DRIVER_ReadEntry(char *page, char **start, off_t off, int count, int *eof, void *data);

/**********************************************************/
/*                                                        */
/* STATIC VARIABLE DECLARATIONS                           */
/* {STATIC VARIABLES defines for LOCAL reference ONLY.}   */
/*                                                        */
/**********************************************************/

/**********************************************************/
/*                                                        */
/* EXPORTED SUBPROGRAM BODIES                             */
/* {C code body of each EXPORTED function routine.}       */
/*                                                        */
/**********************************************************/
/* These are predefined macros that specify different parameters
 * for our driver */ 
MODULE_AUTHOR("Asus Corporation, (C) 2008 - All Rights Reserved");
MODULE_DESCRIPTION("STANDBY_BUTTON Device Driver");
MODULE_SUPPORTED_DEVICE("CVD/E31T");

/* Notifies the kernel that our driver is not GPL. */
MODULE_LICENSE("Proprietary");
//module_param(debug, int, S_IRUGO);

/* tell the kernel the name of our entry points (these macros are defined in linux/module.h */
module_init(STANDBY_BUTTON_DRIVER_init_module);
module_exit(STANDBY_BUTTON_DRIVER_exit_module);/* These are predefined macros that specify different parameters */

/* High level standby button initialization - called at driver load time */
int STANDBY_BUTTON_DRIVER_init_module()
{
    int ret_val;
    struct proc_dir_entry	*gpio_ctrl;
    
    /* Create proc entry which is mapping to GPIO 42.
     */
    gpio_ctrl = create_proc_entry(STANDBY_BUTTON_DRIVER_PROC_ENTRY, S_IFREG | S_IRUGO | S_IWUSR, NULL);
    if( gpio_ctrl ){
    	gpio_ctrl->read_proc = STANDBY_BUTTON_DRIVER_ReadEntry;
    	gpio_ctrl->write_proc = NULL;
    	gpio_ctrl->data = NULL;
    	gpio_ctrl->size = 0;
    }
    else
    {
    	printk(KERN_ERR"%s:%d:Could not create %s proc entry",__FILE__,__LINE__,STANDBY_BUTTON_DRIVER_PROC_ENTRY);	
    }
    
    /* Set GPIO direction as input pin.
     */
    if ((ret_val = PAL_sysGpioCtrl( STANDBY_BUTTON_GPIO, GPIO_PIN, GPIO_INPUT_PIN))<0)
    {
        printk(KERN_ERR "Failed to config the GPIO (%d) \n",STANDBY_BUTTON_GPIO);
        return ret_val;
    }

    return 0;
}

/* called when the system unloads our driver */
void STANDBY_BUTTON_DRIVER_exit_module()
{
    /* do nothing when module is existed.
     */
    return;
}

/**********************************************************/
/*                                                        */
/* LOCAL SUBPROGRAM BODIES                                */
/* {C code body of each LOCAL function routines.}         */
/*                                                        */
/**********************************************************/

static int STANDBY_BUTTON_DRIVER_ReadEntry(char *page, char **start, off_t off, int count, int *eof, void *data)
{
    int len = 0,limit = count - 80;
    int value;

    /* Get RESET_BUTTON_DRIVER_GPIO_NUM_OF_VLYNQ_RXD0 GPIO value.
     */
    value = PAL_sysGpioInBit(STANDBY_BUTTON_GPIO);
    if( len < limit )
    {
        /* Write the value to proc entry(file).
         */
        len += sprintf(page+len,"%d\n",value);
    }
    *eof = 1;
    return len;
}


 /* End of STANDBY_BUTTO_DRIVER_C */
